/*	
**	WINMETA.H
**	
**	CGM (Computer Graphics Metafile) header information for WinGraphics.
*/

#include <fcntl.h>
#define PI 3.141592654
#define S_IWRITE    0000200         /*	write permission, owner */

/****************************************************/
/*	The following codes contain the CLASS+ID section */
/*	add the length section to complete */

#define BEGIN_META	0x0020
#define END_META		0x0040
#define BEGIN_PIC		0x0060
#define BEGIN_PICBODY 0x0080
#define END_PIC		0x00A0

#define VERSION		0x1020
#define DESCRIPTION	0x1040
#define VDC_TYPE		0x1060
#define INT_PREC		0x1080
#define REAL_PREC		0x10A0
#define INDEX_PREC	0x10C0
#define COLOR_PREC	0x10E0
#define COLOR_INDEX_PREC 0x1100
#define MAX_COLOR_INDEX 0x1120
#define COLOR_VAL_EXT 0x1140
#define ELEMENT_LIST	0x1160
#define FONT_LIST		0x11A0

#define SCALING_MODE	0x2020
#define COLOR_MODE	0x2040
#define LINE_MODE		0x2060
#define MARKER_MODE	0x2080
#define EDGE_MODE		0x20A0
#define VDC_EXT		0x20C0

#define VDC_INT_PREC	0x3020
#define TRANSPARENCY	0x3080
#define CLIPPING		0x30C0

#define POLYLINE		0x4020
#define POLYMARKER	0x4060
#define CTEXT			0x4080
#define POLYGON		0x40E0
#define RECTANGLE		0x4160
#define CIRCLE			0x4180

#define LINE_TYPE		0x5040
#define LINE_WIDTH	0x5060
#define LINE_COLOR	0x5080
#define COLOR_TABLE	0x5440
#define MARKER_TYPE	0x50C0
#define MARKER_SIZE	0x50E0
#define MARKER_COLOR	0x5100
#define TEXT_FONT		0x5140
#define TEXT_COLOR	0x51C0
#define TEXT_HEIGHT	0x51E0
#define TEXT_ORIENTATION	0x5200
#define TEXT_ALIGNMENT	0x5240
#define FILL_TYPE		0x52C0
#define FILL_COLOR	0x52E0
#define HATCH_INDEX	0x5300
#define EDGE_TYPE		0x5360
#define EDGE_WIDTH	0x5380
#define EDGE_COLOR	0x53A0
#define EDGE_VISIBLE	0x53C0

/*********************************************************/
/* CGM VALUES															*/

/*	Font Index */
#define HELV			1
#define HELV_B			2
#define HELV_I			3
#define SYM				4
#define TR				5
#define TR_B			6
#define TR_I			7
#define TR_BI			8

/* Text Alignment */
#define LEFTALIGN		1
#define CENTERALIGN	2
#define RIGHTALIGN	3
#define TOPALIGN		2
#define MIDALIGN		3
#define BOTTOMALIGN	4

/*	All Colors */
#define MAXCOLOR		100
#define NUMCOLORS		12
#define BLACK			1

/*	Line/Edge Type */
#define SOLID			1
#define DASH			0xFFFF
#define DOT				3
#define LONGDASH		2
#define DASHDOTDOT	5
#define DASHDOT		4

/* Line/Edge Thickness */
#define WIDTH1			1
#define WIDTH2			30
#define WIDTH3			80
#define WIDTH4			175
#define WIDTH5			275

/*	Fill Type */
#define FILLED			1
#define HATCHED		3
#define HOLLOW			4

/*	Marker Type */
#define MSQUARE		0xFFFF
#define MTRIANGLE		5
#define MCIRCLE		4
#define MSTAR			3
#define MSQUAREFILL	0xFFF8
#define MCIRCLEFILL	0xFFF9

#define ON				1
#define OFF				0

/*********************************************************/
/*	CGM GLOBALS 														*/

#define IBUFLEN		2000

int VDCxl=-30000, VDCxh = 30000, VDCyl=-21000, VDCyh=21000, *ibuf;
char buf[IBUFLEN];
int CGMCurrentx, CGMCurrenty;
int TextAngle(double), CGMx(int), CGMy(int), CGMd(int);
int PLine(int, int *, int *), PMarker(int, int *, int *);
int MoveTo(int, int), LineTo(int, int);
int _CGMFile;

/*********************************************************/
/*	CGM MACROS															*/

#define LOBYTE(i) (i & 0xFF)
#define HIBYTE(i) ((i >> 8) & 0xFF)
#define SWAP(i)	((LOBYTE(i)<<8) + HIBYTE(i))

#define BEGINPICBODY	MetaWrite(BEGIN_PICBODY, 0, buf);
#define ENDPICTURE	MetaWrite(END_PIC, 0, buf);
#define ENDMETA		MetaWrite(END_META, 0, buf); close(_CGMFile);

#define LINETYPE(i)	ibuf[0]=SWAP(i); MetaWrite(LINE_TYPE, 2, buf);
#define LINECOLOR(i)	ibuf[0]=SWAP(i); MetaWrite(LINE_COLOR, 2, buf);
#define LINEWIDTH(i)	ibuf[0]=SWAP(i); MetaWrite(LINE_WIDTH, 2, buf);

#define EDGETYPE(i)	ibuf[0]=SWAP(i); MetaWrite(EDGE_TYPE, 2, buf);
#define EDGECOLOR(i)	ibuf[0]=SWAP(i); MetaWrite(EDGE_COLOR, 2, buf);
#define EDGEWIDTH(i)	ibuf[0]=SWAP(i); MetaWrite(EDGE_WIDTH, 2, buf);
#define EDGE(i)		ibuf[0]=SWAP(i); MetaWrite(EDGE_VISIBLE, 2, buf);

#define FILLCOLOR(i)	ibuf[0]=SWAP(i); MetaWrite(FILL_COLOR, 2, buf);
#define FILLTYPE(i)	ibuf[0]=SWAP(i); MetaWrite(FILL_TYPE, 2, buf);
#define HATCH(i)		ibuf[0]=SWAP(i); MetaWrite(HATCH_PATTERN, 2, buf);

#define TEXTFONT(i)	ibuf[0]=SWAP(i); MetaWrite(TEXT_FONT, 2, buf);
#define TEXTCOLOR(i)	ibuf[0]=SWAP(i); MetaWrite(TEXT_COLOR, 2, buf);
#define TEXTHEIGHT(i)	ibuf[0]=SWAP(i); MetaWrite(TEXT_HEIGHT, 2, buf);

#define MARKERTYPE(i)	ibuf[0]=SWAP(i); MetaWrite(MARKER_TYPE, 2, buf);
#define MARKERSIZE(i)	ibuf[0]=SWAP(i); MetaWrite(MARKER_SIZE, 2, buf);
#define MARKERCOLOR(i)	ibuf[0]=SWAP(i); MetaWrite(MARKER_COLOR, 2, buf);

/*********************************************************/
/* CGM FUNCTIONS														*/

CGMx(x)
int x;
{
	return((int)(((double)VDCxh-VDCxl-2000)/SCREENWIDTH*x + VDCxl+1000));
}

CGMy(y)
int y;
{
	return((int)((VDCyh -1000 - ((double)VDCyh-VDCyl-2000)/SCREENHEIGHT*y)));
}

CGMdy(d)
int d;
{
	return((int)(((double)VDCyh-VDCyl-2000)/SCREENHEIGHT*d));
}

CGMdx(d)
int d;
{
	return((int)(((double)VDCxh-VDCxl-2000)/SCREENWIDTH*d));
}

PLine(n, x, y)
int n, *x, *y;
{
	int i, j;

	if (n*2 > IBUFLEN) n = IBUFLEN/2;
	for (i=j=0; i<n; i++)
	{
		ibuf[j++] = SWAP(x[i]);
		ibuf[j++] = SWAP(y[i]);
	}
	MetaWrite(POLYLINE, 2*j, buf);
}

PMarker(n, x, y)
int n, *x, *y;
{
	int i, j;

	if (n*2 > IBUFLEN) n = IBUFLEN/2;
	for (i=j=0; i<n; i++)
	{
		ibuf[j++] = SWAP(x[i]);
		ibuf[j++] = SWAP(y[i]);
	}
	MetaWrite(POLYMARKER, 2*j, buf);
}

MoveTo(x, y)
int x, y;
{
	CGMCurrentx = x;
	CGMCurrenty = y;
}

LineTo(x, y)
int x, y;
{
	ibuf[0] = SWAP(CGMCurrentx);
	ibuf[1] = SWAP(CGMCurrenty);
	ibuf[2] = SWAP(x);
	ibuf[3] = SWAP(y);
	MetaWrite(POLYLINE, 8, buf);
	CGMCurrentx = x;
	CGMCurrenty = y;
}

Rectangle(x1, y1, x2, y2)
int x1, y1, x2, y2;
{
	ibuf[0] = SWAP(x1);
	ibuf[1] = SWAP(y1);
	ibuf[2] = SWAP(x2);
	ibuf[3] = SWAP(y2);
	MetaWrite(RECTANGLE, 8, buf);
}

/*	
**	Circle(x, y, r)
**	int x, y, r;
**	{
**		ibuf[0] = SWAP(x);
**		ibuf[1] = SWAP(y);
**		ibuf[2] = SWAP(r);
**		MetaWrite(CIRCLE, 6, buf);
**	}
*/
/*	
**	Polygon(n, x, y)
**	int n, *x, *y;
**	{
**		int i, j;
**	
**		for (i=j=0; i<n; i++)
**		{
**			ibuf[j++] = SWAP(x[i]);
**			ibuf[j++] = SWAP(y[i]);
**		}
**		MetaWrite(POLYGON, 2*j, buf);
**	}
*/
TextAngle(theta)
double theta;
{
	int Upx, Upy, Basex, Basey;

	theta *= PI/180;
	Upx = -1000*sin(theta);
	Upy = 1000*cos(theta);
	Basex = 1000*cos(theta);
	Basey = 1000*sin(theta);
	ibuf[0] = SWAP(Upx);
	ibuf[1] = SWAP(Upy);
	ibuf[2] = SWAP(Basex);
	ibuf[3] = SWAP(Basey);
	MetaWrite(TEXT_ORIENTATION, 8, buf);
}

TextAlign(horz, vert)
int horz, vert;
{
	ibuf[0] = SWAP(horz);
	ibuf[1] = SWAP(vert);
	ibuf[2] = ibuf[3] = ibuf[4] = ibuf[5] = 0;
	MetaWrite(TEXT_ALIGNMENT, 12, buf);
}

CGMText(x, y, text)							/*	Writes text string */
int x, y;
char *text;
{
	int i, j;

	i = 0;
	j = 0;
	ibuf[j++] = SWAP(x);
	ibuf[j++] = SWAP(y);
	ibuf[j++] = SWAP(1);
	j *= 2;
	buf[j++] = strlen(text);
	while (buf[j++] = text[i++]);
	j--;
	MetaWrite(CTEXT, j, buf);
}

MetaWrite(code, length, data)				/* Performs binary write */
unsigned code;
int length;
char *data;
{
	unsigned ulen;

	if (length/2 < (length+1)/2)
	{
		data[length] = (char)0;
		length++;
	}
	if (length < 31)
	{
		code+=length;
		code = SWAP(code);
		write(_CGMFile, &code, 2);
	}
	else
	{
		code += 31;
		code = SWAP(code);
		write(_CGMFile, &code, 2);
		ulen = length;
		ulen = SWAP(ulen);
		write(_CGMFile, &ulen, 2);
	}
	if (length)	write(_CGMFile, data, length);
}

CGMPrologue(filename)
char *filename;
{
	int j;
	char *ptr;

	ibuf = (int *)buf;

	_CGMFile = open(filename, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IWRITE);

	ptr = filename;
	MetaWrite(BEGIN_META, strlen(ptr), ptr);

	ibuf[0] = SWAP(1);
	MetaWrite(VERSION, 2, buf);

	ptr = "CGM:JohnV Rev. 1.0";
	MetaWrite(DESCRIPTION, strlen(ptr), ptr);

	ibuf[0] = SWAP(16);
	MetaWrite(COLOR_PREC, 2, buf);
	MetaWrite(COLOR_INDEX_PREC, 2, buf);
	MetaWrite(MAX_COLOR_INDEX, 2, buf);

	for (j=0; j<3; j++) ibuf[j] = SWAP(0);
	for (;j<6; j++)
	{
		ibuf[j] = SWAP(MAXCOLOR);
	}
	MetaWrite(COLOR_VAL_EXT, 2*j, buf);

	ptr = "Picture 1";
	MetaWrite(BEGIN_PIC, strlen(ptr), ptr);

	ibuf[0] = SWAP(0);
	MetaWrite(COLOR_MODE, 2, buf);
	MetaWrite(LINE_MODE, 2, buf);
	MetaWrite(MARKER_MODE, 2, buf);
	MetaWrite(EDGE_MODE, 2, buf);

	for (j=0; j<3; j++)
	{
		ibuf[j] = SWAP(0);
	}
	MetaWrite(SCALING_MODE, 2*j, buf);

	j=0;
	ibuf[j++] = SWAP(VDCxl);
	ibuf[j++] = SWAP(VDCyl);
	ibuf[j++] = SWAP(VDCxh);
	ibuf[j++] = SWAP(VDCyh);
	MetaWrite(VDC_EXT, 2*j, buf);

	ibuf[0] = SWAP(ON);
	MetaWrite(TRANSPARENCY, 2, buf);

	j = 0;
	ibuf[j++] = SWAP(1);
	ibuf[j++] = SWAP(0);
	ibuf[j++] = SWAP(0);
	ibuf[j++] = SWAP(0);
	ibuf[j++] = SWAP(9);
	ibuf[j++] = SWAP(9);
	ibuf[j++] = SWAP(9);
	ibuf[j++] = SWAP(0x12);
	ibuf[j++] = SWAP(0x12);
	ibuf[j++] = SWAP(0x12);
	ibuf[j++] = SWAP(0x1B);
	ibuf[j++] = SWAP(0x1B);
	ibuf[j++] = SWAP(0x1B);
	ibuf[j++] = SWAP(0x24);
	ibuf[j++] = SWAP(0x24);
	ibuf[j++] = SWAP(0x24);
	ibuf[j++] = SWAP(0x2D);
	ibuf[j++] = SWAP(0x2D);
	ibuf[j++] = SWAP(0x2D);
	ibuf[j++] = SWAP(0x36);
	ibuf[j++] = SWAP(0x36);
	ibuf[j++] = SWAP(0x36);
	ibuf[j++] = SWAP(0x40);
	ibuf[j++] = SWAP(0x40);
	ibuf[j++] = SWAP(0x40);
	ibuf[j++] = SWAP(0x49);
	ibuf[j++] = SWAP(0x49);
	ibuf[j++] = SWAP(0x49);
	ibuf[j++] = SWAP(0x52);
	ibuf[j++] = SWAP(0x52);
	ibuf[j++] = SWAP(0x52);
	ibuf[j++] = SWAP(0x5B);
	ibuf[j++] = SWAP(0x5B);
	ibuf[j++] = SWAP(0x5B);
	ibuf[j++] = SWAP(0x64);
	ibuf[j++] = SWAP(0x64);
	ibuf[j++] = SWAP(0x64);
	MetaWrite(COLOR_TABLE, 2*j, buf);

	BEGINPICBODY;
}
