DGROUP		GROUP	_DATA

_TEXT		SEGMENT word public 'CODE'
		ASSUME cs:_TEXT,ds:DGROUP

;
;SET8X8FONT
;
;This procedure calls ROM BIOS interrupt 10h function 11h to set up the
;8x8 graphics character font for EGA/VGA graphics modes 15-18.
;
;C calling conventions:
;
;	void SET8X8FONT();
;

		PUBLIC _SET8X8FONT
_SET8X8FONT	PROC	far

		push	ds
		push	es

		mov	ah,11h		;character generator function
		mov	al,23h		;ROM BIOS 8x8 char table
		mov	bl,3		;43 lines per screen
		mov	dl,43		;redundant with above

		int	10h

		xor	ax,ax
		mov	es,ax		;ES=0
		mov	cx,es:[485h]	;Height = scan lines per char)
		mov	Height,cx
		mov	bx,43h*4	;DS:BX -> int 43h vector
		les	di,es:[bx]	;ES:DI -> start of character table
		mov	RomOffset,di	;RomSegment:RomOffset ->
		mov	RomSegment,es	;base of 8x8 font table

		pop	es
		pop	ds
		ret
_SET8X8FONT	ENDP


;
;TEXT43
;
;This procedure writes a string to the video memory in EGA and VGA 
;graphics modes as an 8x8 block, resulting in an 80x43 character
;display space.  The character matrix can be written in any location
;(to the pixel level), and with any orientation.
;
;C-calling conventions and Parameters:
;
;	int TEXT43(str, x, y, color, page)
;	char *str; 	/* character code */
;	int x, y; 	/* pixel coordinate */
;	int color; 	/* foreground text colors */
;	int page; 	/* video page */
;

ARGstr		EQU	word ptr [bp+6]	;stack frame addresses
ARGx		EQU	word ptr [bp+8]
ARGy		EQU	word ptr [bp+10]
ARGcolor	EQU	byte ptr [bp+12]
ARGpage		EQU	word ptr [bp+14]

VARshift	EQU	[bp-2]
CharCount	EQU	word ptr [bp-4]
Points		EQU	[bp-6]
Table		EQU	[bp-8]

DScaller	EQU	[bp-14]

BytesPerLine	=	80
RMWbits		=	0h		;Read-Modify-Write bits

		PUBLIC	_Text43
_Text43		PROC	far

		push	bp		;preserve caller regs
		mov	bp,sp
		sub	sp,8		;local stack space
		push	di
		push	si
		push	ds		;at address BP-14
		push	es
		mov	CharCount,0	;set up char count

;Calculate first pixel address.
		mov	ax,ARGy		;AX=y
		mov	bx,ARGx		;BX=x
		mov	dx,ARGpage	;DX=page
		call	PixelAddr10	;returns ES:BX -> vbuffer
					;CL=#bits to shift left to mask pixel
		inc	cx
		and	cl,7		;CL=#bits to shif to mask char
		mov	ch,0FFh
		shl	ch,cl		;ch=bit mask for right side of char
		mov	VARshift,cx

		push	es		;preserve video segment
		mov	si,bx		;SI=video buffer offset

;Set up graphics controller registers.
		mov	dx,3CEh		;Graphics Contr addr reg port
		mov	ax,0A05h	;AL=mode reg number
					;AH=Write mode 2, Read Mode 1
		out	dx,ax
		mov	ah,RMWbits	;AH=Read-Modify-Write bits
		mov	al,3		;AL=data rotate function select reg
		out	dx,ax
		mov	ax,0007		;AH=color don't care bits
					;AL=Color don't care reg num
		out	dx,ax		;don't care for all bit planes

;Set up character definition table addressing.
;		xor	ax,ax
;		mov	ds,ax		;DS=0
;		mov	cx,ds:[485h]	;Points=POINTS (scan lines per char)
;		mov	Points,cx
;		mov	bx,43h*4	;DS:BX -> int 43h vector
;		les	di,ds:[bx]	;ES:DI -> start of character table
;		mov	Table,di	;ES:Table -> base of table always

;Set up character definition table addressing.
		mov	cx,Height
		mov	Points,cx
		mov	es,RomSegment
		mov	di,RomOffset
		mov	Table,di

LNextChar:	mov	ax,DScaller	;AX=DS of caller
		mov	ds,ax		;set up DS for string reference
		mov	bx,ARGstr	;BX=string offset
		inc	ARGstr		;prepare for next char
		xor	ax,ax		;zero AH
		mov	al,ds:[bx]	;AL=character
		cmp	al,0		;if char=0
		je	Lexit43		;exit
		inc	CharCount	;inc character counter
		mul	cl		;AX=offset into table (POINTS*code)
		add	di,ax		;DI=address of char def
		pop	ds		;DS:SI -> Vbuffer

;Select output routine depending on whether character is byte aligned.
		mov	bl,ARGcolor	;BL=foreground pixel value
		cmp	byte ptr VARshift,0 ;test #bits to shift
		jne	L20		;jump if not byte aligned

;Routine for byte aligned characters.
		mov	al,8		;AL=bit mask reg number
L10:		mov	ah,es:[di]	;AH=bit pattern for next row of pixels
		out	dx,ax		;update Bit Mask Reg
		and	[si],bl		;update foreground pixels
		inc	di		;ES:DI -> next byte in char table
		add	si,BytesPerLine	;increment to next line in vbuffer
		loop	L10
		push	ds		;save DS
		mov	cx,Points	;CX=Points (rows per char)
		mov	ax,BytesPerLine	;AX=BytesPerLine
		mul	cl		;AX=BytesPerLine*Points
		sub	si,ax		;DS:SI->Vbuffer, last char position
		inc	si		;DS:SI->Vbuffer, next char pos
		mov	di,Table	;ES:DI -> base of char table
		jmp	short LNextChar	;check next character

;Routine for non-byte aligned characters
L20:		push	cx		;preserve loop counter
		mov	cx,VARshift	;CH=mask for left side of char
					;CL=#bits to shift left

;Left side of character.
		mov	al,es:[di]	;AL=bits for next row of pixels
		xor	ah,ah
		shl	ax,cl		;AH=bits for left side of char
					;AL=bits for right side of char
		push	ax		;save bits for right side on stack
		mov	al,8		;AL=bit mask register number
		out	dx,ax		;set bit mask for foreground pixels
		and	[si],bl		;update foreground pixels

;Right side of character.
		pop	ax
		mov	ah,al		;AH=bits for right side of character
		mov	al,8
		out	dx,ax		;set bit mask
		inc	si		;ds:si -> right side of char in buffer
		and	[si],bl		;update foreground pixels

;Increment to next row of pixels in character.
		inc	di		;ES:DI -> next byte in char def table
		dec	si
		add	si,BytesPerLine	;DS:SI -> next line in Vbuffer
		pop	cx
		loop	L20
		push	ds		;save DS
		mov	cx,Points	;CX=Points (rows per char)
		mov	ax,BytesPerLine	;AX=BytesPerLine
		mul	cl		;AX=BytesPerLine*Points
		sub	si,ax		;DS:SI->Vbuffer, last char position
		inc	si		;DS:SI->Vbuffer, next char pos
		mov	di,Table	;ES:DI -> base of char table
		jmp	short LNextChar	;check next character

;Restore default Graphics Controller Registers
Lexit43:	pop	ds		;pop off extra ds
		mov	ax,0FF08h	;default bit mask
		out	dx,ax
		mov	ax,0005		;default mode register
		out	dx,ax
		mov	ax,0003		;default Data Rotate/Function Select
		out	dx,ax
		mov	ax,0F07h	;default color don't care
		out	dx,ax

		mov	ax,CharCount	;return count of characters output

		pop	es
		pop	ds		;restore caller registers
		pop	si
		pop	di
		mov	sp,bp
		pop	bp
		ret

_Text43		ENDP



;
;PIXADDR
;
;Determines buffer address of pixels in EGA and VGA modes 13-18
;
;Caller:	AX=y-coordinate
;		BX=x-coordinate
;		DX=video page
;
;Returns:	AH=bit mask
;		BX=byte offset in buffer
;		CL=number of bits to shift left
;		ES=video buffer segment
;

BytesPerLine	EQU	80		;bytes per horizontal line
PageOffset	EQU	8000h		;byte offset of page (mode 16)
VideoBufferSeg	EQU	0A000h


		PUBLIC	PixelAddr10
PixelAddr10	PROC 	near

		mov	cl,bl		;CL=low order byte of x
		push	dx		;preserve dx
		mov	dx,BytesPerLine	;AX=y*bytes per line
		mul	dx
		pop	dx
		shr	bx,1
		shr	bx,1
		shr	bx,1		;BX = x/8
		add	bx,ax		;BX=y*BytesPerLine + x/8
		cmp	dx,0		;if DX=0
		je	PAGE0		;then page=0, else page=1
PAGE1:		add	bx,PageOffset	;BX=byte offset in video buffer
PAGE0:		mov	ax,VideoBufferSeg
		mov	es,ax		;ES:BX=byte address of pixel
		and	cl,7		;CL=x&7
		xor	cl,7		;CL=number of bits to shift left
		mov	ah,1		;AH=unshifted bit mask
		ret

PixelAddr10	ENDP

;EGALINE.ASM
;
;This procedure draws a horizontal/vertical line in EGA/VGA graphics
;modes, using XOR to update the buffer.  No clipping is performed.
;
;Calling conventions:
;
;	void EGALINE(x1,y1,x2,y2,color,page);
;	int x1,y1,x2,y2;	/* end points */
;	int color;		/* color to XOR */
;	int page;		/* video page */
;

ARGx1		EQU	word ptr [bp+6]	;stack frame addressing
ARGy1		EQU	word ptr [bp+8]
ARGx2		EQU	word ptr [bp+10]
ARGy2		EQU	word ptr [bp+12]
ARGcolor	EQU	byte ptr [bp+14]
ARGpage		EQU	word ptr [bp+16]

VARroutine	EQU	word ptr [bp-2]	;local variables

ByteOffsetShift	=	3		;converts pixels to byte offset
BytesPerLine	=	80
RMWbits		=	18h		;value for Data Rotate/Function reg

		PUBLIC	_EGALINE
_EGALINE	PROC	far

		push	bp		;preserve caller regs
		mov	bp,sp
		sub	sp,2		;local stack space
		push	si
		push	di

;configure the Graphics Controller
		mov	dx,3CEh		;Graphics Controller port addr
		mov	ah,ARGcolor	;AH = pixel value
		xor	al,al		;AL = Set/Reset Register #
		out	dx,ax
		mov	ax,0F01h	;AH = 1111b (bit plane mask for 
					;Enable Set/Reset)
		out	dx,ax		;AL = Enable Set/Reset Reg #
		mov	ah,RMWbits	;bits 3 and 4 of AH = function
		mov	al,3		;AL = Data Rotate/Func Select reg #
		out	dx,ax

;check for vertical line
		mov	si,BytesPerLine	;increment for video buffer
		mov	cx,ARGx2
		sub	cx,ARGx1	;CX = x2-x1
		jz	VertLine	;jump if vertical line

;force x2 > x1
		jns	HorzLine	;jump if x2 > x1
		neg	cx		;CX = x1 - x2
		mov	bx,ARGx2	;exchange x1 and x2
		xchg	bx,ARGx1
		mov	ARGx2,bx
		mov	bx,ARGy2	;exchange y1 and y2
		xchg	bx,ARGy1
		mov	ARGy2,bx

;Horizontal Line routine
HorzLine:	push	ds		;preserve DS
		mov	ax,ARGy1
		mov	bx,ARGx1
		mov	dx,ARGpage
		call	PixelAddr10	;AH = bit mask
					;ES:BX -> vbuffer
					;CL = # bits to shift left
		mov	di,bx		;ES:DI -> buffer
		mov	dh,ah		;DH = unshifted bit mask for left byte
		not	dh
		shl	dh,cl		;DH = reverse bit mask for 1st byte
		not	dh		;DH = bit mask for first byte
		mov	cx,ARGx2
		and	cl,7
		xor	cl,7		;CL = # bits to shift left
		mov	dl,0FFh		;unshifted bit mask for rightmost byte
		shl	dl,cl		;DL = bit mask for rightmost byte

;Determine byte offset of first and last pixel in line
		mov	ax,ARGx2	;AX = x2
		mov	bx,ARGx1	;BX = x1
		mov	cl,ByteOffsetShift;number of bits to shift to 
					;convert pixels to bytes
		shr	ax,cl		;AX = byte offset of x2
		shr	bx,cl		;BX = byte offset of x1
		mov	cx,ax
		sub	cx,bx		;CX = (# bytes in line) - 1

;Get Graphics Controller port address into DX
		mov	bx,dx		;BH = bit mask for 1st byte
					;BL = bit mask for last byte
		mov	dx,3CEh		;DX = Graphics Controller Port
		mov	al,8		;AL = Bit Mask Reg #

;make video buffer addressable thru DS:SI
		push	es
		pop	ds
		mov	si,di		;DS:SI -> vbuffer

;Set pixels in leftmost byte of the line
		or	bh,bh
		js	L43		;jump if byte-aligned
		or	cx,cx
		jnz	L42		;jump if more than 1 byte in line
		and	bl,bh		;BL = bit mask for the line
		jmp	short L44

L42:		mov	ah,bh		;AH = bit mask for 1st byte
		out	dx,ax		;update Graphics Controller
		movsb			;update bit planes
		dec	cx

;use a fast 8086 machine instruction to draw the remainder of the line
L43:		mov	ah,11111111b	;AH = bit mask
		out	dx,ax		;update the Bit Mask Reg
		rep	movsb		;update all pixels in the line

;Set pixels in the rightmost byte
L44:		mov	ah,bl		;AH = bit mask for last byte
		out	dx,ax		;Update Graphics Controller
		movsb			;update bit planes

		pop	ds		;restore DS
		jmp	short LexitEGA

;Vertical Line routine
VertLine:	mov	ax,ARGy1	;AX = y1
		mov	bx,ARGy2	;BX = y2
		mov	cx,bx
		sub	cx,ax		;CX = dy
		jge	L31		;jump if dy >= 0
		neg	cx		;force dy >= 0
		mov	ax,bx		;AX = y2
L31:		inc	cx		;CX = # of pixels to draw
		mov	bx,ARGx1	;BX = x
		push	cx
		push	dx
		mov	dx,ARGpage
		call	PixelAddr10	;AH = bit mask
					;ES:BX -> vbuffer
					;CL = # bits to shift left
		pop	dx

;Set up Graphics Controller
		shl	ah,cl		;AH = bit mask in proper position
		mov	al,8		;AL = Bit Mask Reg #
		out	dx,ax
		pop	cx		;CX = # pixels to draw

;Draw the line
L32:		or	es:[bx],al	;set pixel
		add	bx,si		;increment to next line
		loop	L32
		jmp	LexitEGA

;Restore default Graphics Controller state and return
LexitEGA:		xor	ax,ax		;AL = AH = 0
		out	dx,ax		;Restore Set/Reset Reg
		inc	ax		;AL = 1, AH = 0
		out	dx,ax		;restore Enable Set/Reset Reg
		mov	al,3		;AL = 3, AH = 0
		out	dx,ax		;restore Data Rotate/Function Select 
		mov	ax,0FF08h	;AH = 11111111b, AL = 8
		out	dx,ax		;restore Bit Mask Reg

		pop	di		;restore registers
		pop	si
		mov	sp,bp
		pop	bp
		ret

_EGALINE	ENDP

;
;PSTAT.ASM
;
;Checks the status of the printer using ROM BIOS INT 17h.
;
;C calling conventions:
;
;	int PRINTERSTATUS();
;
;Return Value: an integer whose low byte consists
;of the following bit fields.
;
;	bit	1 indicates
;	0	Time Out
;	1,2	unused
;	3	I/O error
;	4	Printer Selected
;	5	Out of Paper
;	6	Acknowledgement from printer
;	7	Printer idle
;

PrinterNumber	=	0		;Printer number for INT 17h

		PUBLIC	_PRINTERSTATUS
_PRINTERSTATUS	PROC 	far

		mov	ah,2		;AH = 2 (check printer status)
		mov	dx,PrinterNumber;which printer to check
		int	17h		;returns status in AH
		mov	al,ah		;save it in AL
		xor	ah,ah		;AH = 0
		ret
_PRINTERSTATUS	ENDP


;
;BINARYPRN
;
;This procedure calls DOS INT 21h funtion 44h to change the standard
;print device to binary mode.  This enables sending binary data including
;the ctrl-z value (26) which is normally filtered by the operating system.
;
		PUBLIC _BINARYPRN
_BINARYPRN	PROC	far

		mov	ax,4400h	;ah=function 44h, al=subfunction 0
		mov	bx,4		;bx=4 (stdprn handle)
		int	21h		;get device data into dx
		or	dx,0020h	;set bit 5 (raw mode)
		and	dx,00ffh	;clear reserved bits
		mov	ax,4401h	;ah=function 44h, al=subfunction 1
		mov	bx,4		;bx=4 (stdprn)
		int	21h		;set device data for PRN
		ret
_BINARYPRN	ENDP


_TEXT		ENDS


_DATA		SEGMENT word public 'DATA'

RomSegment	DW	?		;address of 8x8 font
RomOffset	DW	?
Height		DW	?		;scan lines in font

_DATA		ENDS

		END
