#include "es1.h"

#define Zero_Order   1
#define First_Order  2
#define Energy_Conserving 3

accel(ilp, iup, q, m, t, p, ke) 
int ilp, iup; 
float q, m, t, *p, *ke; 
{
   int il,iu,j,ng1,i;
   float dxdt,ae,tem,v1s,v2s,vo,vn,s,aa,vyy,vxx;

	
	il = ilp;
	iu = iup;
	dxdt = dx/dt;
	ae = (q/m)*(dt/dxdt);
	if (t != 0.0) ae *= 0.5;
	if (ae != ael) {
		ng1 = ng + 1;
		tem = ae/ael;
		for (j=1;j<=ng1;j++) acc[j] = e[j]*ae;
		ael = ae;
	}
	switch (iw) {
		case Zero_Order :
			v1s = v2s = 0.0;
			for (i=il;i<=iu;i++) {
				j = x[i] + 0.5;
				vo = vx[i];
				vn = vo + acc[j+1];
				v1s += vn;
				v2s += fabs(vn*vo);
				vx[i] = vn;
			}
			*p = m*v1s*dxdt;
			*ke = 0.5*m*v2s*dxdt*dxdt;
			break;
		case First_Order :
			if (t != 0.0) {
				s = 2.0*t/(1.0 + t*t);
				v2s = 0.0;
				for (i=il;i<=iu;i++) {			
					j = x[i];
					aa = acc[j+1] + (x[i] - j)*(acc[j+2] - acc[j+1]);
					vyy = vy[i];
					vxx = vx[i] - t*vyy + aa;
					vyy += s*vxx;
					vxx -= t*vyy;
					v2s += vxx*vxx + vyy*vyy;
					vx[i] = vxx + aa;
					vy[i] = vyy;
				}
				*ke = 0.5*m*v2s*dxdt*dxdt;
			}
			else {
				v1s = v2s = 0.0;
				for (i=il;i<=iu;i++) {			
					j = x[i];
					vo = vx[i];
					vn = vo + acc[j+1] + (x[i] - j) * (acc[j+2] - acc[j+1]);
					v1s += vn;
					v2s += fabs(vo*vn);
					vx[i] = vn;
				}
				*p = m*v1s*dxdt;
				*ke = 0.5*m*v2s*dxdt*dxdt;
			}  
			break;
		case Energy_Conserving:
			v1s = v2s = 0.;
			for (i=il;i<=iu;i++) {
				j = x[i];
				vo = vx[i];
				vn = vo + acc[j+1];
				v1s += vn;
				v2s += fabs(vn*vo);
				vx[i] = vn;
			}
			*p = m*v1s*dxdt;
			*ke = 0.5*m*v2s*dxdt*dxdt;
			break;			
		default:
			printf(" Bad iw switch in accel. \n");
			exit(-1);
			break;
	}
}
