c  One-dimensional electrostatic plasma simulation code.
c  Written by A. Bruce Langdon, Livermore, 1972.
c  Major revisions 6/1976, 2/77, 10/77, 6/78, 3-5/79, 2/85, 10/89, 10/90.
c  Last revised 1/17/91.
c
c
      include 'param.h'
c
c    ngmax = maximum number of cells.
c    nxv   = maximum number of particles.
c    nvy   = first nvy particles may be magnetized (>0, <=nxv).
c            can set nvy=1 if unmagnetized.
c    nspm  = maximum number of species.
c    nth   = number of time steps between history plots.
c    mmax  = maximum number of different fourier modes to plot.
c
      include 'field.h'
c
      include 'ptcl.h'
c
      include 'cntrl.h'
c
      include 'hist.h'
c
      include 'cline.h'
c
      real ke, ms(nspm), qs(nspm), ts(nspm)
      integer ins(nspm1)
c
      data rho0 /0./
      data it,time,ith,ithl/0,0.,0,0/
      data ins(1) /1/
c
c  Input variables:
c  l      = physical length of system.
c  nsp    = number of particle species.
c  dt     = time step.
c  nt     = number of time steps to run (ending time=nt*dt).
c  ng     = number of spacial cells. *must* be power of 2.
c  iw     = mover algorithm selector. see subroutines accel and move.
c  epsi   = multiplier in poisson equation. =1 for rationalized units.
c  a1,a2  = field smoothing and mid-range boost. see fields.
c  e0,w0  = add uniform electric field e0*cos(w0*time).
c  irho   = plotting interval for rho (charge density). =0 for no plot.
c  irhos  = plotting interval for smoothed density.
c  iphi   = plotting interval for phi (potential).
c  ie     = plotting interval for e (electric field).
c  ixvx   = plotting interval for x vs. vx phase space.
c  ifvx   = plotting interval for f(vx) distribution function.
c            >0 gives linear, <0 gives semi-log.
c  ivxvy  = plotting interval for vx vs. vy phase space.
c  mplot  = fourier mode numbers to plot.
c
c  Default input parameters:
      data nsp,l,dt,nt,epsi /1,6.28318530717958,.2,150,1./
      data ng,iw,a1,a2 /32,2,0.,0./
      data e0,w0 /0.,0./
      data irho,irhos,iphi,ie,ixvx,ivxvy,ifvx/7*0/, mplot/mmax*0/
      namelist /in/ l, nsp, nt, dt, epsi, ng, iw, a1, a2, e0, w0,
     .  irho, irhos, iphi, ie, mplot, ixvx, ivxvy, ifvx
c
      call first
c
c  Read namelist input. Echo all values to output, including defaults.
      read(STDIN,in)
      write(STDOUT,in)
      write(textline,901) nsp,l,dt,nt
  901 format(' nsp =',i2,', l =',e12.5,', dt =',e12.5,', nt =',i6)
      call crtbcd(textline)
      write(textline,902) ng,iw,epsi,a1,a2
  902 format(' ng =',i5,', iw =',i1,', epsi =',e12.5,
     .   ',a1 =',e12.5,', a2 =',e12.5)
      call crtbcd(textline)
      write(textline,903) irho,irhos,iphi,ie
  903 format(' irho =',i6,', irhos =',i6,', iphi =',i6,' ie =',i6)
      call crtbcd(textline)
      write(textline,904) ixvx,ivxvy,ifvx
  904 format(' ixvx =',i6,' ivxvy =',i6,' ifvx =',i6)
      call crtbcd(textline)
      write(textline,905) mplot
  905 format(' mplot =',10i6)
      call crtbcd(textline)
c
c  Check ng and nsp to see if out of range.
      if( ng.gt.ngmax .or. nsp.gt.nspm ) then
       write(STDOUT,998)
  998  format('input out of range')
       call last
      endif
c
      call histry
      dx = l/ng
      do 10 is = 1,nsp
   10  call init( ins(is),ins(is+1),ms(is),qs(is),ts(is),nms(is) )
      call fields(0)
      do 11 is = 1,nsp
   11  call setv( ins(is),ins(is+1)-1,qs(is),ms(is),ts(is),pxs(0,is) )
c
c  Begin time step loop.
c  Particle x is at time 0, vx and vy at -.5*dt.
  100 continue
      vl = 0.
      vu = 0.
c  Phase space plot of all species.
      call plotxv(1,ins(nsp+1)-1,vl,vu)
c  Distribution function for species 1.
      call pltfvx(1,ins(2)-1,vl,vu,qs(1),32)
      vmu = 0.
      if( ts(1).ne.0. ) call pltvxy(1,ins(2)-1,vmu)
c     (if make restart file, here is where to save particles.)
      p = 0.
      ke = 0.
      do 101 is = 1,nsp
c  Advance velocities from it-.5 to it+.5.
       call accel( ins(is),ins(is+1)-1,qs(is),ms(is),ts(is),
     .    pxs(ith+1,is),kes(ith,is) )
       p = p + pxs(ith+1,is)
  101  ke = ke + kes(ith,is)
      do 102 is = 1,nsp
c  Advance positions from it to it+1.
  102  call move( ins(is),ins(is+1)-1,qs(is) )
      te = ke + ese(ith)
      write(STDOUT,950) time,ese(ith),p,(kes(ith,is),is=1,2),te
  950 format(' time,ese,p,ke1,ke2,te',f8.2,5e17.9)
      if( it.ge.nt ) go to 103
      if( ith.eq.nth ) call histry
      it = it + 1
      time = it*dt
      ith = it - ithl
c  Get fields at time step it.
      call fields(ith)
      go to 100
c
c  End of run.
c  Particle x is at time nt*dt, vx and vy at (nt-.5)*dt.
  103 continue

      call histry
      call last

      end

      subroutine init(il1,il2,m,q,t,nm)
c  Loads particles for one species at a time.
      real m, nm
      include 'param.h'
      real lg
      include 'ptcl.h'
      include 'cline.h'
      data twopi/6.2831 85307 17958/
c
c  Input variables:
c  n      = number of particles (for this species).
c  wp     = plasma frequency.
c  wc     = cyclotron frequency.
c  qm     = q/m charge:mass ratio.
c  vt1    = rms thermal velocity for random velocities.
c  vt2    = rms thermal velocity for ordered velocities.
c  nlg    = number of loading groups (sharing same ordered velocities).
c  nv2    = multiply Maxwellian (for ordered velocities) by v**nv2.
c  v0     = drift velocity.
c  mode, x1, v1, thetax and thetav are for loading a sinusoidal perturbation.
c  Velocity contributions thru vt1, vt2, v0 and v1 are additive.
c  User changes velocity distribution; normalization not required.
c
c  Default input parameters:
      data n,wp,wc,qm,vt1,vt2,nlg,nv2,v0,mode,x1,v1,thetax,thetav
     .  /128,1.,0.,-1.,2*0.,1,0,0.,1,4*0./
      namelist /in/ n, wp, wc, qm, vt1, vt2, nv2, nlg, v0,
     .  mode, x1, v1, thetax, thetav
      read(STDIN,in)
      write(STDOUT,in)
      write(textline,901) n,wp,wc,qm
  901 format(' n =',i7,', wp =',e12.5,', wc =',e12.5,', qm =',e12.5)
      call crtbcd(textline)
      write(textline,902) vt1,vt2,v0,nv2,nlg
  902 format(' vt1 =',e12.5,', vt2 =',e12.5,', v0 = ',e12.5,
     .  ', nv2 =',i3,', nlg =',i5)
      call crtbcd(textline)
      write(textline,909) mode,x1,v1
  909 format(' mode =',i5,', x1 =',e12.5,', v1 =',e12.5)
      call crtbcd(textline)
      write(textline,910) thetax,thetav
  910 format(14x,'thetax =',e11.4,', thetav =',e11.4)
      call crtbcd(textline)

      if( n+il1-1.gt.nxv .or. (n+il1-1.gt.nvy .and. wc.ne.0.) ) then
   33  write(STDOUT,998)
  998  format('input out of range')
       call last
      endif
c
      ngr = n/nlg
      n = ngr*nlg
      il2 = il1 + n
      q = l*wp*wp/(epsi*n*qm)
      m = q/qm
      nm = n*m
      t = tan(-wc*dt/2.)
c
c  Load first of nlg groups of particles.
      lg = l/nlg
      ddx = l/n
c  Load evenly-spaced, with drift.  Also does cold case.
      do 10 i = 1,ngr
       i1 = i - 1 + il1
       x0 = (i - .5)*ddx
       x (i1) = x0
   10  vx(i1) = v0
      if(vt2.eq.0.) go to 50
c  Load ordered velocities in vx ("quiet start", or at least subdued).
c  Is set up for Maxwellian*v**nv2, but can do any smooth distribution.
c  Hereafter, ngr is preferably a power of 2.
c  First store indefinite integral of distribution function in x array.
c  Use midpoint rule -simple and quite accurate.
      vmax = 5.*vt2
      dv = 2.*vmax/(n - 1)
      vvnv2 = 1.
      x(il1) = 0.
      do 21 i = 2,n
       vv = ((i - 1.5)*dv - vmax)/vt2
       if(nv2.ne.0) vvnv2 = vv**nv2
       fv = vvnv2*exp( -.5*vv**2 )
       i1 = i - 1 + il1
   21  x(i1) = x(i1-1) + amax1(fv,0.)
c  For evenly spaced (half-integer multiples) values of the integral,
c  find corresponding velocity by inverse linear interpolation.
      df = x(i1)/ngr
      i1 = il1
      j = il1
      do 22 i = 1,ngr
       fv = (i - .5)*df
   23  if(fv.ge.x(j+1)) then
        j = j + 1
        if(j.gt.il2-2) then
         write(STDOUT,98)
   98    format('distribution function error')
         call last
        endif
        go to 23
       endif
       vv = dv*(j - il1 + (fv - x(j))/(x(j+1) - x(j))) - vmax
       vx(i1) = vx(i1) + vv
   22  i1 = i1 + 1
c  For ordered velocities, scramble positions to reduce correlations.
c  Scrambling is done by bit-reversed counter -compare sorter in cpft.
c  xs=.000,.100,.010,.110,.001,.101,.011,.111,.0001.. (binary fractions).
      xs = 0.
      do 41 i = 1,ngr
       i1 = i - 1 + il1
       x(i1) = xs*lg + .5*ddx
       xsi = 1.
   42   xsi = .5*xsi
        xs = xs - xsi
       if(xs.ge.0) go to 42
   41  xs = xs + 2.*xsi
      i1 = ngr + il1 - 1
      if( ngr.le.256 ) write(STDOUT,99) (i, x(i), vx(i), i=il1,i1)
   99 format(i6,2f10.4)
c
c  If magnetized, rotate (vx,0) into (vx,vy).
   50 if(wc.ne.0.) then
       do 51 i = 1,ngr
        i1 = i - 1 + il1
        vv = vx(i1)
        theta = twopi*x(i1)/lg
        vx(i1) = vv*cos(theta)
   51   vy(i1) = vv*sin(theta)
      endif
c
c  Copy first group into rest of groups.
      if(nlg.ne.1) then
       j = ngr + 1
       xs = 0.
       do 81 i = j,n,ngr
        xs = xs + lg
        do 81 j = 1,ngr
         i1 = j - 1 + il1
         i2 = i1 + i - 1
         x (i2) = x (i1) + xs
         vx(i2) = vx(i1)
   81    if(wc.ne.0.) vy(i2) = vy(i1)
      endif
c
c  Add random maxwellian.
      if(vt1.ne.0.) then
       do 86 i = 1,n
        i1 = i - 1 + il1
        if(wc.ne.0.)
     .  vy(i1) = vy(i1) + vt1*rm()
   86   vx(i1) = vx(i1) + vt1*rm()
      endif
c
c  Add perturbation.
c  Loading x(t=0), v(t=0), remember so no dt/2 correction.
c  May want to perturb vy too.
      do 91 i = 1,n
       i1 = i - 1 + il1
       theta = twopi*mode*x(i1)/l
       x (i1) = x (i1) + x1*cos(theta + thetax)
   91  vx(i1) = vx(i1) + v1*sin(theta + thetav)
c
c  Apply boundary conditions, collect charge density, etc.
      call setrho(il1,il2-1,q,n*q/l)

      return
      end

      subroutine histry
c  Plot energies etc. vs. time.
      include 'param.h'
      include 'cntrl.h'
      include 'hist.h'
      include 'cline.h'
      real tim(0:nth), pxsl(nspm)
c
      if( it.eq.0 ) then
c  At t=0, just zero arrays.
       mth = nth
       do 11 is = 1,nsp
        pxs(0,is) = 0.
        do 11 i = 0,mth
         kes(i,is) = 0.
   11    pxs(i+1,is) = 0.
       do 12 i = 0,mth
   12   ese(i) = 0.
       do 13 km = 1,mmax
       do 13 i = 0,mth
   13   esem(i,km) = 0.
       return
      endif
c
      tl = ithl*dt
      mth = it - ithl
      do 1 i = 0,mth
    1  tim(i) = i*dt + tl
c
c  Plot mode energies.
      do 50 km = 1,mmax
       k = mplot(km)
       if( k.eq.0 ) go to 52
       call plthst(esem(0,km),tim,mth,tl,time,1)
       write(textline,51) k, tl, time
   50  call crtbcd(textline)
   51 format('mode',i3,' energy, time =',f10.4,' to',f10.4)
   52 continue
c
c  Plot field energy.
      call plthst(ese,tim,mth,tl,time,1)
      write(textline,100) tl, time
  100 format('field energy, time =',f10.4,' to',f10.4)
      call crtbcd(textline)
c
c  Plot kinetic energies.
      do 200 is = 1,nsp
       call plthst(kes(0,is),tim,mth,tl,time,0)
       write(textline,201) is, tl, time
  200  call crtbcd(textline)
  201 format('kinetic energy',i2,', time =',f10.4,' to',f10.4)
c
c  Plot directed (drift) energies.
      do 300 is = 1,nsp
       pxsl(is) = pxs(mth,is)
       do 299 i = 0,mth
  299   pxs(i,is) = pxs(i,is)*pxs(i+1,is)/(2.*nms(is))
       call plthst(pxs(0,is),tim,mth,tl,time,0)
       write(textline,301) is, tl, time
  300  call crtbcd(textline)
  301 format('drift energy',i2,', time =',f10.4,' to',f10.4)
c
c  Plot thermal energies
      do 400 is = 1,nsp
       do 399 i = 0,mth
  399   pxs(i,is) = kes(i,is) - pxs(i,is)
       call plthst(pxs(0,is),tim,mth,tl,time,1)
       write(textline,401) is, tl, time
  400  call crtbcd(textline)
  401 format('thermal energy',i2,', time =',f10.4,' to',f10.4)
c
c  Plot total energy.
      do 499 is = 1,nsp
      do 499 i = 0,mth
  499  ese(i) = ese(i) + kes(i,is)
      call plthst(ese,tim,mth,tl,time,0)
      write(textline,500) tl, time
  500 format('total energy, time =',f10.4,' to',f10.4)
      call crtbcd(textline)
c
      ithl = it
c  Last values now are first values for next time interval.
      ese(0) = ese(mth)
      do 2 is = 1,nsp
       kes(0,is) = kes(mth,is)
       ese(0) = ese(0) - kes(0,is)
       pxs(0,is) = pxsl(is)
       pxs(1,is) = pxs(mth+1,is)
       do 2 i = 1,mth
        kes(i,is) = 0.
    2   pxs(i+1,is) = 0.
      do 3 i = 1,mth
    3  ese(i) = 0.
      do 4 km = 1,mmax
       esem(0,km) = esem(mth,km)
       do 4 i = 1,mth
    4   esem(i,km) = 0.
c
      return
      end

      subroutine fields(ith)
c  Solves for phi and e, computes field energy, etc.
      include 'param.h'
      include 'field.h'
      include 'hist.h'
      include 'cntrl.h'
      real rhok(0:1), phik(0:1), scrach(0:1)
      equivalence (rho,rhok), (phi,phik), (e,scrach)
      parameter(ng2m=ngmax/2)
      real kdx2, ksqi(ng2m), li, sm(ng2m)
      integer cpft
c
c  First time step duties.
      data ng2/0/
      if( ng2.eq.0 ) then
       ng2 = ng/2
c  Set up ratio phik/rhok.
c  a2>0 gives a short-wavelength cutoff (smoothing).
c  a1>0 gives a mid-range boost to compensate for slight attenuation
c    in force calculation.
       data pi/3.1415 92653 58979/
       do 1 k = 1,ng2-1
        kdx2 = (pi/ng)*k
        sm(k) = exp(a1*sin(kdx2)**2 - a2*tan(kdx2)**4)
    1   ksqi(k) = +epsi/((2.0*sin(kdx2)/dx)**2)*sm(k)**2
       sm(ng2) = exp(a1)
       if(a2.ne.0) sm(ng2) = 0.
       ksqi(ng2) = +epsi/((2.0/dx)**2)*sm(ng2)**2
      endif
c
c  Transform charge density.
      rho(0) = rho(0) + rho(ng)
      rho(ng) = rho(0)
      call plotf(rho,'charge density',irho)
      hdx = 0.5*dx
      do 10 j = 0,ng-1
       rhok(j) = rho(j)*hdx
   10  scrach(j) = 0.
      ier = cpft (rhok,scrach,ng,1,1)
      call rpft2(rhok,scrach,ng,1)
      rhok(0) = 0.
c
c  Calculate phik and field energy.
      eses = 0.
      phik(0) = 0.
      do 20 k = 1,ng2-1
       kk = ng - k
       phik(k ) = ksqi(k)*rhok(k )
       phik(kk) = ksqi(k)*rhok(kk)
   20  eses = eses + rhok(k)*phik(k) + rhok(kk)*phik(kk)
      phik(ng2) = ksqi(ng2)*rhok(ng2)
      ese(ith) = (2.0*eses + rhok(ng2)*phik(ng2))/(2.0*l)
c
c  Save specified mode energies.
      li = 1.0/l
      do 21 km = 1,mmax
       k = mplot(km)
       if( k.eq.0 ) go to 22
       kk = ng - k
       esem(ith,km) = (rhok(k)*phik(k) + rhok(kk)*phik(kk))*li
   21  if( k.eq.kk ) esem(ith,km) = 0.25*esem(ith,km)
   22 continue
c
      do 23 k = 1,ng2-1
       kk = ng - k
       rhok(k ) = sm(k)*rhok(k )
   23  rhok(kk) = sm(k)*rhok(kk)
      rhok(ng2) = sm(ng2)*rhok(ng2)
c
c  Inverse transform phi and smoothed rho.
      do 30 k = 0,ng-1
       rho(k) = rhok(k)*li
   30  phi(k) = phik(k)*li
      call rpfti2(phi,rho,ng,1)
      ier = cpft (phi,rho,ng,1,-1)
      phi(ng) = phi(0)
      rho(ng) = rho(0)
      call plotf(rho,'smoothed density',irhos)
      call plotf(phi,'electric potential',iphi)
c
c  Uniform field.
      e0t = e0*cos(w0*time)
c
c  Select electric field differencing.
      go to (100,100,200), iw
c
c  Centered difference across 2 cells.
  100 continue
      hdxi = 0.5/dx
      do 101 j = 1,ng-1
  101  e(j) = (phi( j-1) - phi(j+1))*hdxi + e0t
       e(0) = (phi(ng-1) - phi(  1))*hdxi + e0t
       e(ng) = e(0)
      go to 40
c
c  Centered difference across 1 cell.
  200 continue
      dxi = 1.0/dx
      do 201 j = 0,ng-1
  201  e(j) = (phi(j) - phi(j+1))*dxi + e0t
       e(ng) = e(0)
c
   40 continue
      call plotf(e,'electric field',ie)
c  Clear out old charge density.
      do 41 j = 1,ng
   41  rho(j) = rho0
       rho(0) = 0.
c
c  Electric field has not been renormalized yet.
      ael = 1.
      return
      end

      subroutine accel(ilp,iup,q,m,t,p,ke)
c  Advances velocity one time step, computes momentum and kinetic energy
      real ke, m
      include 'param.h'
      include 'field.h'
      include 'ptcl.h'
      real a(0:ngmax)
      equivalence (a,e)
      il = ilp
      iu = iup
c
c  Renormalize acceleration if need be.
      dxdt = dx/dt
      ae = (q/m)*(dt/dxdt)
      if( t.ne.0. ) ae = 0.5*ae
      if( ae.ne.ael ) then
       tem = ae/ael
       do 1 j = 0,ng
    1   a(j) = a(j)*tem
       ael = ae
      endif
c
c  Select acceleration weighting.
      go to (100,200,300), iw
c
c  NGP, grid points at j*dx.
  100 continue
      v1s = 0.
      v2s = 0.
      do 101 i = il,iu
       j = x(i) + 0.5
       vo = vx(i)
       vn = vo + a(j)
       v1s = v1s + vn
       v2s = v2s + vn*vo
  101  vx(i) = vn
      p = p + m*v1s*dxdt
      ke = ke + 0.5*m*v2s*dxdt*dxdt
      return
c
c  Linear, momentum conserving.
  200 continue
      if( t.ne.0. ) go to 250
      v1s = 0.
      v2s = 0.
      do 201 i = il,iu
       j = x(i)
       vo = vx(i)
       vn = vo + a(j) + (x(i) - j)*(a(j+1) - a(j))
       v1s = v1s + vn
       v2s = v2s + vo*vn
  201  vx(i) = vn
      p = p + m*v1s*dxdt
      ke = ke + 0.5*m*v2s*dxdt*dxdt
      return
c
c  Linear, energy conserving.
  300 continue
      v1s = 0.
      v2s = 0.
      do 301 i = il,iu
       j = x(i)
       vo = vx(i)
       vn = vo + a(j)
       v1s = v1s + vn
       v2s = v2s + vn*vo
  301  vx(i) = vn
      p = p + m*v1s*dxdt
      ke = ke + 0.5*m*v2s*dxdt*dxdt
      return
c
c  Linear, momentum conserving, uniformly magnetized.
  250 continue
      s = 2.0*t/(1.0 + t*t)
      v2s = 0.
      do 251 i = il,iu
       j = x(i)
       aa = a(j) + (x(i) - j)*(a(j+1) - a(j))
       vyy = vy(i)
       vxx = vx(i) - t*vyy + aa
       vyy = vyy + s*vxx
       vxx = vxx - t*vyy
       v2s = v2s + vxx*vxx + vyy*vyy
       vx(i) = vxx + aa
  251  vy(i) = vyy
      ke = ke + 0.5*m*v2s*dxdt*dxdt
      return
c
      end

      subroutine move(ilp,iup,q)
c  Advances position one time step and accumulates charge density.
      include 'param.h'
      include 'field.h'
      include 'ptcl.h'
      il = ilp
      iu = iup
      qdx = q/dx
      xn = ng
      go to (100,200,200), iw
c
c  NGP
  100 continue
      do 101 i = il,iu
       x(i) = x(i) + vx(i)
       if( x(i).lt.0. ) x(i) = x(i) + xn
       if( x(i).ge.xn ) x(i) = x(i) - xn
       j = x(i) + 0.5
  101  rho(j) = rho(j) + qdx
      return
c
c  Linear
  200 continue
      do 201 i = il,iu
       x(i) = x(i) + vx(i)
       if( x(i).lt.0. ) x(i) = x(i) + xn
       if( x(i).ge.xn ) x(i) = x(i) - xn
       j = x(i)
       drho = qdx*(x(i) - j)
       rho(j  ) = rho(j  ) - drho + qdx
  201  rho(j+1) = rho(j+1) + drho
      return
c
      end

      subroutine setrho(il,iu,q,rhos)
c  Converts position to computer normalization and accumulates charge
c  density.
      include 'param.h'
      include 'field.h'
      include 'ptcl.h'
c
      qdx = q/dx
c  If is first group of particles, then clear out rho.
      if( il.eq.1 ) then
       do 1 j = 1,ng
    1   rho(j) = rho0
        rho(0) = 0.
       dxi = 1.0/dx
       xn = ng
      endif
c  Add on fixed neutralizing charge density
c  (not needed when all species are mobile -but harmless.)
      rho0 = rho0 - rhos
      do 3 j = 1,ng
    3  rho(j) = rho(j) - rhos
c
      go to (100,200,200), iw
c
c  NGP
  100 continue
      do 101 i = il,iu
       x(i) = x(i)*dxi
       if( x(i).lt.0. ) x(i) = x(i) + xn
       if( x(i).ge.xn ) x(i) = x(i) - xn
       j = x(i) + 0.5
  101  rho(j) = rho(j) + qdx
      return
c
c  Linear
  200 continue
      do 201 i = il,iu
       x(i) = x(i)*dxi
       if( x(i).lt.0. ) x(i) = x(i) + xn
       if( x(i).ge.xn ) x(i) = x(i) - xn
       j = x(i)
       drho = qdx*(x(i) - j)
       rho(j  ) = rho(j  ) - drho + qdx
  201  rho(j+1) = rho(j+1) + drho
      return
      end

      subroutine setv(il,iu,q,m,t,p)
c  Converts particle velocities at t=0 to computer normalization at
c  t=-dt/2.
      real m
      include 'param.h'
      include 'field.h'
      include 'ptcl.h'
      dtdx = dt/dx
c  Rotate v thru angle +0.5*wc*dt and normalize vy.
c  If t=0, there is no magnetic field. the rotation is omitted
c  and no references are made to vy.
      if( t.ne.0. ) then
       c = 1.0/sqrt(1.0 + t*t)
       s = c*t
       do 1 i = il,iu
        vxx = vx(i)
        vx(i)= c*vxx + s*vy(i)
        vy(i) = -s*vxx + c*vy(i)
    1   vy(i) = vy(i)*dtdx
      endif
c  Normalize vx.
      do 3 i = il,iu
    3  vx(i) = vx(i)*dtdx
c  Electric impulse to go back 1/2 time step.
      data dum/0./
      call accel(il,iu,-0.5*q,m,0.,p,dum)
      return
      end

      subroutine plotf(f,label,intrvl)
      character*(*) label
c  Plot field at certain times.
      include 'param.h'
      include 'field.h'
      include 'cntrl.h'
      include 'cline.h'
      real xj(0:ngmax), f(0:ngmax)
      data xj(1)/0./
      if( intrvl.le.0 ) return
      if( (it/intrvl)*intrvl.ne.it ) return
c
      if( xj(1).ne.dx ) then
       do 1 j = 0,ng
    1   xj(j) = j*dx
      endif
c
      call myframe
      call cartmm(ng,rmin,rmax,f,1)
      call mapg(0.,l,rmin,rmax)
      call trace(xj,f,ng+1)
      call setch(10.,1.,1,0,1,0)
      write(textline,3) label, time
    3 format(a,' at time =',f10.4)
      call crtbcd(textline)
      return
      end

      subroutine plotxv(il,iu,vl,vu)
c  Plot x-vx phase space at certain times.
      include 'ptcl.h'
      include 'param.h'
      include 'cntrl.h'
      include 'cline.h'
c  These arrays are used to group plot points for efficiency.
      common /scratch/ ji(64), xs(0:63), vs(0:63)
      if( ixvx.le.0 ) return
      if( (it/ixvx)*ixvx.ne.it ) return
      dxdt = dx/dt
c
c  Set velocity range etc. if need be.
      if( vl.ge.vu ) then
       vl = vx(il)
       vu = vl
       do 1 i = il,iu
        vl = amin1(vl,vx(i))
    1   vu = amax1(vu,vx(i))
       vl = vl*dxdt
       vu = vu*dxdt
      endif
c
      call myframe
      call maps(0.,l,vl,vu)
      call setpch(1,0,0,0,1)
c  If fewer than 2000 particles, plot pluses for best visibility.
      if( iu-il.le.2000 ) then
       do 3 i = il,iu
    3   call pointc(1h+, (x(i)-.5*vx(i))*dx, vx(i)*dxdt, 1)
      else
c  If more than 13000 particles, plot every 3rd, or 5th...
       int = 2*((iu - il)/13000) + 1
       do 32 i0 = il,iu,64*int
        iu64 = min0(63, (iu - i0)/int)
        do 31 i = 0,iu64
         xs(i) = (x(i0+int*i) - .5*vx(i0+int*i))*dx
   31    vs(i) = vx(i0+int*i)*dxdt
   32   call points(xs, vs, iu64+1)
c      do 32 i = il,iu,int
c  32   call point((x(i) - .5*vx(i))*dx, vx(i)*dxdt, 1)
      endif
c
c  Label plot.
      call setch(10.,1.,1,0,1,0)
      tim = time - 0.5*dt
      write(textline,4) tim
    4 format('vx vs. x, time =',f10.4)
      call crtbcd(textline)
      return
      end

      subroutine pltvxy(il,iu,vmu)
c  Plot vx-vy phase space at certain times.
      include 'ptcl.h'
      include 'param.h'
      include 'cntrl.h'
      include 'cline.h'
c  These arrays are used to group plot points for efficiency.
      common /scratch/ ji(64), xs(0:63), ys(0:63)
      if( ivxvy.le.0 ) return
      if( (it/ivxvy)*ivxvy.ne.it ) return
      dxdt = dx/dt
c
c  Set velocity range etc. if need be.
      if( vmu.eq.0. ) then
       vl = vx(il)
       vu = vl
       do 1 i = il,iu
        vl = amin1(vl,vx(i),vy(i))
    1   vu = amax1(vu,vx(i),vy(i))
       vmu = amax1(abs(vl),abs(vu))*dxdt
      endif
c
      call myframe
      call maps(-vmu,vmu,-vmu,vmu)
      call setpch(1,0,0,0,1)
c  If fewer than 2000 particles, plot pluses for best visibility.
      if( iu - il.lt.2000 ) then
       do 3 i = il,iu
    3   call pointc(1h+, vx(i)*dxdt,vy(i)*dxdt, 1)
      else
c  If more than 13000 particles, plot every 3rd, or 5th...
       int = 2*((iu - il)/13000) + 1
       do 32 i0 = il,iu,64*int
        iu64 = min0(63, (iu - i0)/int)
        do 31 i = 0,iu64
         xs(i) = vx(i0+int*i)*dxdt
   31    ys(i) = vy(i0+int*i)*dxdt
   32   call points(xs, ys, iu64+1)
c      do 32 i = il,iu,int
c  32   call point(vx(i)*dxdt, vy(i)*dxdt, 1)
      endif
c
c  Label plot.
      call setch(10.,1.,1,0,1,0)
      tim = time - 0.5*dt
      write(textline,4) tim
    4 format('vy vs. vx, time  = ',f10.4)
      call crtbcd(textline)
      return
      end

      subroutine pltfvx(il,iu,vl,vu,q,nbinsp)
c  Plot distribution function f(vx) at selected times.
c  If plot interval ifvx is negative, plot is semilog.
      include 'ptcl.h'
      include 'param.h'
      include 'cntrl.h'
      include 'cline.h'
      parameter(maxbins=50)
      common /scratch/ vbin(0:maxbins), fbin(0:maxbins)
c  nbins+1 must not exceed dimension of these arrays.
      if( ifvx.eq.0 ) return
      if( (it/ifvx)*ifvx.ne.it ) return
      nbins = amin0(nbinsp,maxbins) - 1
      dxdt = dx/dt
c
c  Set velocity range etc. if need be.
      if( vl.ge.vu ) then
       vl = vx(il)
       vu = vl
       do 1 i = il,iu
        vl = amin1(vl,vx(i))
    1   vu = amax1(vu,vx(i))
       vl = vl*dxdt
       vu = vu*dxdt
      endif
      if( vl.eq.vu .or. nbins.le.0 ) return
      dvi = nbins/(vu - vl)
      dq = abs(q)*dvi
c
c  Assign vx's to bins with linear weighting.
      do 3 i = 0,nbins
       fbin(i) = 0.
    3  vbin(i) = i/dvi + vl
      do 4 i = il,iu
       tem = (vx(i)*dxdt - vl)*dvi
       j = tem
       tem = (tem - j)*dq
       if( tem.ge.0. .and. j.le.nbins ) then
c  Get all contributions to fbin(0), fbin(nbins) the same as if
c  the bins were extended to encompass all particles.
c**not done in book**
        if(j.ge.0)       fbin(j  ) = fbin(j  ) - tem + dq
        if(j+1.le.nbins) fbin(j+1) = fbin(j+1) + tem
       endif
    4 continue
c
      call myframe
      call cartmm(nbins+1,rmin,rmax,fbin,1)
      if( ifvx.gt.0 ) then
       call maps(vl,vu,0.,rmax)
      else
       rmin = amax1(rmin,1.e-5*rmax)
       call mapssl(vl,vu,rmin,rmax)
      endif
      call trace(vbin,fbin,nbins+1)
      call setch(10.,1.,1,0,1,0)
      tim = time - 0.5*dt
      write(textline,7) tim
    7 format('f(vx), time =',f10.4)
      call crtbcd(textline)
      return
      end

      subroutine plthst(rec,tim,mth,tl,tu,linlog)
c  Plot time history, linear or log.
      real rec(0:mth), tim(0:mth)
      call myframe
      call cartmm(mth+1,rl,ru,rec,1)
      if( linlog.eq.1 .and. rl.ge.0. ) then
       rl = amax1(rl,1.e-5*ru)
       call mapgsl(tl,tu,rl,ru)
      else
       call mapg(tl,tu,rl,ru)
      endif
      call trace(tim,rec,mth+1)
      call setch(10.,1.,1,0,1,0)
      return
      end

      function rm()
      rm = 0.
      do 1 i = 1,12
    1 rm = rm + ranf()
      rm = rm - 6.
      return
      end

      subroutine first
c  Connect input file, create text and plot output files.
c  Use a problem name to facilitate identification of files when
c  several problems are run by the same user.
c  This routine is not particular to ES1 and can be used for other codes.
c  A. B. Langdon, LLL, early 1979. Last revised 10/90.

c  Under Unicos, it is at present necessary to have the input file in the
c  directory where the problem is to run.
c
c  param.h is needed only for STDIN, STDOUT
      include 'param.h'
      include 'cline.h'
      character *8 ifile, ofile, dfile, runame
      character*32 boxid
      data dfile, ifile, ofile /'+','i','o'/
c  Get problem name from execute line. Prompt if none there.
      if( izgetsr(runame,1,1,dum,0).ne.0 ) then
c     No execute line message.
       call izsndmr('Problem name: ',14,1)
c     Wait for message.
       ier = izgetsr(runame,1,0,dum,0)
      endif
c  If problem name is "ee", input file must be "iee", drop file will
c  "+ee", and print file will be "oee".
      ifile = 'i' // runame(1:7)
      dfile = 'd' // runame(1:7)
      open(STDIN,file=ifile,status='old')
      ofile = 'o' // runame(1:7)
c  Open output file, but first destroy existing file of same name, if any.
      open(STDOUT,file=ofile,err=1)
       close(STDOUT,status='delete')
    1  continue
      open(STDOUT,file=ofile,status='new')
c  First line in input file should be:
c  'box lnn remark...'.
      read(STDIN,995) textline
  995 format(a)
c  Identification on top of plot fiche will be 'box lnn runame' and
c  first 14 characters of 'remark'.
      boxid = textline(1:8) // runame // textline(9:24)
      call setbox(boxid,32)
c     Only 30 characters appear on top of fiche (using TV80LIB).
c  Initialize plotting.
      call gfsize(3,250000)
      call fr80id(tim,1,1)
      call keep80(1,3)
      call dders(-1)
      call setch(1.,42.,1,0,1,0)
      write(STDOUT,996) runame
  996 format(' 1d Plasma Simulation ',/' Problem name ',a//
     .  ' Input file:')
      call crtbcd('1d Plasma Simulation')
      call crtbcd('Problem name ' // runame)
c  Copy entire input file into output file and first plot frame.
  990  write(STDOUT,997) textline
  997  format(1x,a)
       call crtbcd(textline)
       read(STDIN,fmt=995,iostat=ios) textline
c     Check for end-of-file.
       if(ios.eq.0) go to 990
  991 rewind STDIN
      read(STDIN,995) textline
      call myframe
      call setch(1.,42.,1,0,1,0)
      call crtbcd(textline)
      return
c
      end

      subroutine myframe
c  Advance to next plot frame. Place frame count in lower right, tiny numbers.
      character*5 anum
      data nframe/0/
      nframe = nframe + 1
      call frame(1)
      call setch(124.,1.,1,0,0,0)
      write(anum,'(i5)') nframe
      call crtbcd(anum)
      return
      end

      subroutine last
c  Called at end of run.
c  Closes output files, terminates code and does a couple of other things.
      include 'param.h'
      include 'cline.h'
      character line(10)*8
      call gtabls(line,3,2)
      write(*,10) line(3)
      write(STDOUT, 10) line(3)
      call frame
      call setch(1.,1.,1,0,3,0)
      write(textline,10) line(3)
   10 format('Plot file name is ',a8)
      call crtbcd(textline)
      call plote
      stop

      end
