      subroutine rpft2(a,b,n,incp)
      real a(0:1), b(0:1)
c  Real data, periodic, Fourier transform, two at a time.
c
c  Interface to complex periodic Fourier transform, to do pairs of
c  transforms of real sequences.
c
c  The two sequences are elements 0,inc,2*inc...(n-1)*inc of arrays a,b.
c
c  After a complex periodic Fourier transform, with a and b as the
c  real and imaginary parts, rpft2 separates the transforms of a and b
c  and packs them, times 2, back into arrays a and b.
c  Thus, the contents of a and b are replaced by twice their transforms
c  by the calls:
c    cpft (a, b, n, inc, sign)
c    rpft2(a, b, n, inc)
c  Twice the real parts of the first half of the complex Fourier
c  coefficients of a (cosine coef.) are in a(0), a(1)...a(n/2), if
c  inc=1.  Twice the imaginary parts (sine coef.) are stored in
c  reverse order, in a(n-1), a(n-2)...a(n/2+1).  Likewise for b.
c
c  No parameter 'sign' is provided for the purpose of changing the sign
c  of the sine coefficients.  This may be done with parameter 'sign' of
c  the Fourier transform, cpft.
c
c  Time required is less than 1/10 of that for cpft.
c
c  Should be re-coded in assembly language.
c
c  Written by A. Bruce Langdon, LRL Livermore, May 1971.
c  Revised Feb. 1985.
c
      real ip, im
      inc = incp
      ninc = n*inc
      a(0) = a(0) + a(0)
      b(0) = b(0) + b(0)
      lp = inc
      lm = ninc - lp
      if( lp.ge.lm ) go to 2
1     rp = a(lp)
      rm = a(lm)
      ip = b(lp)
      im = b(lm)
      a(lp) = rm + rp
      b(lm) = rm - rp
      b(lp) = ip + im
      a(lm) = ip - im
      lp = lp + inc
      lm = ninc - lp
      if( lp.lt.lm ) go to 1
2     if( lp.gt.ninc ) return
      a(lp) = a(lp) + a(lp)
      b(lp) = b(lp) + b(lp)
      return
      end

      subroutine rpfti2(a,b,n,incp)
      real a(0:1), b(0:1)
c  Real data, periodic, Fourier transform inverse, two at a time.
c
c  Interface to complex periodic Fourier transform, to do pairs of
c  transforms of real sequences.
c
c  Unpacks the cosine and sine coefficients of a and b and combines
c  them so that a + i b is the complex periodic Fourier transform of
c  the original sequences.  rpfti2 reverses the effect of rpft2, except
c  that a and b are doubled.
c  The calls
c    rpfti2(a, b, n, inc)
c    cpft  (a, b, n, inc, -sign)
c  invert the transform done earlier, except that the arrays have been
c  multiplied by 2*n.
c
c  Should be re-coded in assembly language.
c
c  Written by A. Bruce Langdon, LRL Livermore, May 1971.
c  Revised Feb. 1985.
c
      inc = incp
      ninc = n*inc
      lp = inc
      lm = ninc - lp
      if( lp.ge.lm ) return
3     ca = a(lp)
      sb = b(lm)
      cb = b(lp)
      sa = a(lm)
      a(lp) = ca - sb
      a(lm) = ca + sb
      b(lp) = cb + sa
      b(lm) = cb - sa
      lp = lp + inc
      lm = ninc - lp
      if( lp.lt.lm ) go to 3
      return
      end
