#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <graph.h>
#include <string.h>
#include <ctype.h>

#define IsZerothSet(x)		(((x)&  1) ==  1)
#define IsFirstSet(x)		(((x)&  2) ==  2)
#define IsSecondSet(x)		(((x)&  4) ==  4)
#define IsThirdSet(x)		(((x)&  8) ==  8)
#define IsFourthSet(x)		(((x)& 16) == 16)
#define IsFifthSet(x)		(((x)& 32) == 32)
#define IsSixthSet(x)		(((x)& 64) == 64)
#define IsSeventhSet(x)		(((x)&128) ==128)
#define IsEighthSet(x)		(((x)&256) ==256)
#define IsNinthSet(x)		(((x)&512) ==512)
#define Is10and11Set(x)		(((x)&3072)==3072)
#define Is12Set(x)			(((x)&4096)==4096)

#define TurnOn(x, flag)		((x) |= (flag))
#define TurnOff(x, flag) 	((x) &= ~(flag))

/****************************************************************/

typedef struct menu_item {
	int ulx;
	int uly;
	int state;
	int length;
	char *title;
	char code;
} *MenuItemType;

typedef struct xy_data *XY_DATA;

struct xy_data {
	float far *y;
	float far *x;
	int *npoint;
	int  color;
	XY_DATA next;
}; 

typedef struct window_struct {
	float ymin;
	float ymax;
	float xmin;
	float xmax;
	float xscale;
	float yscale;
	char *X_Label;
	char *Y_Label;
	int ulx;
	int uly;
	int lrx;
	int lry;
	int ulxc;
	int ulyc;
	int lrxc;
	int lryc;
	int state;
	XY_DATA data;
} *WindowType;

typedef struct dialogbox_item {
	int type;
	char *name;
	void *ptr;
} *DialogBoxItemType;

typedef struct dialogbox {
	char *msg;
	char *ok_name;
	int number;
	DialogBoxItemType list;
} *DialogBoxType;

/*************************************/
/* Wingraphics configuration. */

#define		WIN				window[item]

#define		SLOW						1
#define		FAST						8

#define		FLOAT						3
#define		INTEGER					5
#define		CHAR						7

#define		ON							1
#define		ERASE						4
#define		CHECKMARK				8
#define		COVER						8
#define 		YAUTORESCALE			16
#define 		YAUTORESCALEBEFORE	32
#define 		XAUTORESCALE			64
#define 		XAUTORESCALEBEFORE	128
#define		ONBEFORE					3072
#define		NOERASEBEFORE    		4096

#define		NOMSG						-1
#define		QUITMSG					5

#define		FIRSTQUAD				0
#define		SECONDQUAD				1
#define		THIRDQUAD				2
#define		FOURTHQUAD				3
#define		UPPERHALF				4
#define		LOWERHALF				5
#define		FULLSCREEN				6

#define		MAXTOOLS					5
#define		MAXFILETOOL	 			4
#define		MAXMENUBARITEM			6

#define		MENUBAR_ULY				12
#define		MENUBAR_LRY				24

#define		CHAR_HEIGHT				11
#define		CHAR_WIDTH				8
#define		CHAR_DBWIDTH			16

#define		WINBNDTHICKNESS		2

#define		BKGDCOLOR				0
#define		MSGCOLOR					15
#define		BKGDMSGCOLOR			4
#define		MENUTEXTCOLOR			0
#define		MENUBKGDCOLOR			3
#define		XMENUTEXTCOLOR			14
#define		XMENUBKGDCOLOR			1
#define		CHECKMARKCOLOR			12

#define		WINBKGDCOLOR			0
#define		YLABELTEXTCOLOR		11
#define		LABELTEXTCOLOR			11
#define		WINTITLECOLOR			1
#define		WINBORDERCOLOR			8
#define		XWINBORDERCOLOR		7
#define		GRAPHBORDERCOLOR		12

#define		XCOLOR   				7
#define		XSHADOWCOLOR			8
#define		DIALOGBOXTEXTCOLOR   0
#define		DIALOGBOXBKGDCOLOR   7
#define		DIALOGBOXBORDERCOLOR 12

#define		DBL_MIN					1E-200
#define		DBL_CAP					1E+50

#define		X3DFACTOR				3
#define		Y3DFACTOR				3

/****************************************/
/* Key definitions.  */

#define 		EGA  						16
#define 		TEXT 	    				3

#define		TRUE						1
#define		FALSE						0

#define		NULL						0

#define		BKSP						8
#define		ESC						27
#define		RETURN					13

#define		LT_ARROW					575
#define		ST_LT_ARROW				52

#define  	RT_ARROW					577
#define  	ST_RT_ARROW				54

#define		UP_ARROW					572
#define		ST_UP_ARROW				56

#define		DN_ARROW					580
#define		ST_DN_ARROW				50

#define		OFF_SET					500

#define		ALT_R						519
#define		ALT_S						531
#define		ALT_E						518
#define		ALT_T						520
#define		ALT_D						532
#define		ALT_F						525
#define		ALT_Q						516

/****************************************/
/* Wingraphics global variables. */

/****************************************************************/
/* Check if this is the origin for the global variables or just
   an external reference. */

#if defined(GLOBALORIGIN)
	#define GLOBAL
#else
	#define GLOBAL		extern
#endif

/*******************************************************************/

GLOBAL int runFlag, stepFlag, HighestP;

GLOBAL int MAXWINDOWS;
GLOBAL int MINWINWIDTH;
GLOBAL int MINWINHEIGHT;
GLOBAL int SCREENWIDTH;
GLOBAL int SCREENHEIGHT;
GLOBAL int NUMPAGES;

GLOBAL int COLOR[8];

GLOBAL FILE *fp;
GLOBAL char *InputFile;
GLOBAL char DFile[15];
GLOBAL char PSFile[15];
GLOBAL char CGMFile[15];

GLOBAL char CheckMark[2];
GLOBAL int	TOOLS_ULX;
GLOBAL int  FILETOOL_ULX;
GLOBAL int	DIAGNOSTIC_ULX;
GLOBAL int  DIALOGBOX_ULX;
GLOBAL int  DIALOGBOX_ULY;

GLOBAL int *priorityq;
GLOBAL WindowType window;
GLOBAL MenuItemType diagnostic;
GLOBAL MenuItemType menubar;
GLOBAL MenuItemType tool;
GLOBAL MenuItemType print;
GLOBAL struct dialogbox rescaledb;
GLOBAL struct dialogbox cgmdb;
GLOBAL struct dialogbox psdb;
GLOBAL struct dialogbox dfdb;
GLOBAL struct dialogbox messagedb;

GLOBAL int VideoPage;

GLOBAL XY_DATA SetupDataStruct();

GLOBAL void PaintWindow(), CGMWindow(), PSWindow(), (*plotptr)();

GLOBAL void CGM(), PS(), DF(), ScreenDump(), WriteMargin();
GLOBAL void	InitRescaleDialogBox(), SetWindowPosition();
GLOBAL void InitMenuBar(), InitTools();
GLOBAL void InitDiagnostics(), InitFileTool();
GLOBAL void InitCGMDialogBox(), InitPSDialogBox();
GLOBAL void InitMessageDialogBox(), InitDFDialogBox();
GLOBAL int  DialogBox();
GLOBAL int  CGMx(int), CGMy(int);

extern void Dump();

GLOBAL int PostCnt, CGMCnt;
GLOBAL int PSDumpLimit, CGMDumpLimit;
GLOBAL int PSDumpPeriod, CGMDumpPeriod;
GLOBAL float TimeStep;
GLOBAL char *TimePtr;

#undef GLOBAL
/***************************************/
/* Wingraphics macros.   */

#define IsOn(object)							IsZerothSet((object).state)
#define IsPtrOn(object)						IsZerothSet((object)->state)
#define IsNoErase(object)					IsSecondSet((object).state)
#define IsChecked(object)					IsThirdSet((object)->state)
#define IsCovering(object)					IsThirdSet((object).state)
#define IsYAutoRescale(object)			IsFourthSet((object).state)
#define IsYAutoRescaleBefore(object)	IsFifthSet((object).state)
#define IsXAutoRescale(object)			IsSixthSet((object).state)
#define IsXAutoRescaleBefore(object)	IsSeventhSet((object).state)
#define WasOnBefore(object)				Is10and11Set((object).state)
#define IsNoEraseBefore(object)			Is12Set((object).state)

#define IsScatter(object)					!IsEighthSet((object).state) && !IsNinthSet((object).state)
#define IsLinear(object)					 IsEighthSet((object).state) && !IsNinthSet((object).state)
#define IsSemilog(object)					!IsEighthSet((object).state) &&  IsNinthSet((object).state)
#define Is3D(object)							 IsEighthSet((object).state) &&  IsNinthSet((object).state)

#define On(object)							TurnOn((object).state, ON)
#define Off(object)							TurnOff((object).state,ON)

#define OnBefore(object)					TurnOn((object).state,  ONBEFORE)
#define WinOnBefore(object)				TurnOn((object).state, VideoPage ? 2048 : 1024)
#define OffBefore(object)					TurnOff((object).state, ONBEFORE)

#define NoErase(object)						TurnOn((object).state,  ERASE)
#define Erase(object)						TurnOff((object).state, ERASE+NOERASEBEFORE)
#define NoEraseBefore(object)				TurnOn((object).state,  NOERASEBEFORE)
#define EraseBefore(object)				TurnOff((object).state, NOERASEBEFORE)

#define CheckMarkOn(object)				TurnOn((object).state,  CHECKMARK)
#define CheckMarkOff(object)				TurnOff((object).state, CHECKMARK)

#define Covering(object)					TurnOn((object).state,  COVER)
#define NotCovering(object)				TurnOff((object).state, COVER)

#define YAutoRescale(object)				TurnOn((object).state,  YAUTORESCALE)
#define YNotAutoRescale(object)			TurnOff((object).state, YAUTORESCALE)

#define YAutoRescaleBefore(object)   	TurnOn((object).state, YAUTORESCALEBEFORE)
#define YNotAutoRescaleBefore(object)	TurnOff((object).state,YAUTORESCALEBEFORE)

#define XAutoRescale(object)				TurnOn((object).state,  XAUTORESCALE)
#define XNotAutoRescale(object)			TurnOff((object).state, XAUTORESCALE)

#define XAutoRescaleBefore(object)   	TurnOn((object).state, XAUTORESCALEBEFORE)
#define XNotAutoRescaleBefore(object)	TurnOff((object).state,XAUTORESCALEBEFORE)

#define SetLinear(object)					TurnOn((object).state, 256)	
#define SetSemilog(object)					TurnOn((object).state, 512)	
#define Set3D(object)						TurnOn((object).state, 768)	

#define IsReadable(msg)						((msg) > NOMSG && (msg) <= QUITMSG) 
#define Width(window)						((window).lrx - (window).ulx)
#define Height(window)						((window).lry - (window).uly)

#define PrinterSelected						IsFourthSet(PRINTERSTATUS())

/****************************************************************/
