/* 
**	ES1 - ELECTROSTATIC 1 DIMENSIONAL PLASMA SIMULATION
**
**
**	REVISION/PROGRAMMER/DATE
**
**	1.0 Conversion from FORTRAN to C/T. Lasinski
**
**	2.0 Conversion to MS C 5.0, improvement of speed and graphics resolution (to
**	EGA), addition of diagnostics (E,,KE,TE)/John P. Verboncoeur/12-12-87
**
**	2.1 Dynamic array allocation/John P. Verboncoeur/02-23-88
**	2.101 CGA code/John P. Verboncoeur/03-27-88
**	2.102 Fix Maxwellian thermal loaders/John P. Verboncoeur/04-01-88
**	2.103 Fix Magnetized velocity rotator/John P. Verboncoeur/04-18-88
**	2.104 Register optimizations/John P. Verboncoeur/04-21-88
**	2.105 Add kperp (charge disk)/John P. Verboncoeur/05-07-88
**	2.106 Add Field Energy histories for Fourier modes 1-3/John P.
**			Verboncoeur/05-08-88
**	2.107 Enhance display paging scheme/John P. Verboncoeur/06-14-88
**	2.108 Make '.INP' the default input deck extension/John P.
**			Verboncoeur/06-18-88
**	2.109 Enable arbitrary number of text lines in input decks/John P.
**			Verboncoeur/06-20-88
**	2.11  Semi-log plots/John P. Verboncoeur/07-29-88
**	
**	3.0 WinGraphics interface/John P. Verboncoeur and Vahid Vahedi/02-28-89
**
**      4.0 XGRAFIX interface/Vahid Vahedi and John Verboncoeur/01-04-91
**
*/ 

#define GLOBALORIGIN
#include "es1.h"
#undef GLOBALORIGIN

#include "../xgrafix/xgrafix.h"

main(argc, argv)
int argc;
char *argv[];
{
  int i, msg=0;

  display_title();

  /*********************************************************/
  /* Read input file, and initialize params and vars. This */
  /* function calls SPECIES and LOAD or Restore            */
   
  Start(argc,argv);
  history();
  InitWindows(argc, argv);

  /*********************************************************/
  /* If collisions are included in the simulation load the */
  /* ion and electron-neutral collision cros-sections     */

  while(True)
  {
    if(theStepFlag || theRunFlag) 
    {
      p = 0.0;
      for (i=1; i <= nsp; i++) 
      {
	accel(ins[i], ins[i+1]-1, qs[i], ms[i], ts[i],
	      &pxs_hist[i], &kes_hist[i]);
	p += pxs_hist[i];
      }
      for (i=1;i <= ng;i++) rho[i] = rho0;
      rho[ng1] = 0.0;
      
      for (i=1; i <= nsp; i++)
	move(ins[i],ins[i+1]-1,qs[i]);
      
      rho[1] += rho[ng1];
      rho[ng1] = rho[1];
      fields(ith);
      
      ith = ++it -ithl;
      t= it*dt;
      
      history();

      Refresh_Screen();
    }
    Event_Manager();  
  }
}   /* End of MAIN */

/****************************************************************/

display_title()
{
  printf("\n\nES1 - Electrostatic 1 Dimensional Code\n");
  printf("version 4.0\n");
  printf("(c) Copyright 1987-89 Regents of the University of California\n");
  printf("Plasma Theory and Simulation Group\n");
  printf("University of California - Berkeley\n");
}

/****************************************************************/

Start(argc,argv) 
int argc;
char *argv[];
{
  char a_char[80];
  int i, j;
  
  /**********************************************************/
  /* Setting the global parameters to their default values. */
  
  l=6.283185307;
  dt=0.2;
  nsp=1;
  
  epsi=1.0;
  ng=32;
  iw=2;
  
  ins[1] = 1;
  interval=1;
  /***********************************************************/
  
  if (argc<2)	InputDeck = fopen("es1data","r");
  else InputDeck = fopen(argv[1],"r");
  
  if (InputDeck == NULL)
  {
    for (i=0; argv[1][i] != 0; i++) a_char[i] = argv[1][i];
    a_char[i++] = '.';		/*try to add .inp */
    a_char[i++] = 'i';
    a_char[i++] = 'n';
    a_char[i++] = 'p';
    a_char[i++] = 0;
    InputDeck = fopen(a_char,"r");
  }
  if (InputDeck == NULL){
    printf("\nCan't find input file %s\n",argv[1]);
    printf("\nCorrect syntax is: ES1 file.INP");
    exit(-1);
  }
  
  /* read lines until we get to numbers */
  
  while (fscanf(InputDeck,"%d %lG %lG %d %d %lG",
		&nsp, &l, &dt, &nt, &mmax, &la) <6)
    fscanf(InputDeck, "%s", a_char);
  /* note: la is l/a */
  
  while (fscanf(InputDeck," %d %d %lG %lG %lG %lG %lG",
		&ng, &iw, &epsi, &a1, &a2, &e0, &w0) < 7)
    fscanf(InputDeck, "%s", a_char);
  
  if (nsp > NSPM) {
    printf("Number of species nsp cannot exceed NSPM\n");
    exit(-1);
  }
  if (ng > NGMAX) {
    printf("Number of grids ng cannot exceed NGMAX\n");
    exit(-1);
  }
  dx = l/ng;
  ng1= ng+1;
  k_hi= ng/2; 
  
  /*******************************/
  /* Allocating space for arrays */
  
  nms= (double *)malloc((nsp+1)*sizeof(double));
  ms = (double *)malloc((nsp+1)*sizeof(double));
  qs = (double *)malloc((nsp+1)*sizeof(double));
  ts = (double *)malloc((nsp+1)*sizeof(double));
  
  x_array= (double *)malloc((ng+1)*sizeof(double));
  for (i=0; i<= ng; i++) x_array[i] = i*dx;
  
  rho= (double *)malloc((ng+2)*sizeof(double));
  phi= (double *)malloc((ng+2)*sizeof(double));
  phik=(double *)malloc((ng+2)*sizeof(double));
  
  k_array= (double *)malloc(ng*sizeof(double));
  for(i=0; i< k_hi; i++) k_array[i]= i*2*PI/l;
  
  e  = (double *)malloc((ng+2)*sizeof(double));
  acc= (double *)malloc((ng+2)*sizeof(double));

  t_array= (double *)malloc(HISTMAX*sizeof(double));
  ese= (double *)malloc(HISTMAX*sizeof(double));
  ke = (double *)malloc(HISTMAX*sizeof(double));
  te = (double *)malloc(HISTMAX*sizeof(double));
  
  kes_hist= (double *)malloc((nsp+1)*sizeof(double));
  pxs_hist= (double *)malloc((nsp+1)*sizeof(double));
  esem_hist=(double *)malloc((mmax+1)*sizeof(double));
  
  kes = (double **)malloc(nsp*sizeof(double *));
  for (i=0; i< nsp; i++) 
    kes[i] = (double *)malloc(HISTMAX*sizeof(double));
  
  pxs = (double **)malloc(nsp*sizeof(double *));
  for (i=0; i< nsp; i++) 
    pxs[i] = (double *)malloc(HISTMAX*sizeof(double));
  
  esem = (double **)malloc(mmax*sizeof(double *));
  for (i=0; i< mmax; i++) 
    esem[i] = (double *)malloc(HISTMAX*sizeof(double));
  
  x  = (double *)malloc(16001*sizeof(double));
  vx = (double *)malloc(16001*sizeof(double));
  vy = (double *)malloc(16001*sizeof(double));
  
  if (!x || !vx || !vy)
  {
    printf("START: Could not get enough memory for x or v's.");
    exit(1);
  }
  
  printf(" nsp = %2d     l = %8.5f \n",nsp,l);
  printf(" dt = %4.2f    nt = %4d \n",dt,nt);
  printf(" ng = %5d   iw = %2d \n",ng,iw);
  printf(" epsi = %4.2f  a1 = %4.2f  a2 = %4.2f \n",epsi,a1,a2);
  
  for (i=1;i <= nsp;i++) 
    init(&ins[i],&ins[i+1],&ms[i],&qs[i],&ts[i],&nms[i]);
  fclose(InputDeck);
  
  for (i=1; i<= nsp; i++) np[i]= ins[i+1] -ins[i];
  
  for (i=0; i < nsp; i++)
  {
    for (j=0; j < HISTMAX; j++) 
    {
      kes[i][j] = 0.0;
      pxs[i][j] = 0.0;
    }
  }
  rho[1] += rho[ng1];
  rho[ng1] = rho[1];
  fields(0);
  
  for (i=1; i <= nsp; i++) 
    setv(ins[i], ins[i+1]-1, qs[i], ms[i], ts[i], &pxs_hist[i], &kes_hist[i]);
}

/**************************************************************/

history()
{
  int i, j, k;
  static int count=1;
  
  if (--count) return;				/* only accum every interval steps */
  if (hist_hi >= HISTMAX)			/* comb time histories */
  {
    for (j=0; j< nsp; j++)
    {
      for (i=1, k=4; i<HISTMAX/4; i++, k+=4)
      {
	kes[j][i] = kes[j][k];
	pxs[j][i] = pxs[j][k];
      }
    }
    for (j=0; j< mmax; j++)
    {
      for (i=1, k=4; i<HISTMAX/4; i++, k+=4)
	esem[j][i] = esem[j][k];
    }
    for (i=1, k=4; i<HISTMAX/4; i++, k+=4)
    {
      ese[i] = ese[k];
      ke[i] = ke[k]; 
      te[i] = te[k];
      t_array[i] = t_array[k];
    }
    hist_hi = i;
    interval *= 4;
  }
  t_array[hist_hi]= t;
  for (i=0; i< mmax; i++) 
    esem[i][hist_hi] = log10(fabs(esem_hist[i+1]) + DBL_MIN);
  
  ke[hist_hi]=0;
  for (j=0; j< nsp; j++)
  {
    ke[hist_hi] += kes_hist[j+1];
    kes[j][hist_hi] = log10(kes_hist[j+1]+DBL_MIN);
    pxs[j][hist_hi] = pxs_hist[j+1];
  }
  te[hist_hi] = log10(ke[hist_hi] + ese_hist +DBL_MIN);
  ke[hist_hi] = log10(ke[hist_hi]+DBL_MIN);
  ese[hist_hi] = log10(ese_hist+DBL_MIN);
  hist_hi++;
  count = interval;
}

/**************************************************************/

