#include "es1.h"

/* the function rand() returns pseudo-random numbers in the range     */
/* from 0 to (2^15)-1 on some machines and 0 to (2^32)-1 on other     */
/* machines. Choose one of the two definitions for the MACRO frand(). */

#ifdef rand32
#define frand()    ((double) rand()/2147483648.0)
#endif

#ifdef rand15
#define frand()    ((double) rand()/32768.0)
#endif

init(il1,il2,m,q,t,nm,rho0)
int *il1,*il2;
double *m,*q,*t,*nm,rho0;
{
  double wp=1.0, wc=0.0, qm=-1.0, vt1=0.0, vt2=0.0, v0=0.0,
         x1=0.0, v1=0.0, thetax = 0.0, thetav=0.0, ddx, x0,
         vmax, dv, vvnv2, vv, fv, df, xs, xsi, theta, lg;

  int ngr, i, j, i1, i2, n=128, nlg=1, nv2=0, mode=1;
  char a_char[80];
  
  printf("\n");
  while (fscanf(InputDeck, "%d %d %d %d", &n, &nv2, &nlg, &mode) < 4)
    fscanf(InputDeck,"%s",a_char);
  while (fscanf(InputDeck, "%lG %lG %lG %lG %lG %lG", &wp, &wc, &qm, &vt1, &vt2, &v0) < 6)
    fscanf(InputDeck,"%s",a_char);
  while (fscanf(InputDeck,"%lG %lG %lG %lG", &x1, &v1, &thetax, &thetav) < 4)
    fscanf(InputDeck,"%s",a_char);
  printf("\n");
  printf(" n = %4d  nv2 = %4d  nlg = %4d  mode = %4d \n", 
	 n,nv2,nlg,mode);
  printf(" wp = %6.3f   wc = %6.3f   qm = %6.3f \n",wp,wc,qm);
  printf(" vt1 = %6.3f  vt2 = %6.3f  v0 = %6.3f \n",vt1,vt2,v0);
  printf(" x1 = %6.3f   v1 = %6.3f   thetax = %6.3f   thetav = %6.3f \n",
	 x1,v1,thetax,thetav);
  printf("\n");
  
  *t = tan(-0.5*wc*dt);
  *il2 = *il1 + n;
  *q = l*wp*wp/(epsi*n*qm);
  *m = *q/qm;
  *nm = n*(*m);
  ngr = n/nlg;
  lg = l/nlg;
  ddx = l/n;
  for (i=1;i <= ngr;i++) {
    i1 = i - 1 + *il1;
    x0 = ( i - 0.5)*ddx;
    x[i1] = x0;
    vx[i1] = v0;
  }
  if (vt2 != 0.0) 
  {
    vmax = 5.0*vt2;
    dv = 2.0*vmax/(n - 1);
    vvnv2 = 1.0;
    x[*il1] = 0.0;
    for (i=2;i <= n;i++)
    {
      vv = ((i - 1.5)*dv - vmax)/vt2;
      if (nv2 != 0) vvnv2 = pow(vv,(double)nv2);
      fv = vvnv2*exp(-0.5*vv*vv);
      i1 = i - 1 + *il1;
      x[i1] = x[i1 - 1] + ((fv >= 0.0) ? fv : 0.0);
    }
    df = x[i1]/ngr;
    i1 = *il1;
    j = *il1;
    for (i=1;i <= ngr;i++)
    {
      fv = (i - 0.5)*df;
      while (fv >= x[j+1]) {
	j++;
	if (j > (*il2 - 2)) perror("distribution function error");
      }
      vv = dv*(j - *il1 + (fv - x[j])/(x[j+1] - x[j])) - vmax;
      vx[i1] += vv;
      i1++;
    }
    xs = 0.0;
    for (i=1;i <= ngr;i++)
    {
      i1 = i - 1 + *il1;
      x[i1] = xs*lg + 0.5*ddx;
      xsi = 1.0;
      do
      {
	xsi *= 0.5;
	xs -= xsi;
      } while (xs >= 0.0);
      xs += 2.0*xsi;
    }
    i1 = ngr + *il1 - 1;
  }
  if (wc != 0.0) 
  {
    for (i=1; i<=ngr; i++) 
    {
      i1 = i - 1 + *il1;
      vv = vx[i1];
      theta = 2*PI*frand();
      vx[i1] = vv*cos(theta);
      vy[i1] = vv*sin(theta);
    }
  }
  if (nlg != 1)
  {
    j = ngr + 1;
    xs = 0.0;
    for (i=j;i <= n;i+=ngr)
    {
      xs += lg;
      for (j=1;j <= ngr;j++) 
      {
	i1 = j - 1 + *il1;
	i2 = i1 + i - 1;
	x[i2] = x[i1] + xs;
	vx[i2] = vx[i1];
	if (wc != 0.0) vy[i2] = vy[i1];
      }
    }
  }
  if (vt1 != 0.0)
  {
    for (i=1;i <= n;i++)
    {
      i1 = i - 1 + *il1;
      for (j=0; j<12; j++)
      {
	if (wc != 0.0) vy[i1] += vt1*(frand() - 0.5);
	vx[i1] += vt1*(frand() - 0.5);
      }
    }
  }
  for (i=1;i <= n;i++)
  {
    i1 = i - 1 + *il1;
    theta = 2*PI*mode*x[i1]/l;
    x[i1] += x1*cos(theta + thetax);
    vx[i1] += v1*sin(theta + thetav);
  }
  setrho(*il1,*il2 - 1,*q,(*q)*n/l);
}

