#include "../xgrafix/xgrafix.h"
#include "es1.h"

/****************************************************************/

InitWindows(argc, argv)
int argc;
char *argv[];
{
  int i, j;
  XY_DATA data_prevptr, data_ptr;
  XY_LABEL label_ptr;
  char **buffer;

  theTimeStep = &t;
  Init_X(argc, argv);

  /*********************************************/
  /* Set up each window structure              */
  
  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Scale       = dx/dt;
  label_ptr->Y_Min         = -3.0;
  label_ptr->Y_Max         =  3.0;
  label_ptr->Y_Label       = "Vx-X Phase Space";
  label_ptr->Y_Auto_Rescale= False;

  label_ptr->X_Scale       = dx;
  label_ptr->X_Min         = 0;
  label_ptr->X_Max         = l;
  label_ptr->X_Label       = "X";
  label_ptr->X_Auto_Rescale= False;

  data_prevptr = NULL;
  for (i=nsp; i>=1; i--)
  {
    data_ptr         = SetupDataStruct();
    data_ptr->y	     = &vx[ins[i]];
    data_ptr->x	     = &x[ins[i]];
    data_ptr->npoints= &np[i];
    data_ptr->color  = Color[i-1];
    data_ptr->next   = data_prevptr;
    data_prevptr     = data_ptr;
  }
  
  SetupWindow(label_ptr, data_ptr, SCATTERED, OPEN_STATE, 300, 10);   

  /*********************************************/

  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "rho(x)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Min         = 0;
  label_ptr->X_Max         = l;
  label_ptr->X_Label       = "X";
  label_ptr->X_Auto_Rescale= False;

  data_ptr         = SetupDataStruct();
  data_ptr->y	   = &rho[1];
  data_ptr->x	   = x_array;
  data_ptr->npoints= &ng1;
  data_ptr->color  = Color[2];
  
  SetupWindow(label_ptr, data_ptr, LINEAR, ICONIC_STATE, 400, 100);   

  /*********************************************/

  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "E field(x)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Min         = 0;
  label_ptr->X_Max         = l;
  label_ptr->X_Label       = "X";
  label_ptr->X_Auto_Rescale= False;

  data_ptr         = SetupDataStruct();
  data_ptr->y	   = &e[1];
  data_ptr->x	   = x_array;
  data_ptr->npoints= &ng1;
  data_ptr->color  = Color[3];
  
  SetupWindow(label_ptr, data_ptr, LINEAR, OPEN_STATE, 10, 500);   

  /*********************************************/

  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "Potential(x)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Min         = 0;
  label_ptr->X_Max         = l;
  label_ptr->X_Label       = "X";
  label_ptr->X_Auto_Rescale= False;

  data_ptr         = SetupDataStruct();
  data_ptr->y	   = &phi[1];
  data_ptr->x	   = x_array;
  data_ptr->npoints= &ng1;
  data_ptr->color  = Color[4];
  
  SetupWindow(label_ptr, data_ptr, LINEAR, OPEN_STATE, 400, 300);   
	
  /*********************************************/

  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "Potential(k)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Label       = "k";
  label_ptr->X_Auto_Rescale= True;

  data_ptr         = SetupDataStruct();
  data_ptr->y	   = &phik[1];
  data_ptr->x	   = k_array;
  data_ptr->npoints= &k_hi;
  data_ptr->color  = Color[3];
  
  SetupWindow(label_ptr, data_ptr, SEMILOG, ICONIC_STATE, 600, 300);   

  /*********************************************/

  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "Kinetic Energy(t)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Label       = "Time";
  label_ptr->X_Auto_Rescale= True;
  
  data_prevptr = NULL;
  for (i=nsp-1; i>=0; i--)
  {
    data_ptr         = SetupDataStruct();
    data_ptr->y	     = kes[i];
    data_ptr->x	     = t_array;
    data_ptr->npoints= &hist_hi;
    data_ptr->color  = Color[i];
    data_ptr->next   = data_prevptr;
    data_prevptr     = data_ptr;
  }
  data_ptr         = SetupDataStruct();
  data_ptr->y	     = ke;
  data_ptr->x	     = t_array;
  data_ptr->npoints= &hist_hi;
  data_ptr->color  = Color[4];
  data_ptr->next   = data_prevptr;
  data_prevptr     = data_ptr;

  SetupWindow(label_ptr, data_ptr, SEMILOG, ICONIC_STATE, 800, 200);   

  /*********************************************/
  
  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "Field Energy(t)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Label       = "Time";
  label_ptr->X_Auto_Rescale= True;
  
  data_ptr         = SetupDataStruct();
  data_ptr->y	     = ese;
  data_ptr->x	     = t_array;
  data_ptr->npoints= &hist_hi;
  data_ptr->color  = Color[4];
  
  SetupWindow(label_ptr, data_ptr, SEMILOG, ICONIC_STATE, 700, 400);   

  /*********************************************/

  buffer= (char **)malloc(mmax*sizeof(char *));
  
  for(i=0; i < mmax; i++)
  {
    buffer[i]= (char *)malloc(15*sizeof(char));
    sprintf(buffer[i], "Mode %d ESE", i+1);
    
    label_ptr  = SetupLabelStruct();
    label_ptr->Y_Label       = buffer[i];
    label_ptr->Y_Auto_Rescale= True;
    
    label_ptr->X_Label       = "Time";
    label_ptr->X_Auto_Rescale= True;
    
    data_ptr         = SetupDataStruct();
    data_ptr->y	     = esem[i];
    data_ptr->x	     = t_array;
    data_ptr->npoints= &hist_hi;
    data_ptr->color  = Color[4];
    
    SetupWindow(label_ptr, data_ptr, SEMILOG, ICONIC_STATE, 700, 400);   
  }

  /*********************************************/

  label_ptr  = SetupLabelStruct();
  label_ptr->Y_Label       = "Total Energy(t)";
  label_ptr->Y_Auto_Rescale= True;

  label_ptr->X_Label       = "Time";
  label_ptr->X_Auto_Rescale= True;
  
  data_ptr         = SetupDataStruct();
  data_ptr->y	     = te;
  data_ptr->x	     = t_array;
  data_ptr->npoints= &hist_hi;
  data_ptr->color  = Color[4];
  
  SetupWindow(label_ptr, data_ptr, SEMILOG, ICONIC_STATE, 700, 600);   
} 

/***************************************************************/
/* Dumping the current state of the system into a binary file. */
/* Note: the binary file is written in the format used by IBM  */
/* and DEC compatible machines (low bytes followed by high     */
/* bytes). For this Dump() and Restore() call the functions    */
/* myfread() and myfwrite().                                   */

char Revision[]={'1','.','0','2'};

void Dump(filename)
char *filename;
{
}

/****************************************************************/

void Restore(filename)
char *filename;
{
}

/****************************************************************/

myfread(ptr, size, nitems, stream)
char *ptr;
int size;
int nitems;
FILE *stream;
{
  int i, j;
  
  for(j= 0; j <nitems; j++)
    for(i= size-1; i >=0; i--)
      fread(ptr+j*size+i, sizeof(char), 1, stream);
}

/****************************************************************/

myfwrite(ptr, size, nitems, stream)
char *ptr;
int size;
int nitems;
FILE *stream;
{
  int i, j;
  
  for(j= 0; j <nitems; j++)
    for(i= size-1; i >=0; i--)
      fwrite(ptr+j*size+i, sizeof(char), 1, stream);
}

/****************************************************************/




