#define GLOBALORIGIN
#include "xgrafix.h"
#undef GLOBALORIGIN

/****************************************************************/
/* XGRAFIX is a costomized environment to run in X11.           */
/* Authors: Vahid Vahedi & John Verboncoeur.                    */
/****************************************************************/

Init_X(argc, argv)
int argc;
char *argv[];
{
  XColor theRGBColor, theHardwareColor;
  int    i=0;
  char   temp[60];

  /*****************************************/
  /* Getting the code and input file names */

  sprintf(theCodeName, "%s", argv[0]);
 
  if(argc >1){
    strcpy(theInputFile, argv[1]);
    
    if(argc>2)
    {
      strcpy(temp, argv[2]);
      if(access(temp, F_OK)) strcat(temp, theDumpExtension);
    }
    if(argc>2 && !access(temp, F_OK))
    {
      for(i=strlen(temp)-1; i>0; i--) if(temp[i] == '.') break;
      if(i){
	strncpy(theDumpFile, temp, i);
	strncpy(thePSFile, temp, i);
      } else {
	strcpy(theDumpFile, temp);
	strcpy(thePSFile, temp);
      }
    } 
    else
    {
      for(i=strlen(theInputFile)-1; i>0; i--)
	if(theInputFile[i] == '.') break;
      if(i){
	strncpy(theDumpFile, theInputFile, i);
	strncpy(thePSFile, theInputFile, i);
      } else {
	strcpy(theDumpFile, theInputFile);
	strcpy(thePSFile,   theInputFile);
      }
    }
    strcat(theDumpFile, theDumpExtension);
    strcat(thePSFile,   thePSExtension);
  }
  else
  {
    strcpy(theInputFile, "No Input File");
    strcpy(theDumpFile, theDefaultDumpFile);
    strcpy(thePSFile,   theDefaultPSFile);
  }
  /*****************************************/
    
  theDisplay = XOpenDisplay(ReqDisp);

  if (theDisplay == NULL) 
  {
    fprintf(stderr, "XGRAFIX: Cannot establish connection to the XServer %s\n",
	    XDisplayName(NULL));
    exit(1);
  }

  theScreen = DefaultScreen(theDisplay);
  theDepth  = DefaultDepth(theDisplay, theScreen);

  theBlackPixel = BlackPixel(theDisplay, theScreen);
  theWhitePixel = WhitePixel(theDisplay, theScreen);

  theColormap   = DefaultColormap(theDisplay, theScreen);
  theRootWindow = RootWindow(theDisplay, theScreen);

  theDisplayWidth = DisplayWidth(theDisplay,theScreen);
  theDisplayHeight= DisplayHeight(theDisplay,theScreen);

  printf("XGRAFIX is running on %s version %d of the X Window System, X%d R%d\n",
	 ServerVendor(theDisplay), VendorRelease(theDisplay),
	 ProtocolVersion(theDisplay), ProtocolRevision(theDisplay));

  if (theDepth == 1) 
    printf("Color plane depth ......... %d (monochrome)\n",theDepth);
  else
    printf("Color plane depth ......... %d\n", theDepth);
  
  printf("Display width ............. %d\n", theDisplayWidth);
  printf("Display height ............ %d\n", theDisplayHeight);
  printf("The display %s\n\n", XDisplayName(theDisplay));

  /****************************/
  /* Initializing the colors  */

  if (theDepth > 1)
  {
    for (i =0; i <MAXCOLORS; i++) 
    {
      if(XLookupColor(theDisplay, theColormap, theColorNames[i], &theRGBColor, &theHardwareColor))
      	if(XAllocColor(theDisplay, theColormap, &theHardwareColor)) Color[i] = theHardwareColor.pixel;
    }
    theWinTextColor= Color[0];
    theDBTextColor = Color[5];
    theBorderColor = Color[24];
    theBKGDColor   = Color[9];
    theGrayShade1  = Color[10];
    theGrayShade2  = Color[11];
    theGrayShade3  = Color[12];
    theGrayShade4  = Color[13];
    theGrayShade5  = Color[14];
    theGrayShade6  = Color[15];
    theGrayShade7  = Color[16];
    theGrayShade8  = Color[17];
    theGrayShade9  = Color[18];
    theGrayShade10 = Color[19];
  }
  else
  { 
    for (i = 0; i < MAXCOLORS; i++) Color[i]= theWhitePixel;
    theWinTextColor= theWhitePixel;
    theDBTextColor = theWhitePixel;
    theBorderColor = theWhitePixel;
    theBKGDColor   = theBlackPixel;
    theGrayShade1  = theBlackPixel; 
    theGrayShade2  = theWhitePixel;
    theGrayShade3  = theBlackPixel;
    theGrayShade4  = theWhitePixel; 
    theGrayShade5  = theBlackPixel; 
    theGrayShade6  = theBlackPixel;
    theGrayShade7  = theWhitePixel; 
    theGrayShade8  = theWhitePixel; 
    theGrayShade9  = theWhitePixel; 
    theGrayShade10 = theBlackPixel; 
  }
  
  /************************************/
  /* Initializing the XGRAFIX font    */
  
  if((theFontStruct = XLoadQueryFont(theDisplay, theFontName)) == NULL)
  {
    printf("can't open the specified font\n");
    exit(-1);
  }
  theFont       = theFontStruct->fid;
  theFontAscent = theFontStruct->ascent;
  theFontDescent= theFontStruct->descent;
  theFontHeight = theFontAscent + theFontDescent;

  /*****************************/
  /* Initializing the curosrs  */

  theButtonCursor    = XCreateFontCursor(theDisplay, BUTTONCURSOR);
  theCrosshairCursor = XCreateFontCursor(theDisplay, CROSSHAIRCURSOR);

  /************************************/
  /* Initializing the XGRAFIX icon    */

  theIcon = XCreateBitmapFromData(theDisplay, theRootWindow,xgrafix_bits,
				  xgrafix_width, xgrafix_height);
  theIconULX= 2;
  theIconULY= theDisplayHeight -40;

  /********************************/
  /* Initializing the main menu   */

  theRunFlag = False;
  theStepFlag= False;
  Init_Main_Menu();

  /**********************************************************/
  /* Initializing the dialog boxs and the related variables */

  Init_DF_DialogBox();
  Init_PS_DialogBox();
  Init_Rescale_DialogBox();
  Init_Message_DialogBox();
  
  theOpenDialogBox       = NULL;
  theOpenDialogBoxWindow = NULL; 
  theDialogBoxButtonLabelX = 10;
  theDialogBoxButtonLabelY = 2*(theFontHeight +10) +theFontAscent;
  theDialogBoxButtonSpacing= theFontHeight+28;

  theCrosshairedWindow   = NULL;

  /*************************************/
  /* Initializing the window link list */

  theWindowList = NULL;
}
 
/****************************************************************/

XY_DATA SetupDataStruct()
{
  XY_DATA data;
  
  if (!(data= (XY_DATA)malloc(sizeof(struct xy_data)))) putchar(7);
  
  data->z      = NULL;
  data->y      = NULL;
  data->x      = NULL;
  data->mpoints= 0;
  data->npoints= 0;
  data->color  = 0;
  data->next   = NULL;

  return(data);
}

/****************************************************************/

XY_LABEL SetupLabelStruct()
{
  XY_LABEL label;
  
  if (!(label= (XY_LABEL)malloc(sizeof(struct xy_label)))) putchar(7);
  
  label->Z_Scale        = 1;
  label->Z_Min          = 0;
  label->Z_Max          = 0;
  label->Z_Label        = NULL;
  label->Z_Auto_Rescale = True;

  label->Y_Scale        = 1;
  label->Y_Min          = 0;
  label->Y_Max          = 0;
  label->Y_Label        = NULL;
  label->Y_Auto_Rescale = True;

  label->X_Scale        = 1;
  label->X_Min          = 0;
  label->X_Max          = 0;
  label->X_Label        = NULL;
  label->X_Auto_Rescale = True;
 
  return(label);
}

/****************************************************************************/

SetupWindow(label_ptr, data_ptr, WINTYPE, STATE, ulx, uly)
XY_LABEL label_ptr;  /* Contains all the info about the labels              */
XY_DATA  data_ptr;   /* Contains all the info about data being plotted      */
int      WINTYPE;    /* SCATTERED, LINEAR, SEMILOG, or THREED_WM            */
int      STATE;      /* OPEN_STATE or ICONIC_STATE                          */
int      ulx, uly;   /* the requested position of the upper left coner      */
{
  XSetWindowAttributes  theWindowAttributes;
  XSizeHints            theSizeHints;
  unsigned long         theWindowMask;
  WindowType            theNewWindow;
  XWMHints              theWMHints;
  XGCValues             theGCValues;
  int                   i, l, offset=0;

  if (!(theNewWindow = (WindowType)malloc(sizeof(struct window_struct)))) putchar(7);

  theNewWindow->state = 0;
  theNewWindow->data  = data_ptr;
  theNewWindow->label = label_ptr;
  theNewWindow->width = DEFAULT_WIN_WIDTH;
  theNewWindow->height= DEFAULT_WIN_HEIGHT;
  theNewWindow->next  = theWindowList;
  theWindowList       = theNewWindow;
  strcpy(theNewWindow->ps_file_name, thePSFile);
  strcpy(theNewWindow->plot_title, label_ptr->Y_Label);

  /****************************************************/
  /* Setting the windiw's attributes and creating it  */

  theWindowAttributes.border_pixel      = theWhitePixel;
  theWindowAttributes.background_pixel  = theGrayShade3;
  theWindowMask = (CWBackPixel | CWBorderPixel);

  theNewWindow->xwin = XCreateWindow(theDisplay, theRootWindow, ulx, uly,
				     DEFAULT_WIN_WIDTH, DEFAULT_WIN_HEIGHT,
				     DEFAULT_WIN_BORDER_WIDTH, theDepth,
				     InputOutput, CopyFromParent,
				     theWindowMask, &theWindowAttributes);

  /**************************************/
  /* Setting the window manager's hints */

  if (STATE == OPEN_STATE)
    theWMHints.initial_state = NormalState;
  else 
    theWMHints.initial_state = IconicState;

  if(theIconULX +8.5*max(strlen(label_ptr->Y_Label), 10) > theDisplayWidth)
  {
    theIconULX  =  2;
    theIconULY -= 40;
  }
   
  theWMHints.icon_pixmap = theIcon;
  theWMHints.icon_x      = theIconULX;
  theWMHints.icon_y      = theIconULY;
  theWMHints.flags       = IconPixmapHint | StateHint | IconPositionHint;
  
  XSetWMHints(theDisplay, theNewWindow->xwin, &theWMHints);
  XStoreName(theDisplay, theNewWindow->xwin, label_ptr->Y_Label);

  theIconULX += 8.5*max(strlen(label_ptr->Y_Label), 10);
  
  /****************************************/
  /* Setting the size and normal hints    */

  theSizeHints.flags  = USPosition | USSize;
  theSizeHints.x      = ulx;
  theSizeHints.y      = uly;
  theSizeHints.width  = DEFAULT_WIN_WIDTH;
  theSizeHints.height = DEFAULT_WIN_HEIGHT;
  
  XSetNormalHints( theDisplay, theNewWindow->xwin, &theSizeHints);
  
  /***************************************/
  /* Initializing a GC for the window    */

  theNewWindow->xwingc = XCreateGC(theDisplay, theNewWindow->xwin,
				   (unsigned long) 0, &theGCValues);
  if (theNewWindow->xwingc)
  {
    XSetForeground(theDisplay, theNewWindow->xwingc, theWhitePixel);
    XSetBackground(theDisplay, theNewWindow->xwingc, theBlackPixel);
  }
  else
  {
    printf("XGRAFIX: can't create a GC for the %s window.", label_ptr->Y_Label);
    XDestroyWindow(theDisplay, theNewWindow->xwin);
  }

  XSelectInput(theDisplay, theNewWindow->xwin, 
	       StructureNotifyMask | SubstructureNotifyMask | ButtonPressMask | ExposureMask);

  /*********************************************/
  /* Setting the font type used in the window */

  XSetFont(theDisplay, theNewWindow->xwingc, theFont);

  /********************************************/
  /* Setting up the buttons for the window    */

  sprintf(theNewWindow->xbutton[0].Label, "%s", "Rescale");
  sprintf(theNewWindow->xbutton[1].Label, "%s", "Trace");
  sprintf(theNewWindow->xbutton[2].Label, "%s", "Print");
  sprintf(theNewWindow->xbutton[3].Label, "%s", "Crosshair");
  
  for(i=0; i< MAX_WIN_BUTTON; i++)
  {
    l = strlen(theNewWindow->xbutton[i].Label);
    theNewWindow->xbutton[i].xwin  = XCreateSimpleWindow(theDisplay, theNewWindow->xwin, offset, 0,
					      XTextWidth(theFontStruct, theNewWindow->xbutton[i].Label, l)+18,
					      theFontHeight+16, 1, theGrayShade5, theBlackPixel);

    XSelectInput(theDisplay, theNewWindow->xbutton[i].xwin,
		 ButtonPressMask | ButtonReleaseMask | ExposureMask);

    theNewWindow->xbutton[i].xwingc = XCreateGC(theDisplay, theNewWindow->xbutton[i].xwin,
						(unsigned long) 0, &theGCValues);
    
    XSetFont(theDisplay, theNewWindow->xbutton[i].xwingc, theFont);
    XDefineCursor(theDisplay, theNewWindow->xbutton[i].xwin, theButtonCursor);

    offset += XTextWidth(theFontStruct, theNewWindow->xbutton[i].Label, l)+22;
  }
  XMapSubwindows(theDisplay, theNewWindow->xwin);

  /****************************************************/
  /* Setting the window type and the other state-bits */

  if (WINTYPE == SCATTERED)
  {
    theNewWindow->paint_function = Paint_Window;
    theNewWindow->print_function = PostScriptOpenWindow;
  }
  if (WINTYPE == LINEAR)       
  {
    Set_Linear(theNewWindow);
    theNewWindow->paint_function = Paint_Window;
    theNewWindow->print_function = PostScriptOpenWindow;
  }
  if (WINTYPE == SEMILOG)
  {
    Set_Semilog(theNewWindow);
    theNewWindow->paint_function = Paint_Window;
    theNewWindow->print_function = PostScriptOpenWindow;
  }
  if (WINTYPE == THREED_WM)
  {
    Set_ThreeDWM(theNewWindow);
    theNewWindow->paint_function = Paint_ThreeDWM_Window;
    theNewWindow->print_function = NULL;
  }

  if (label_ptr->Y_Auto_Rescale)   Y_AutoRescale(theNewWindow); 
  if (label_ptr->X_Auto_Rescale)   X_AutoRescale(theNewWindow);  

  if (STATE == OPEN_STATE)  Open(theNewWindow);
  else                      Iconify(theNewWindow);

  XMapWindow(theDisplay, theNewWindow->xwin);
}

/****************************************************************/

Init_Main_Menu()
{
  XSetWindowAttributes  theWindowAttributes;
  XSizeHints            theSizeHints;
  unsigned long         theWindowMask;
  XWMHints              theWMHints;
  XGCValues             theGCValues;
  int                   i;

  /*******************************************************/
  /* Setting the main menu's attributes and creating it  */

  theWindowAttributes.border_pixel      = theWhitePixel;
  theWindowAttributes.background_pixel  = theBlackPixel;
  theWindowMask = (CWBackPixel | CWBorderPixel);

  theMainMenu.xwin = XCreateWindow(theDisplay, theRootWindow, DEFAULT_MAIN_MENU_X,
				   DEFAULT_MAIN_MENU_Y, DEFAULT_MAIN_MENU_WIDTH,
				   MAX_MAIN_MENU_BUTTON*(theFontHeight+19),
				   DEFAULT_WIN_BORDER_WIDTH, theDepth,
				   InputOutput, CopyFromParent,
				   theWindowMask, &theWindowAttributes);
  
  /********************************************************/
  /* Setting the window manager's hints for the main menu */
  
  theWMHints.initial_state = NormalState;
  theWMHints.icon_pixmap   = theIcon;
  theWMHints.icon_x        = theIconULX;
  theWMHints.icon_y        = theIconULY;
  theWMHints.flags         = IconPixmapHint | StateHint | IconPositionHint;
 
  XSetWMHints(theDisplay, theMainMenu.xwin, &theWMHints);

  /*********************************/
  /* Setting the main menu's name  */

  XStoreName(theDisplay, theMainMenu.xwin, strcat(theCodeName, " Control Panel"));
  theIconULX+= 9*max(strlen(theCodeName), 10);

  /****************************************/
  /* Setting the size and normal hints    */

  theSizeHints.flags  = USPosition | USSize;
  theSizeHints.x      = DEFAULT_MAIN_MENU_X;
  theSizeHints.y      = DEFAULT_MAIN_MENU_Y;
  theSizeHints.width  = DEFAULT_MAIN_MENU_WIDTH;
  theSizeHints.height = MAX_MAIN_MENU_BUTTON*(theFontHeight+19);
  
  XSetNormalHints( theDisplay, theMainMenu.xwin, &theSizeHints);
  
  XSelectInput(theDisplay, theMainMenu.xwin,
	       StructureNotifyMask | ExposureMask | SubstructureNotifyMask);

  /********************************************/
  /* Setting up the buttons for the main menu */

  if (!(theMainMenu.xbutton = (ButtonType *)malloc(MAX_MAIN_MENU_BUTTON*sizeof(ButtonType)))) putchar(7);

  sprintf(theMainMenu.xbutton[0].Label, "Time = %lG", *theTimeStep);
  sprintf(theMainMenu.xbutton[1].Label, "%s", "Run");
  sprintf(theMainMenu.xbutton[2].Label, "%s", "Stop");
  sprintf(theMainMenu.xbutton[3].Label, "%s", "Step");
  sprintf(theMainMenu.xbutton[4].Label, "%s", "Save");
  sprintf(theMainMenu.xbutton[5].Label, "%s", "Quit");
  
  /*************************************************/
  /* Setting up the time counter for the main menu */
        
  theMainMenu.xbutton[0].xwin = XCreateSimpleWindow(theDisplay, theMainMenu.xwin, 0, 0,
						     DEFAULT_MAIN_MENU_WIDTH-2,
						     theFontHeight+16, 1,
						     theGrayShade5, theBlackPixel);
  
  XSelectInput(theDisplay, theMainMenu.xbutton[0].xwin, ExposureMask);
  
  theMainMenu.xbutton[0].xwingc = XCreateGC(theDisplay, theMainMenu.xbutton[0].xwin,
					    (unsigned long) 0, &theGCValues);
  
  XSetFont(theDisplay, theMainMenu.xbutton[0].xwingc, theFont);

  /**************************************************/
  /* Setting up the other buttons for the main menu */

  for(i=1; i< MAX_MAIN_MENU_BUTTON; i++)
  {
    theMainMenu.xbutton[i].xwin = XCreateSimpleWindow(theDisplay, theMainMenu.xwin,
						      0, i*(theFontHeight+19),
						      DEFAULT_MAIN_MENU_WIDTH-2,
						      theFontHeight+16, 1,
						      theGrayShade5, theBlackPixel);
    
    XSelectInput(theDisplay, theMainMenu.xbutton[i].xwin,
		 ButtonPressMask | ButtonReleaseMask | ExposureMask);
    
    theMainMenu.xbutton[i].xwingc = XCreateGC(theDisplay, theMainMenu.xbutton[i].xwin,
					      (unsigned long) 0, &theGCValues);
    
    XSetFont(theDisplay, theMainMenu.xbutton[i].xwingc, theFont);
    XDefineCursor(theDisplay, theMainMenu.xbutton[i].xwin, theButtonCursor);
  }
  XMapSubwindows(theDisplay, theMainMenu.xwin);
  XMapWindow(theDisplay, theMainMenu.xwin);
}

/*****************************************************************/

Event_Manager()
{
  if(theStepFlag) theStepFlag =0;
  if(theRunFlag)
    while(XPending(theDisplay)) Process_One_Event();
  else                          Process_One_Event();
}

/*****************************************************************/

Process_One_Event()
{
  XEvent        theEvent;
  int           i, j, doneFlag;
  WindowType    theWindowPtr;
  Window        thePushedButton;
  static Window thePrevExposedWindow=0;
  ButtonType    theButton;
  unsigned int  button;
  
  doneFlag =1;
  XNextEvent(theDisplay, &theEvent);
  switch (theEvent.type)
  {
    /************************/
  case MapNotify:
    for(theWindowPtr = theWindowList; theWindowPtr != NULL; theWindowPtr = theWindowPtr->next)
      if (theWindowPtr->xwin == theEvent.xmap.window)
      {
	Open(theWindowPtr);
	Iconic_Before(theWindowPtr);
	break;
      }
    thePrevExposedWindow=0;
    break;

    /************************/
  case UnmapNotify:
    for(theWindowPtr = theWindowList; theWindowPtr != NULL; theWindowPtr = theWindowPtr->next)
      if (theWindowPtr->xwin == theEvent.xmap.window)  Iconify(theWindowPtr);
    thePrevExposedWindow=0;
    break;
    
    /************************/
  case ConfigureNotify:
    for(theWindowPtr = theWindowList; theWindowPtr != NULL; theWindowPtr = theWindowPtr->next)
      if (theWindowPtr->xwin == theEvent.xmap.window)
      {
	theWindowPtr->height = theEvent.xconfigure.height;
	theWindowPtr->width  = theEvent.xconfigure.width;
      }
    thePrevExposedWindow=0;
    break;
    
    /************************/
  case Expose:
    if(thePrevExposedWindow != theEvent.xexpose.window)
    {
      for(theWindowPtr = theWindowList; doneFlag && theWindowPtr != NULL; theWindowPtr = theWindowPtr->next)
      {
	if (theWindowPtr->xwin == theEvent.xexpose.window) 
	{
	  Iconic_Before(theWindowPtr);
	  (*(theWindowPtr->paint_function))(theWindowPtr);
	  if(theCrosshairedWindow) SetCrosshairParameters();
	  break;
	}
	for(j=0; j< MAX_WIN_BUTTON; j++)
	{
	  if(theWindowPtr->xbutton[j].xwin == theEvent.xexpose.window)
	  {
	    Paint_Button(theWindowPtr->xbutton[j], BUTTON_NORMAL);
	    doneFlag=0;
	    break;
	  }
	}
      }	
      
      for(j=0; j< MAX_MAIN_MENU_BUTTON; j++) 
      {
	if(theMainMenu.xbutton[j].xwin == theEvent.xexpose.window)
	{
	  Paint_Button(theMainMenu.xbutton[j], BUTTON_NORMAL);
	  break;
	}
      }
      
      if(theCrosshairedWindow)
      {
	for(j=0; j< 2; j++) 
	{
	  if(theXYCrosshairWindow.xbutton[j].xwin == theEvent.xexpose.window)
	  {
	    Paint_Button(theXYCrosshairWindow.xbutton[j], BUTTON_PUSHED);
	    break;
	  }
	}
      }
      
      if(theOpenDialogBox)             /** Drawing the open dialog box and the associated buttons **/
      {
	if (theOpenDialogBox->xwin == theEvent.xexpose.window)
	  Paint_DialogBox(theOpenDialogBox);
	
	for(j=0; j< theOpenDialogBox->number; j++)
	{
	  if(theOpenDialogBox->button_list[j].xbutton.xwin == theEvent.xexpose.window)
	  {
	    if(j== theOpenDialogBox->on_button)
	      Paint_Button(theOpenDialogBox->button_list[j].xbutton, BUTTON_PUSHED);
	    else
	      Paint_Button(theOpenDialogBox->button_list[j].xbutton, BUTTON_NORMAL);
	    break;
	  }
	}
      }
      thePrevExposedWindow = theEvent.xexpose.window;
    }
    break;
    
    /************************/
  case ButtonPress:
    if(theCrosshairedWindow && theEvent.xbutton.window == theCrosshairedWindow->xwin)
    {
      UpdateCrosshair(theEvent.xbutton.x, theEvent.xbutton.y);
      break;
    }
    for(theWindowPtr = theWindowList; doneFlag && theWindowPtr != NULL; theWindowPtr = theWindowPtr->next)
    {
      for(j=0; j< MAX_WIN_BUTTON; j++)
      {
	if(theWindowPtr->xbutton[j].xwin == theEvent.xbutton.window)
	{
	  theButton = theWindowPtr->xbutton[j];
	  button    = theEvent.xbutton.button;
	  Paint_Button(theButton, BUTTON_PUSHED);
	  
	  while(True)
	  {
	    while(XCheckTypedEvent(theDisplay, ButtonPress, &theEvent));
	    XMaskEvent(theDisplay, ButtonReleaseMask, &theEvent);
	    if(theEvent.xbutton.button == button) break;
	  }
	  Paint_Button(theButton, BUTTON_NORMAL);
	  Local_Window_Menu(theWindowPtr, j);
	  doneFlag=0;
	  break;
	}
      }
    }
    for(j=1; j< MAX_MAIN_MENU_BUTTON; j++)
    {
      if(theMainMenu.xbutton[j].xwin == theEvent.xbutton.window)
      {
	theButton = theMainMenu.xbutton[j];
	button    = theEvent.xbutton.button;
	Paint_Button(theButton, BUTTON_PUSHED);
	
	while(True)
	{
	  while(XCheckTypedEvent(theDisplay, ButtonPress, &theEvent));
	  XMaskEvent(theDisplay, ButtonReleaseMask, &theEvent);
	  if(theEvent.xbutton.button == button) break;
	}
	Paint_Button(theButton, BUTTON_NORMAL);
	Main_Menu(j);
	doneFlag=0;
	break;
      }
    }
    if(theOpenDialogBox) 
    {
      for(j=0; j< theOpenDialogBox->number; j++)
      {
	if(theOpenDialogBox->button_list[j].xbutton.xwin == theEvent.xbutton.window)
	{
	  if(theOpenDialogBox->on_button > -1)
	  {
	    Paint_Button(theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton, BUTTON_NORMAL);
	    if(theOpenDialogBox->on_button == j)
	    {
	      theOpenDialogBox->on_button = -1;
	      break;
	    }
	  }
	  theOpenDialogBox->on_button =j;
	  XSetInputFocus(theDisplay, theOpenDialogBox->button_list[j].xbutton.xwin,
			 RevertToParent, CurrentTime);

	  theButton = theOpenDialogBox->button_list[j].xbutton;
	  button    = theEvent.xbutton.button;
	  theCurrentCursorPosition = strlen(theOpenDialogBox->button_list[j].xbutton.Label);
	  Paint_Button(theButton, BUTTON_PUSHED);
	  
	  while(True)
	  {
	    while(XCheckTypedEvent(theDisplay, ButtonPress, &theEvent));
	    XMaskEvent(theDisplay, ButtonReleaseMask, &theEvent);
	    if(theEvent.xbutton.button == button) break;
	  }
	  DialogBox_Menu(theOpenDialogBox);
	  break;
	}
      }
    }
    thePrevExposedWindow=0;
    break;
    
    /***********************/
  case KeyPress:
    if(theOpenDialogBox && theOpenDialogBox->on_button > -1)
      if(theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.xwin == theEvent.xkey.window)
	ReadFromKeyborad(&(theEvent.xkey));
    thePrevExposedWindow=0;
    break;
    
    /************************/
  default:
    thePrevExposedWindow=0;
    break;
  }
}

/****************************************************************/

Refresh_Screen()
{
  WindowType theWindowPtr;
    
  sprintf(theMainMenu.xbutton[0].Label, "Time = %lG", *theTimeStep);
  Paint_Button(theMainMenu.xbutton[0], BUTTON_NORMAL);
 
  for(theWindowPtr = theWindowList; theWindowPtr != NULL; theWindowPtr = theWindowPtr->next)
    if(Is_Open(theWindowPtr))  (*(theWindowPtr->paint_function))(theWindowPtr);
}

/****************************************************************/

Paint_Window(theWindow)
WindowType theWindow;
{
  XY_DATA  data_Ptr, ptr;
  XY_LABEL label_Ptr;
  XPoint  *X_Points, Tick_Marks[2];
  XRectangle theClientArea;

  int n, i, j, k1, k2, maxlen, x1, y1, x2, y2; 
  char buffer1[20], buffer2[20];
  double m, b, m1, b1, tgrid;
  double fx1, fx2, fy1, fy2;
  
  data_Ptr = theWindow->data;
  label_Ptr= theWindow->label;

  /**************************************************************/
  if(theOpenDialogBox && theOpenDialogBoxWindow == theWindow && Is_Semilog(theWindow))
  {
    label_Ptr->Y_Max= log10(label_Ptr->Y_Max +DBL_MIN);
    label_Ptr->Y_Min= log10(label_Ptr->Y_Min +DBL_MIN);
  }

  /**************************************************************/
  /* Making the whole window drawable except the button region. */ 

  theClientArea.x = 0;
  theClientArea.y = theFontHeight+16;
  theClientArea.width = theWindow->width;
  theClientArea.height= theWindow->height;
  XSetClipRectangles(theDisplay, theWindow->xwingc, 0, 0, &theClientArea, 1, Unsorted);

  /****************************/
  /* AutoRescaling the y-axis */
  
  if (Is_Y_AutoRescale(theWindow) && (n= *(theWindow->data->npoints)))
  {
    label_Ptr->Y_Max /= label_Ptr->Y_Scale;
    label_Ptr->Y_Min /= label_Ptr->Y_Scale;
    
    if(Is_Y_AutoRescale_Before(theWindow) && !strcmp(label_Ptr->X_Label, "Time"))
    {
      for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
      {
	label_Ptr->Y_Max = max(label_Ptr->Y_Max, ptr->y[n-1]);
	label_Ptr->Y_Min = min(label_Ptr->Y_Min, ptr->y[n-1]);
      }
    }
    else
    {
      label_Ptr->Y_Max= label_Ptr->Y_Min= data_Ptr->y[0];
      for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
      {
	n= *(ptr->npoints);
	for (i=0; i<n; i++)
	{
	  label_Ptr->Y_Max = max(label_Ptr->Y_Max, ptr->y[i]);
	  label_Ptr->Y_Min = min(label_Ptr->Y_Min, ptr->y[i]);
	}
      }
      Y_AutoRescale_Before(theWindow);
    }
    label_Ptr->Y_Max *= label_Ptr->Y_Scale;
    label_Ptr->Y_Min *= label_Ptr->Y_Scale;
    
    if(Is_Semilog(theWindow)) label_Ptr->Y_Min = max(label_Ptr->Y_Min, label_Ptr->Y_Max -6.0);
  }

  /***************************/
  /* AutoRescaling the x-axis*/

  if (Is_X_AutoRescale(theWindow) && (n= *(theWindow->data->npoints)))
  {
    label_Ptr->X_Max /= label_Ptr->X_Scale;
    label_Ptr->X_Min /= label_Ptr->X_Scale;
    
    if(Is_X_AutoRescale_Before(theWindow) && !strcmp(label_Ptr->X_Label, "Time"))
    {
      for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
      {
	label_Ptr->X_Max = max(label_Ptr->X_Max, ptr->x[n-1]);
	label_Ptr->X_Min = min(label_Ptr->X_Min, ptr->x[n-1]);
      }
    }
    else
    {
      label_Ptr->X_Max= label_Ptr->X_Min= data_Ptr->x[0];
      for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
      {
	n= *(ptr->npoints);
	for (i=0; i<n; i++)
	{
	  label_Ptr->X_Max = max(label_Ptr->X_Max, ptr->x[i]);
	  label_Ptr->X_Min = min(label_Ptr->X_Min, ptr->x[i]);
	}
      }
      X_AutoRescale_Before(theWindow);
    }
    label_Ptr->X_Max *= label_Ptr->X_Scale;
    label_Ptr->X_Min *= label_Ptr->X_Scale;
  }

  if(Is_Semilog(theWindow))
  {
    label_Ptr->Y_Max= ceil(label_Ptr->Y_Max - DBL_MIN);
    label_Ptr->Y_Min= floor(label_Ptr->Y_Min);
  }

  /*******************************/
  /* Draw labels and plot border */
  
  if(!Was_Open_Before(theWindow) || Is_Y_AutoRescale(theWindow))
  {
    XClearArea(theDisplay, theWindow->xwin, 0, 0, 0, 0, False);
    if(Is_Semilog(theWindow))
    {
      sprintf(buffer1, "1E%.3G", label_Ptr->Y_Max);
      sprintf(buffer2, "1E%.3G", label_Ptr->Y_Min);
    }
    else
    {
      sprintf(buffer1, "%.3G", label_Ptr->Y_Max);
      sprintf(buffer2, "%.3G", label_Ptr->Y_Min);
    }

    i = strlen(buffer1);
    j = strlen(buffer2);
    
    k1 = XTextWidth(theFontStruct, buffer1, i);
    k2 = XTextWidth(theFontStruct, buffer2, j);
    maxlen= (k1 > k2) ? k1 : k2;
    
    x1= maxlen+3;
    x2= theWindow->width-3;
    y1= theFontHeight+20;
    y2= theWindow->height-(theFontAscent+theFontDescent+3);

    XSetForeground(theDisplay, theWindow->xwingc, theWinTextColor);
    XDrawString(theDisplay, theWindow->xwin, theWindow->xwingc, maxlen-k1+1, y1+theFontAscent, buffer1, i);
    XDrawString(theDisplay, theWindow->xwin, theWindow->xwingc, maxlen-k2+1, y2-2, buffer2, j);
     
    /**********************************************************/
    /* Drawing the boundary of the client area and tick marks */
   
    XSetForeground(theDisplay, theWindow->xwingc, theBorderColor);
    XDrawRectangle(theDisplay, theWindow->xwin, theWindow->xwingc, x1, y1, x2-x1, y2-y1);

    if(Is_Semilog(theWindow))
      for(i=1; i<label_Ptr->Y_Max -label_Ptr->Y_Min; i++)
      {
	Tick_Marks[0].x= x1-3;
	Tick_Marks[0].y= y1 +i*(y2-y1)/(label_Ptr->Y_Max -label_Ptr->Y_Min);
	Tick_Marks[1].x= x1;
	Tick_Marks[1].y= y1 +i*(y2-y1)/(label_Ptr->Y_Max -label_Ptr->Y_Min);
	
	XDrawLines(theDisplay, theWindow->xwin, theWindow->xwingc, Tick_Marks, 2, CoordModeOrigin);
      }
    else if(label_Ptr->Y_Max >0 && label_Ptr->Y_Min <0)
    {
      Tick_Marks[0].x= x1-3;
      Tick_Marks[0].y= y1 +label_Ptr->Y_Max*(y2-y1)/(label_Ptr->Y_Max -label_Ptr->Y_Min);
      Tick_Marks[1].x= x1;
      Tick_Marks[1].y= y1 +label_Ptr->Y_Max*(y2-y1)/(label_Ptr->Y_Max -label_Ptr->Y_Min);
	
      XDrawLines(theDisplay, theWindow->xwin, theWindow->xwingc, Tick_Marks, 2, CoordModeOrigin);
    }
    theWindow->ulxc = x1;
    theWindow->ulyc = y1;
    theWindow->lrxc = x2;
    theWindow->lryc = y2;
  }
  x1 = theWindow->ulxc+1;
  y1 = theWindow->ulyc+1;
  x2 = theWindow->lrxc-1;
  y2 = theWindow->lryc-1;

  if (!Was_Open_Before(theWindow) || Is_X_AutoRescale(theWindow) || Is_Y_AutoRescale(theWindow))
  {
    XClearArea(theDisplay, theWindow->xwin, 0, y2+2, x2, theWindow->height, False);
    
    sprintf(buffer1, "%lG", label_Ptr->X_Max);
    sprintf(buffer2, "%lG", label_Ptr->X_Min);

    i = strlen(buffer1);
    j = strlen(buffer2);
    
    maxlen= XTextWidth(theFontStruct, buffer1, i);
    
    XSetForeground(theDisplay, theWindow->xwingc, theWinTextColor);
    XDrawString(theDisplay, theWindow->xwin, theWindow->xwingc, x2-maxlen-1,
		y2+theFontAscent+2, buffer1, i);
    XDrawString(theDisplay, theWindow->xwin, theWindow->xwingc, x1, 
		y2+theFontAscent+2, buffer2, j);

    i = strlen(label_Ptr->X_Label);
    maxlen= XTextWidth(theFontStruct, label_Ptr->X_Label, i);
    XDrawString(theDisplay, theWindow->xwin, theWindow->xwingc, (x1+x2-maxlen)/2, 
		y2+theFontAscent+2, label_Ptr->X_Label, i);
  }

  if (!Was_Open_Before(theWindow) || Is_X_AutoRescale(theWindow) ||
      Is_Y_AutoRescale(theWindow) || !Is_Trace_On(theWindow))
  {
    XSetForeground(theDisplay, theWindow->xwingc, theBKGDColor);
    XFillRectangle(theDisplay, theWindow->xwin, theWindow->xwingc, x1, y1, x2-x1, y2-y1+1);
    Open_Before(theWindow);
  }
    
  if (label_Ptr->Y_Max == label_Ptr->Y_Min || label_Ptr->X_Max == label_Ptr->X_Min) return;

  /*******************************************/
  /* Plotting the graph(es) into the window. */

  theClientArea.x = x1;
  theClientArea.y = y1;
  theClientArea.width = x2-x1;
  theClientArea.height= y2-y1+1;
  XSetClipRectangles(theDisplay, theWindow->xwingc, 0, 0, &theClientArea, 1, Unsorted);
  
  m = (y1-y2)/(label_Ptr->Y_Max -label_Ptr->Y_Min +DBL_MIN)*label_Ptr->Y_Scale;
  b = (label_Ptr->Y_Max*y2 - label_Ptr->Y_Min*y1)/(label_Ptr->Y_Max - label_Ptr->Y_Min + DBL_MIN) + 0.5;
  m1= (x2-x1)/(label_Ptr->X_Max -label_Ptr->X_Min +DBL_MIN)*label_Ptr->X_Scale;
  b1= (label_Ptr->X_Max*x1 - label_Ptr->X_Min*x2)/(label_Ptr->X_Max - label_Ptr->X_Min + DBL_MIN) + 0.5;
  
  for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
  {
    XSetForeground(theDisplay, theWindow->xwingc, ptr->color);
    n = *(ptr->npoints);
    
    if (!(X_Points= (XPoint *)malloc(n*sizeof(XPoint)))) putchar(7);
    for (i=0; i<n; i++)
    {
      X_Points[i].x= m1*ptr->x[i]+b1;
      X_Points[i].y= m*ptr->y[i]+b;
    }

    if (Is_Semilog(theWindow) || Is_Linear(theWindow))
      XDrawLines(theDisplay, theWindow->xwin, theWindow->xwingc, X_Points, n, CoordModeOrigin);
    
    else if (Is_Scatter(theWindow))
      XDrawPoints(theDisplay, theWindow->xwin, theWindow->xwingc, X_Points, n, CoordModeOrigin);
    
    else
      printf("Unrecognized plot type in Paint_Window()");

    free(X_Points);
  }
  /**************************************************************/

  if(theOpenDialogBox && theOpenDialogBoxWindow == theWindow && Is_Semilog(theWindow))
  {
    label_Ptr->Y_Max= pow(10.0, label_Ptr->Y_Max);
    label_Ptr->Y_Min= pow(10.0, label_Ptr->Y_Min);
  }
}

/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/

#define PI      3.141592653589793
 
Paint_ThreeDWM_Window(theWindow)
WindowType theWindow;
{
  XY_DATA  data_Ptr, ptr;
  XY_LABEL label_Ptr;
  XPoint  **X_Points, thePolygon[5];
  XRectangle theClientArea;
  char buffer[20];

  int m, n,  i, j, k, x1, y1, x2, y2;
  double r, phi, theta, x, y, z;
  double sinphi, cosphi;
  double sintheta, costheta;
  double t[13], t1[13], ep[3], sp[3], rp[3], dp[3];
  double xmax, xmin, ymax, ymin;
  double m1, b1, m2, b2;
  double AX(), BY();

  data_Ptr = theWindow->data;
  label_Ptr= theWindow->label;

  /**************************************************************/
  /* Making the whole window drawable except the button region. */ 

  theClientArea.x = 0;
  theClientArea.y = theFontHeight+16;
  theClientArea.width = theWindow->width;
  theClientArea.height= theWindow->height;

  XSetForeground(theDisplay, theWindow->xwingc, theBlackPixel);
  XFillRectangle(theDisplay, theWindow->xwin, theWindow->xwingc,
		 0, theFontHeight+16, theWindow->width, theWindow->height);
  XSetClipRectangles(theDisplay, theWindow->xwingc, 0, 0, &theClientArea, 1, Unsorted);

  /****************************/
  /* AutoRescaling the z-axis */
  
  label_Ptr->Z_Max= label_Ptr->Z_Min= data_Ptr->z[0][0];
  for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
  {
    m= *(ptr->mpoints);
    n= *(ptr->npoints);
    for (i=0; i<m; i++)
      for (j=0; j<n; j++)
      {
	label_Ptr->Z_Max = max(label_Ptr->Z_Max, ptr->z[i][j]);
	label_Ptr->Z_Min = min(label_Ptr->Z_Min, ptr->z[i][j]);
      }
  }
  label_Ptr->Z_Max *= label_Ptr->Z_Scale;
  label_Ptr->Z_Min *= label_Ptr->Z_Scale;

  /****************************/
  /* AutoRescaling the y-axis */
  
  label_Ptr->Y_Max= label_Ptr->Y_Min= data_Ptr->y[0];
  for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
  {
    m= *(ptr->mpoints);
    n= *(ptr->npoints);
    label_Ptr->Y_Max = max(label_Ptr->Y_Max, ptr->y[n-1]);
  }
  label_Ptr->Y_Max *= label_Ptr->Y_Scale;
  label_Ptr->Y_Min *= label_Ptr->Y_Scale;
  
  /***************************/
  /* AutoRescaling the x-axis*/
  
  label_Ptr->X_Max= label_Ptr->X_Min= data_Ptr->x[0];
  for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
  {
    m= *(ptr->mpoints);
    n= *(ptr->npoints);
    label_Ptr->X_Max = max(label_Ptr->X_Max, ptr->x[n-1]);
  }
  label_Ptr->X_Max *= label_Ptr->X_Scale;
  label_Ptr->X_Min *= label_Ptr->X_Scale;

  /***************************/

  theta = PI/8;  phi = PI/8;
/*  sintheta = sin(theta);
  costheta = cos(theta);
  sinphi   = sin(phi);
  cosphi   = cos(phi);

  x = max(fabs(label_Ptr->X_Max), fabs(label_Ptr->X_Min));
  y = max(fabs(label_Ptr->Y_Max), fabs(label_Ptr->Y_Min));
  z = max(fabs(label_Ptr->Z_Max), fabs(label_Ptr->Z_Min));
  r = max(z/(fabs(costheta)), y/fabs(sintheta)/fabs(sinphi));
  r = max(r, x/fabs(sintheta)/fabs(cosphi));
  r *= 2;

  ep[0] = r*sintheta*cosphi;
  ep[1] = r*sintheta*sinphi;
  ep[2] = r*cosphi;	  
  
  sp[0] = (label_Ptr->X_Max +label_Ptr->X_Min)/2;
  sp[1] = (label_Ptr->Y_Max +label_Ptr->Y_Min)/2;
  sp[2] = (label_Ptr->Z_Max +label_Ptr->Z_Min)/2;
  
  init3dto2d(ep, sp, t); */
  
  /*******************************************/
  
  x1= 5;                   y1= theFontHeight+20;
  x2= theWindow->width-5;  y2= theWindow->height-5;
  
  /*******************************************/
  
  t1[0] = sin(theta);   t1[4] = y1;                t1[8] = label_Ptr->X_Min;
  t1[1] = cos(theta);   t1[5] = y2;                t1[9] = label_Ptr->X_Max;
  t1[2] = x1;           t1[6] = label_Ptr->Y_Min;  t1[10]= label_Ptr->Z_Min;
  t1[3] = x2;           t1[7] = label_Ptr->Y_Max;  t1[11]= label_Ptr->Z_Max;

  /*******************************************/

  for(ptr= data_Ptr; ptr != NULL; ptr= ptr->next)
  {
    m= *(ptr->mpoints);  n= *(ptr->npoints);
    X_Points= (XPoint **)malloc(m*sizeof(XPoint *));
    for (i=0; i<m; i++)  X_Points[i]= (XPoint *)malloc(n*sizeof(XPoint));
    
/*    for (i=0; i<m; i++)
      for (j=0; j<n; j++)
      {
	rp[0] = ptr->x[i]; rp[1] = ptr->y[j]; rp[2] = ptr->z[i][j];
	prespectify(rp, dp, t);
	project(&X_Points[i][j], dp, t[12]);
      }
 
    xmax = xmin = X_Points[0][0].x;  ymax = ymin = X_Points[0][0].y;
    for (i=0; i<m; i++)
      for (j=0; j<n; j++)
      {
	xmax = max(xmax, X_Points[i][j].x);  xmin = min(xmin, X_Points[i][j].x);
	ymax = max(ymax, X_Points[i][j].y);  ymin = min(ymin, X_Points[i][j].y);
      }
    m1 = (y1-y2)/(ymax -ymin);  b1 = (ymax*y2 - ymin*y1)/(ymax -ymin) + 0.5;
    m2 = (x2-x1)/(xmax -xmin);  b2 = (xmax*x1 - xmin*x2)/(xmax -xmin) + 0.5;

    for (i=0; i<m; i++)
      for (j=0; j<n; j++)
      {
	X_Points[i][j].x *= m2; X_Points[i][j].x += b2;
	X_Points[i][j].y *= m1; X_Points[i][j].y += b1;
      }

    XSetForeground(theDisplay, theWindow->xwingc, theWhitePixel);
    for (i=0; i<m-1; i++)
      for (j=0; j<n-1; j++)
      {
	k= 10+10*(ptr->z[i][j] -label_Ptr->Z_Min)/(label_Ptr->Z_Max -label_Ptr->Z_Min);
	if(k>17) k=17; 
	XSetForeground(theDisplay, theWindow->xwingc, Color[k]); 
	thePolygon[0].x = X_Points[i][j].x;     thePolygon[0].y = X_Points[i][j].y;
	thePolygon[1].x = X_Points[i+1][j].x;   thePolygon[1].y = X_Points[i+1][j].y;
	thePolygon[2].x = X_Points[i+1][j+1].x; thePolygon[2].y = X_Points[i+1][j+1].y;
	thePolygon[3].x = X_Points[i][j+1].x;   thePolygon[3].y = X_Points[i][j+1].y;
	thePolygon[4].x = X_Points[i][j].x;     thePolygon[4].y = X_Points[i][j].y;
	XFillPolygon(theDisplay, theWindow->xwin, theWindow->xwingc,
		     thePolygon, 5, Nonconvex, CoordModeOrigin);
      } */

   for (i=0; i<m-1; i++)
      for (j=0; j<n-1; j++)
      {
	k= 20 +13*(ptr->z[i][j] -label_Ptr->Z_Min)/(label_Ptr->Z_Max -label_Ptr->Z_Min);
	XSetForeground(theDisplay, theWindow->xwingc, Color[k]); 
	thePolygon[0].x = AX(ptr->x[i], ptr->y[j], ptr->z[i][j], t1); 
	thePolygon[0].y = BY(ptr->x[i], ptr->y[j], ptr->z[i][j], t1); 
	thePolygon[1].x = AX(ptr->x[i+1], ptr->y[j], ptr->z[i+1][j], t1); 
	thePolygon[1].y = BY(ptr->x[i+1], ptr->y[j], ptr->z[i+1][j], t1); 
	thePolygon[2].x = AX(ptr->x[i+1], ptr->y[j+1], ptr->z[i+1][j+1], t1); 
	thePolygon[2].y = BY(ptr->x[i+1], ptr->y[j+1], ptr->z[i+1][j+1], t1); 
	thePolygon[3].x = AX(ptr->x[i], ptr->y[j+1], ptr->z[i][j+1], t1); 
	thePolygon[3].y = BY(ptr->x[i], ptr->y[j+1], ptr->z[i][j+1], t1); 
	thePolygon[4].x = AX(ptr->x[i], ptr->y[j], ptr->z[i][j], t1); 
	thePolygon[4].y = BY(ptr->x[i], ptr->y[j], ptr->z[i][j], t1); 
	XFillPolygon(theDisplay, theWindow->xwin, theWindow->xwingc,
		     thePolygon, 5, Nonconvex, CoordModeOrigin);
      }

    /* for (i=0; i<m; i++)
       XDrawLines(theDisplay, theWindow->xwin, theWindow->xwingc,
		 X_Points[i], n, CoordModeOrigin); */
    
    for (i=0; i<m; i++)  free(X_Points[i]);
    free(X_Points); 
  }
printf("Here2!\n");
}

/****************************************************************/

init3dto2d(ep, sp, t)
double ep[3], sp[3], t[13];
{
  double dx, dy, dz, d;
  double cosx, cosy, cosz, sinz;
  double u, xp, yp,dp;
  double cosg, sing;
    
  dx= ep[0] -sp[0];
  dy= ep[1] -sp[1];

  dz= ep[2] -sp[2];
  d = sqrt(dx*dx +dy*dy +dz*dz);

  cosx = dx/d;
  cosy = dy/d;
  cosz = dz/d;
  sinz = sin(acos(cosz));

  u  = sqrt(cosy*cosy +cosz*cosz);  
  xp = -cosx*cosz/u;
  yp = -cosy/u;
  dp = sqrt(xp*xp +yp*yp);
  cosg = yp/dp;
  sing = xp/dp;

  t[0] = u*cosg;
  t[1] = (-cosx*cosy*cosg -sing*cosz)/u;
  t[2] = (-cosx*cosz*cosg +sing*cosy)/u;
  t[3] = cosg*(-u*sp[0] +cosx*sp[2]) +sing*sp[1];
  t[4] = u*sing;
  t[5] = (-cosx*cosy*sing +cosg*cosz)/u;
  t[6] = (-cosx*cosz*sing -cosg*cosy)/u;
  t[7] = sing*(-u*sp[0] +cosx*sp[2]) -cosg*sp[1];
  t[8] = cosx;
  t[9] = cosy;
  t[10]= cosz;
  t[11]= -cosg*sp[0] -u*sp[2];
  t[12]= d;
}

/****************************************************************/

prespectify(rp, dp, t)
double rp[3], dp[3], t[13];
{
  dp[0] = t[0]*rp[0] +t[1]*rp[1] +t[2]*rp[2] +t[3];
  dp[1] = t[4]*rp[0] +t[5]*rp[1] +t[6]*rp[2] +t[7];
  dp[2] = t[8]*rp[0] +t[9]*rp[1] +t[10]*rp[2]+t[11];
}

/****************************************************************/

project(xpoint, rp, t)
double rp[3], t;
XPoint *xpoint;
{
  double s;

  s = t/(t -rp[2]);
  xpoint->x = s*rp[0];
  xpoint->y = s*rp[1];
}

/***********************************/

double AX(x, y, z, t)
double x, y, z, t[13];
{
  double temp;
  
  temp = (y - t[6])/(t[7] - t[6]) - (x - t[8])/(t[9] - t[8])*t[1] + t[1];
  temp *= (t[3] - t[2])/(1 + t[1]);
  temp += t[2];
  return (temp);
}

/***********************************/

double BY(x, y, z, t)
double x, y, z, t[13];
{
  double temp;
  
  temp = (x - t[8])/(t[9] - t[8])*t[0] - (z - t[10])/(t[11] - t[10]) + 1;
  temp *= (t[5] - t[4])/(1 + t[0]);
  temp += t[4];
  return (temp);
}

/****************************************************************/
/****************************************************************/
/****************************************************************/

Paint_Button(theButton, Button_Flag)
ButtonType theButton;
int        Button_Flag;
{
  int        upper_shade, upper_mid_shade;
  int        lower_mid_shade, lower_shade;
  int        text_shade, bkgd_shade;
  int        width, height;
  XPoint     X_Points[6];
  XWindowAttributes theWindowAttributes;

  XGetWindowAttributes(theDisplay, theButton.xwin, &theWindowAttributes);
  width  = theWindowAttributes.width;
  height = theWindowAttributes.height;

  if(Button_Flag == BUTTON_NORMAL)
  {
    bkgd_shade  = theGrayShade7;  text_shade      = theBlackPixel;
    upper_shade = theGrayShade10; upper_mid_shade = theGrayShade9;
    lower_shade = theGrayShade1;  lower_mid_shade = theGrayShade4;
  }
  else
  {
    bkgd_shade  = theGrayShade5;  text_shade      = theWhitePixel;
    upper_shade = theGrayShade2;  upper_mid_shade = theGrayShade5;
    lower_shade = theGrayShade8;  lower_mid_shade = theGrayShade5;
  }
  XSetWindowBackground(theDisplay, theButton.xwin, bkgd_shade);
  XClearWindow(theDisplay, theButton.xwin);
  
  XSetForeground(theDisplay, theButton.xwingc, upper_shade);
  X_Points[0].x= 0;       X_Points[0].y= height;
  X_Points[1].x= 0;       X_Points[1].y= 0;
  X_Points[2].x= width;   X_Points[2].y= 0;
  X_Points[3].x= width-1; X_Points[3].y= 1;
  X_Points[4].x= 1;       X_Points[4].y= 1;
  X_Points[5].x= 1;       X_Points[5].y= height-1;
  XDrawLines(theDisplay, theButton.xwin, theButton.xwingc, X_Points, 6, CoordModeOrigin);

  XSetForeground(theDisplay, theButton.xwingc, upper_mid_shade);
  X_Points[0].x= 2;       X_Points[0].y= height-2;
  X_Points[1].x= 2;       X_Points[1].y= 2;
  X_Points[2].x= width-2; X_Points[2].y= 2;
  X_Points[3].x= width-3; X_Points[3].y= 3;
  X_Points[4].x= 3;       X_Points[4].y= 3;
  X_Points[5].x= 3;       X_Points[5].y= height-3;
  XDrawLines(theDisplay, theButton.xwin, theButton.xwingc, X_Points, 6, CoordModeOrigin);
  
  XSetForeground(theDisplay, theButton.xwingc, upper_mid_shade);
  X_Points[0].x= 4;       X_Points[0].y= height-4;
  X_Points[1].x= 4;       X_Points[1].y= 4;
  X_Points[2].x= width-4; X_Points[2].y= 4;
  X_Points[3].x= width-5; X_Points[3].y= 5;
  X_Points[4].x= 5;       X_Points[4].y= 5;
  X_Points[5].x= 5;       X_Points[5].y= height-5;
  XDrawLines(theDisplay, theButton.xwin, theButton.xwingc, X_Points, 6, CoordModeOrigin);

  XSetForeground(theDisplay, theButton.xwingc, lower_shade);
  X_Points[0].x= 0;       X_Points[0].y= height;
  X_Points[1].x= width;   X_Points[1].y= height;
  X_Points[2].x= width;   X_Points[2].y= 0;
  X_Points[3].x= width-1; X_Points[3].y= 1;
  X_Points[4].x= width-1; X_Points[4].y= height-1;
  X_Points[5].x= 1;       X_Points[5].y= height-1;
  XDrawLines(theDisplay, theButton.xwin, theButton.xwingc, X_Points, 6, CoordModeOrigin);
  
  XSetForeground(theDisplay, theButton.xwingc, lower_mid_shade);
  X_Points[0].x= 2;       X_Points[0].y= height-2;
  X_Points[1].x= width-2; X_Points[1].y= height-2;
  X_Points[2].x= width-2; X_Points[2].y= 2;
  X_Points[3].x= width-3; X_Points[3].y= 3;
  X_Points[4].x= width-3; X_Points[4].y= height-3;
  X_Points[5].x= 3;       X_Points[5].y= height-3;
  XDrawLines(theDisplay, theButton.xwin, theButton.xwingc, X_Points, 6, CoordModeOrigin);

  XSetForeground(theDisplay, theButton.xwingc, lower_mid_shade);
  X_Points[0].x= 4;       X_Points[0].y= height-4;
  X_Points[1].x= width-4; X_Points[1].y= height-4;
  X_Points[2].x= width-4; X_Points[2].y= 4;
  X_Points[3].x= width-5; X_Points[3].y= 5;
  X_Points[4].x= width-5; X_Points[4].y= height-5;
  X_Points[5].x= 5;       X_Points[5].y= height-5;
  XDrawLines(theDisplay, theButton.xwin, theButton.xwingc, X_Points, 6, CoordModeOrigin);

  if(theButton.Label)
  {
    XSetForeground(theDisplay, theButton.xwingc, text_shade);
    if(width > XTextWidth(theFontStruct, theButton.Label, strlen(theButton.Label)))
      XDrawString(theDisplay, theButton.xwin, theButton.xwingc,
		  (width - XTextWidth(theFontStruct, theButton.Label, strlen(theButton.Label)))/2,
		  theFontAscent+8, theButton.Label, strlen(theButton.Label));
    else
      XDrawString(theDisplay, theButton.xwin, theButton.xwingc,
		  width -XTextWidth(theFontStruct, theButton.Label, strlen(theButton.Label)),
		  theFontAscent+8, theButton.Label, strlen(theButton.Label));
  }
}

/****************************************************************/

Local_Window_Menu(theWindow, theButtonNumber)
WindowType theWindow;
int theButtonNumber;
{
  switch(theButtonNumber)
  {
     case 0:
       theOpenDialogBoxWindow = theWindow;
       RescaleWindow(); 
       break;

     case 1:
       if(Is_Trace_On(theWindow))
	 Trace_Off(theWindow); 
       else
	 Trace_On(theWindow);
       break;

     case 2:
       theOpenDialogBoxWindow = theWindow;
       PostScriptWindow();
       break;

     case 3:
       if(Is_Crosshair_On(theWindow))
	 DestroyCrosshair(theWindow);
       else
	 Init_Crosshair(theWindow);
       break;

     default:
       break;
  }
}

/****************************************************************/

Main_Menu(theButtonNumber)
int theButtonNumber;
{
  int i;
  
  switch(theButtonNumber)
  {
     case 1:
       if(!theOpenDialogBox && !theCrosshairedWindow) 
	 theRunFlag = True;
       break;

     case 2:
       if(!theOpenDialogBox && !theCrosshairedWindow) 
	 theRunFlag = False;
       break;

     case 3:
       if(!theOpenDialogBox && !theCrosshairedWindow)
       {
	 theRunFlag= False;
	 theStepFlag= True;
       }
       break;

     case 4:
       DialogBox(theDFDB);
       break;

     case 5:
       ExitMessage("Do you really want to exit?");
       break;

     default:
       break;
  }
}

/****************************************************************/

RescaleWindow()
{
  sprintf(theRescaleDB->msg, "Rescaling the %s window", theOpenDialogBoxWindow->label->Y_Label);
  if(Is_Semilog(theOpenDialogBoxWindow))
  {
    theOpenDialogBoxWindow->label->Y_Max= pow(10.0, theOpenDialogBoxWindow->label->Y_Max);
    theOpenDialogBoxWindow->label->Y_Min= pow(10.0, theOpenDialogBoxWindow->label->Y_Min);
  }
  theRescaleDB->button_list[0].ptr= (char *) &(theOpenDialogBoxWindow->label->Y_Max);
  theRescaleDB->button_list[1].ptr= (char *) &(theOpenDialogBoxWindow->label->Y_Min);
  theRescaleDB->button_list[2].ptr= (char *) &(theOpenDialogBoxWindow->state);
  theRescaleDB->button_list[3].ptr= (char *) &(theOpenDialogBoxWindow->label->X_Max);
  theRescaleDB->button_list[4].ptr= (char *) &(theOpenDialogBoxWindow->label->X_Min);
  theRescaleDB->button_list[5].ptr= (char *) &(theOpenDialogBoxWindow->state);
  
  DialogBox(theRescaleDB);
}

/****************************************************************/

PostScriptWindow()
{
  sprintf(thePSDB->msg, "Write the %s window into a PostScript file",
	  theOpenDialogBoxWindow->label->Y_Label);
  thePSDB->button_list[0].ptr= theOpenDialogBoxWindow->ps_file_name;
  thePSDB->button_list[1].ptr= theOpenDialogBoxWindow->plot_title;
  DialogBox(thePSDB);
}

/****************************************************************/

ExitMessage(Message)
char *Message;
{
  theMessageDB->button_list[0].name= "Exit   ";
  strcpy(theMessageDB->msg, Message);
  DialogBox(theMessageDB);
}

/****************************************************************/

Init_Crosshair(theWindow)
WindowType theWindow;
{
  XSetWindowAttributes  theSetWindowAttributes;
  XSizeHints            theSizeHints;
  unsigned long         theWindowMask;
  XWMHints              theWMHints;
  XGCValues             theGCValues;
  int                   i, x, y;

  if(theCrosshairedWindow)
  {
    printf("XGRAFIX: Can't Cross-hair more than one window at a time!!\n");
    printf(" This option should be available in the next version..\n");
    return;
  }

  if(!theOpenDialogBox)
  {
    thePrevRunFlag = theRunFlag;
    theRunFlag     = False;
  }
  theCrosshairedWindow = theWindow;
  Crosshair_On(theCrosshairedWindow);
  XDefineCursor(theDisplay, theWindow->xwin, theCrosshairCursor);
  
  XGetNormalHints(theDisplay, theWindow->xwin, &theSizeHints);

  if((x = theSizeHints.x -203) < 0)
    x= theSizeHints.x +theSizeHints.width +10;
  y= theSizeHints.y;

  /*******************************************************/
  /* Setting the xy cross-hair window's attributes and creating it  */
  
  theSetWindowAttributes.border_pixel     = theWhitePixel;
  theSetWindowAttributes.background_pixel = theBlackPixel;
  theWindowMask = (CWBackPixel | CWBorderPixel);

  theXYCrosshairWindow.xwin = XCreateWindow(theDisplay, theRootWindow, x, y,
					    200, 2*(theFontHeight+19), 2,
					    theDepth, InputOutput, CopyFromParent,
					    theWindowMask, &theSetWindowAttributes);
  
  /********************************************************/
  /* Setting the window manager's hints for the xy cross-hair window */
  
  theWMHints.initial_state = NormalState;
  theWMHints.icon_pixmap   = theIcon;
  theWMHints.flags         = IconPixmapHint | StateHint;
  
  XSetWMHints(theDisplay, theXYCrosshairWindow.xwin, &theWMHints);

  /*********************************/
  /* Setting the xy cross-hair window's name  */
  
  XStoreName(theDisplay, theXYCrosshairWindow.xwin, "Crosshair Coordinates");
  
  /****************************************/
  /* Setting the size and normal hints    */

  theSizeHints.flags  = USPosition | USSize;
  theSizeHints.x      = x;
  theSizeHints.y      = y;
  theSizeHints.width  = 200; 
  theSizeHints.height = 2*(theFontHeight+19);
  
  XSetNormalHints( theDisplay, theXYCrosshairWindow.xwin, &theSizeHints);
  
  XSelectInput(theDisplay, theXYCrosshairWindow.xwin,
	       StructureNotifyMask | ExposureMask | SubstructureNotifyMask);
  
  /********************************************/
  /* Setting up the buttons for the xy cross-hair window */
  
  if (!(theXYCrosshairWindow.xbutton = (ButtonType *)malloc(2*sizeof(ButtonType)))) putchar(7);
  
  strcpy(theXYCrosshairWindow.xbutton[0].Label, "X = 0.0");
  strcpy(theXYCrosshairWindow.xbutton[1].Label, "Y = 0.0");
  
  for(i=0; i< 2; i++)
  {
    theXYCrosshairWindow.xbutton[i].xwin  = XCreateSimpleWindow(theDisplay, theXYCrosshairWindow.xwin,
								0, i*(theFontHeight+19),
								198, theFontHeight+16, 1,
								theGrayShade5, theBlackPixel);
    
    XSelectInput(theDisplay, theXYCrosshairWindow.xbutton[i].xwin,
		 ButtonPressMask | ButtonReleaseMask | ExposureMask);
    
    theXYCrosshairWindow.xbutton[i].xwingc = XCreateGC(theDisplay,
						       theXYCrosshairWindow.xbutton[i].xwin,
						       (unsigned long) 0, &theGCValues);
    
    XSetFont(theDisplay, theXYCrosshairWindow.xbutton[i].xwingc, theFont);
  }
  SetCrosshairParameters();
  XMapSubwindows(theDisplay, theXYCrosshairWindow.xwin);
  XMapWindow(theDisplay, theXYCrosshairWindow.xwin);
}

/****************************************************************/

UpdateCrosshair(x, y)
int x, y;
{
  if(x <theCrosshairx1 || x >theCrosshairx2 ||
     y <theCrosshairy1 || y >theCrosshairy2)  return;
  
  /*****************************/
  /* Updating the Coordinates. */

  sprintf(theXYCrosshairWindow.xbutton[0].Label, "X = %lG", x*theCrosshairmx+theCrosshairbx);
  if(Is_Semilog(theCrosshairedWindow))
    sprintf(theXYCrosshairWindow.xbutton[1].Label, "Y = %lG", pow(10.0, y*theCrosshairmy+theCrosshairby));
  else
    sprintf(theXYCrosshairWindow.xbutton[1].Label, "Y = %lG", y*theCrosshairmy+theCrosshairby);
  
  Paint_Button(theXYCrosshairWindow.xbutton[0], BUTTON_PUSHED);
  Paint_Button(theXYCrosshairWindow.xbutton[1], BUTTON_PUSHED);
}

/****************************************************************/

SetCrosshairParameters()
{
  XY_LABEL label_Ptr;
  
  theCrosshairx1 = theCrosshairedWindow->ulxc+1;
  theCrosshairy1 = theCrosshairedWindow->ulyc+1;
  theCrosshairx2 = theCrosshairedWindow->lrxc-1;
  theCrosshairy2 = theCrosshairedWindow->lryc-1;
  
  label_Ptr= theCrosshairedWindow->label;
  theCrosshairmx = (label_Ptr->X_Max -label_Ptr->X_Min)/(theCrosshairx2 -theCrosshairx1);
  theCrosshairbx = (label_Ptr->X_Min*theCrosshairx2 
		    -label_Ptr->X_Max*theCrosshairx1)/(theCrosshairx2 -theCrosshairx1);
  theCrosshairmy = (label_Ptr->Y_Max -label_Ptr->Y_Min)/(theCrosshairy1 -theCrosshairy2);
  theCrosshairby = (label_Ptr->Y_Min*theCrosshairy1 
		    -label_Ptr->Y_Max*theCrosshairy2)/(theCrosshairy1 -theCrosshairy2);
}

/****************************************************************/

DestroyCrosshair(theWindow)
WindowType theWindow;
{
  int i;

  Crosshair_Off(theCrosshairedWindow);
  XUndefineCursor(theDisplay, theCrosshairedWindow->xwin);
  theCrosshairedWindow = NULL;

  if(!theOpenDialogBox)
    theRunFlag = thePrevRunFlag;
  
  for(i=0; i< 2; i++)
    XFreeGC(theDisplay, theXYCrosshairWindow.xbutton[i].xwingc);

  XDestroyWindow(theDisplay, theXYCrosshairWindow.xwin);
}

/****************************************************************/

Init_Rescale_DialogBox()
{  
  theRescaleDB = (DialogBoxType)malloc(sizeof(struct dialogbox));
  
  theRescaleDB->number = 8;
  theRescaleDB->name   = "Rescale Dialog Box";
  theRescaleDB->button_list= (DialogBoxItemType)malloc(theRescaleDB->number*sizeof(struct dialogbox_item));

  theRescaleDB->button_list[0].type= DOUBLE;
  theRescaleDB->button_list[0].name= "ymax";
  
  theRescaleDB->button_list[1].type= DOUBLE;
  theRescaleDB->button_list[1].name= "ymin";
  
  theRescaleDB->button_list[2].type= Y_AUTORESCALE;
  theRescaleDB->button_list[2].name= "Autorescale Y";
  
  theRescaleDB->button_list[3].type= DOUBLE;
  theRescaleDB->button_list[3].name= "xmax";
  
  theRescaleDB->button_list[4].type= DOUBLE;
  theRescaleDB->button_list[4].name= "xmin";
  
  theRescaleDB->button_list[5].type= X_AUTORESCALE;
  theRescaleDB->button_list[5].name= "Autorescale X";

  theRescaleDB->button_list[6].type= NULLTYPE;
  theRescaleDB->button_list[6].name= NULL;

  theRescaleDB->button_list[7].type= NULLTYPE;
  theRescaleDB->button_list[7].name= NULL;
}

/****************************************************************/

Init_DF_DialogBox()
{       
  theDFDB = (DialogBoxType)malloc(sizeof(struct dialogbox));	
    
  theDFDB->number = 3;
  theDFDB->name   = "Save Dialog Box";
  sprintf(theDFDB->msg, "Dumping the present state into a file");
  theDFDB->button_list= (DialogBoxItemType)malloc(theDFDB->number*sizeof(struct dialogbox_item));	
  
  theDFDB->button_list[0].type= CHAR;
  theDFDB->button_list[0].name= "File name";
  theDFDB->button_list[0].ptr= theDumpFile;

  theDFDB->button_list[1].type= NULLTYPE;
  theDFDB->button_list[1].name= NULL;

  theDFDB->button_list[2].type= NULLTYPE;
  theDFDB->button_list[2].name= NULL;
}

/****************************************************************/

Init_PS_DialogBox()
{  
  thePSDB = (DialogBoxType)malloc(sizeof(struct dialogbox));	
    
  thePSDB->number = 4;
  thePSDB->name   = "PostScript Dialog Box";
  thePSDB->button_list= (DialogBoxItemType)malloc(thePSDB->number*sizeof(struct dialogbox_item));	

  thePSDB->button_list[0].type= CHAR;
  thePSDB->button_list[0].name= "File name";
  thePSDB->button_list[0].ptr= thePSFile;

  thePSDB->button_list[1].type= CHAR;
  thePSDB->button_list[1].name= "Plot title";
  thePSDB->button_list[1].ptr= thePSFile;

  thePSDB->button_list[2].type= NULLTYPE;
  thePSDB->button_list[2].name= NULL;

  thePSDB->button_list[3].type= NULLTYPE;
  thePSDB->button_list[3].name= NULL;
}

/****************************************************************/

Init_Message_DialogBox()
{
  theMessageDB = (DialogBoxType)malloc(sizeof(struct dialogbox));	

  theMessageDB->number     = 2;
  theMessageDB->name       = "Message Dialog Box";
  theMessageDB->button_list= (DialogBoxItemType)malloc(theMessageDB->number*sizeof(struct dialogbox_item));

  theMessageDB->button_list[0].type= NULLTYPE;
  theMessageDB->button_list[0].name= NULL;

  theMessageDB->button_list[1].type= NULLTYPE;
  theMessageDB->button_list[1].name= NULL;
}

/****************************************************************/

DialogBox(theDialogBox)
DialogBoxType theDialogBox;
{
  XSetWindowAttributes  theWindowAttributes;
  XSizeHints            theSizeHints;
  unsigned long         theWindowMask;
  XWMHints              theWMHints;
  XGCValues             theGCValues;
  
  int         button_x, button_y, button_width, button_height;
  int         i, width=0, height;

  /****************************************************/
  /* Testing for previous instances of any dialog box */

  if(theOpenDialogBox) 
  {
    printf("DialogBox(): One dialog box is already open. Can't open another one!\n");
    return(-1);
  }

  theOpenDialogBox= theDialogBox;

  if(!theCrosshairedWindow)
  {
    thePrevRunFlag = theRunFlag;
    theRunFlag     = False;
  }

  /**************************************/
  /* Getting the size of the dialog box */
  
  for(i=0; i< theDialogBox->number; i++)
    if(theDialogBox->button_list[i].name)
      width= max(width, XTextWidth(theFontStruct, theDialogBox->button_list[i].name, 
				   strlen(theDialogBox->button_list[i].name)));
  
  button_x     = 10 +width +10;
  button_y     = theDialogBoxButtonLabelY -theFontAscent-10;
  button_width = 200;
  button_height= theFontHeight +16;
  
  theDialogBox->width = max(width+40+button_width, 
			    20+XTextWidth(theFontStruct, theDialogBox->msg, 
					  strlen(theDialogBox->msg)));
  height= button_y +theDialogBox->number*theDialogBoxButtonSpacing;
  
  /********************************************/
  /* Setting the Attributes of the dialog box */
  
  theWindowAttributes.border_pixel     = theWhitePixel;
  theWindowAttributes.background_pixel = theGrayShade3;
  theWindowMask = (CWBackPixel | CWBorderPixel);
  
  theDialogBox->xwin = XCreateWindow(theDisplay, theRootWindow, DEFAULT_DIALOGBOX_X,
				     DEFAULT_DIALOGBOX_Y, theDialogBox->width, height,
				     DEFAULT_WIN_BORDER_WIDTH, theDepth,
				     InputOutput, CopyFromParent,
				     theWindowMask, &theWindowAttributes);
  
  /*********************************************************/
  /* Setting the window manager's hints for the dialog box */
  
  theWMHints.initial_state = NormalState;
  theWMHints.icon_pixmap   = theIcon;
  theWMHints.flags         = IconPixmapHint | StateHint;
  
  XSetWMHints(theDisplay, theDialogBox->xwin, &theWMHints);
  XStoreName(theDisplay, theDialogBox->xwin, theDialogBox->name);
 
  /********************************************************/
  /* Setting the size and normal hints for the dialog box */

  theSizeHints.flags  = USPosition | USSize;
  theSizeHints.x      = DEFAULT_DIALOGBOX_X;
  theSizeHints.y      = DEFAULT_DIALOGBOX_Y;
  theSizeHints.width  = theDialogBox->width;
  theSizeHints.height = height;
  
  XSetNormalHints( theDisplay, theDialogBox->xwin, &theSizeHints);
  
  /****************************************/
  /* Initializing a GC for the dialog box */
  
  theDialogBox->xwingc = XCreateGC(theDisplay, theDialogBox->xwin,
				   (unsigned long) 0, &theGCValues);
  if (theDialogBox->xwingc)
  {
    XSetForeground(theDisplay, theDialogBox->xwingc, theWhitePixel);
    XSetBackground(theDisplay, theDialogBox->xwingc, theBlackPixel);
  }
  else
  {
    printf("XGRAFIX: can't create a GC for the Dialogbox");
    XDestroyWindow(theDisplay, theDialogBox->xwin);
  }
  XSetFont(theDisplay, theDialogBox->xwingc, theFont);
  XSelectInput(theDisplay, theDialogBox->xwin,
	       StructureNotifyMask | ExposureMask | SubstructureNotifyMask);
  
  /**************************************/
  /* defining buttons of the dialog box */
 
  for(i=0; i< theDialogBox->number; i++)
  {
    if(theDialogBox->button_list[i].type == DOUBLE)
      sprintf(theDialogBox->button_list[i].xbutton.Label, "%lG",
	      *((double *)theDialogBox->button_list[i].ptr)); 
    
    else if(theDialogBox->button_list[i].type == INTEGER)
      sprintf(theDialogBox->button_list[i].xbutton.Label, "%d",
	      *((int *)theDialogBox->button_list[i].ptr)); 
    
    else if(theDialogBox->button_list[i].type == CHAR)
      sprintf(theDialogBox->button_list[i].xbutton.Label, "%s",
	      theDialogBox->button_list[i].ptr); 
    
    else if(theDialogBox->button_list[i].type%2 == 0 && i< theDialogBox->number-2)
    {
      if(theDialogBox->button_list[i].type & *((int *)theDialogBox->button_list[i].ptr))
	sprintf(theDialogBox->button_list[i].xbutton.Label, "%s", "ON"); 
      else
	sprintf(theDialogBox->button_list[i].xbutton.Label, "%s", "OFF"); 
    }
    else if(i == theDialogBox->number-2)
      sprintf(theDialogBox->button_list[i].xbutton.Label, "%s", "OK"); 
    else if(i == theDialogBox->number-1)
      sprintf(theDialogBox->button_list[i].xbutton.Label, "%s", "CANCEL"); 
    
    theDialogBox->button_list[i].xbutton.xwin  = XCreateSimpleWindow(theDisplay, theDialogBox->xwin, button_x,
								     button_y +i*theDialogBoxButtonSpacing,
								     button_width, button_height,
								     2, theGrayShade5, theBlackPixel);
    
    XSelectInput(theDisplay, theDialogBox->button_list[i].xbutton.xwin,
		 ButtonPressMask | ButtonReleaseMask | ExposureMask | KeyPressMask | KeyReleaseMask);
    
    theDialogBox->button_list[i].xbutton.xwingc = XCreateGC(theDisplay,
							    theDialogBox->button_list[i].xbutton.xwin,
							    (unsigned long) 0, &theGCValues);
    
    XSetFont(theDisplay, theDialogBox->button_list[i].xbutton.xwingc, theFont);
    XDefineCursor(theDisplay, theDialogBox->button_list[i].xbutton.xwin, theButtonCursor);
  }
 
  theDialogBox->on_button= -1;
  /**********************************************/
  /* Mapping the Window and dumping stuff in it */
  
  XMapSubwindows(theDisplay, theDialogBox->xwin);
  XMapWindow(theDisplay, theDialogBox->xwin);
}

/*****************************************************************/

Paint_DialogBox(theDialogBox)
DialogBoxType theDialogBox;
{
  int i, msg_length;
  
  XSetForeground(theDisplay, theDialogBox->xwingc, theDBTextColor);
  msg_length= XTextWidth(theFontStruct, theDialogBox->msg, strlen(theDialogBox->msg));
  XDrawString(theDisplay, theDialogBox->xwin, theDialogBox->xwingc, (theDialogBox->width -msg_length)/2, 
	      theFontHeight+6, theDialogBox->msg, strlen(theDialogBox->msg));
  
  for(i=0; i< theDialogBox->number; i++)
    if(theDialogBox->button_list[i].name)
      XDrawString(theDisplay, theDialogBox->xwin, theDialogBox->xwingc,
		  theDialogBoxButtonLabelX, theDialogBoxButtonLabelY +i*theDialogBoxButtonSpacing,
		  theDialogBox->button_list[i].name, strlen(theDialogBox->button_list[i].name));
}

/****************************************************************/

DialogBox_Menu(theDialogBox)
DialogBoxType theDialogBox;
{
  int i, theButtonNumber;
  
  theButtonNumber= theDialogBox->on_button;
  
  if(theButtonNumber == theDialogBox->number-2)          /* OK button */
  {
    for(i=0; i< theDialogBox->number-2; i++)
    {
      if(theDialogBox->button_list[i].type == DOUBLE)
	sscanf(theDialogBox->button_list[i].xbutton.Label, "%lG", (double *)theDialogBox->button_list[i].ptr);
      
      else if(theDialogBox->button_list[i].type == INTEGER)
	sscanf(theDialogBox->button_list[i].xbutton.Label, "%d", (int *)theDialogBox->button_list[i].ptr);
      
      else if(theDialogBox->button_list[i].type == CHAR)
	strcpy(theDialogBox->button_list[i].ptr, theDialogBox->button_list[i].xbutton.Label);
      
      else if(theDialogBox->button_list[i].type%2 == 0)
      { 
	if(!strcmp(theDialogBox->button_list[i].xbutton.Label, "ON"))
	  *((int *)theDialogBox->button_list[i].ptr) |= theDialogBox->button_list[i].type;
	else
	  *((int *)theDialogBox->button_list[i].ptr) &= ~(theDialogBox->button_list[i].type);
      }
    }
    DispatchMessage(theDialogBox);
    DestroyDialogBox(theDialogBox);
    theDialogBox->on_button= -1;
  }
  else if(theButtonNumber == theDialogBox->number-1)     /* CANCEL button */
  {
    DestroyDialogBox(theDialogBox);
    theDialogBox->on_button= -1;
  }
  else if(theDialogBox->button_list[theButtonNumber].type%2 == 0)
  {
    if(!strcmp(theDialogBox->button_list[theButtonNumber].xbutton.Label, "ON"))
      sprintf(theDialogBox->button_list[theButtonNumber].xbutton.Label, "%s", "OFF"); 
    else
      sprintf(theDialogBox->button_list[theButtonNumber].xbutton.Label, "%s", "ON"); 
    Paint_Button(theOpenDialogBox->button_list[theButtonNumber].xbutton, BUTTON_NORMAL);
    theDialogBox->on_button= -1;
  }
}

/****************************************************************/

DispatchMessage(theDialogBox)
DialogBoxType theDialogBox; 
{
  if(!strcmp(theDialogBox->name, "Rescale Dialog Box"))
  {    
    if(Is_Y_AutoRescale_Before(theOpenDialogBoxWindow) &&
       !Is_Y_AutoRescale(theOpenDialogBoxWindow))
      Y_AutoRescale_Off_Before(theOpenDialogBoxWindow);
    
    if(Is_X_AutoRescale_Before(theOpenDialogBoxWindow) &&
       !Is_X_AutoRescale(theOpenDialogBoxWindow))
      X_AutoRescale_Off_Before(theOpenDialogBoxWindow);
    if(Is_Scatter(theOpenDialogBoxWindow) || Is_Linear(theOpenDialogBoxWindow)
       || Is_ThreeDWM(theOpenDialogBoxWindow))
    {
      Iconic_Before(theOpenDialogBoxWindow);
      (*(theOpenDialogBoxWindow->paint_function))(theOpenDialogBoxWindow);
    }
  }
  
  else if(!strcmp(theDialogBox->name, "Save Dialog Box"))
    Dump(theDumpFile);

  else if(!strcmp(theDialogBox->name, "PostScript Dialog Box"))
    (*(theOpenDialogBoxWindow->print_function))();
 
  else if(!strcmp(theDialogBox->name, "Message Dialog Box"))
    exit(1);
}

/****************************************************************/

DestroyDialogBox(theDialogBox)
DialogBoxType theDialogBox;
{
  int i;
  
  if(!strcmp(theDialogBox->name, "Rescale Dialog Box") &&
     Is_Semilog(theOpenDialogBoxWindow))
  {
    theOpenDialogBox= NULL;
    theOpenDialogBoxWindow->label->Y_Max= log10(theOpenDialogBoxWindow->label->Y_Max +DBL_MIN);
    theOpenDialogBoxWindow->label->Y_Min= log10(theOpenDialogBoxWindow->label->Y_Min +DBL_MIN);
   
    Iconic_Before(theOpenDialogBoxWindow);
    (*(theOpenDialogBoxWindow->paint_function))(theOpenDialogBoxWindow);
  }

  theOpenDialogBox= NULL;
  theOpenDialogBoxWindow = NULL;
  
  if(!theCrosshairedWindow)
    theRunFlag = thePrevRunFlag;
  
  for(i=0; i< theDialogBox->number; i++)
    XFreeGC(theDisplay,  theDialogBox->button_list[i].xbutton.xwingc);
  
  XFreeGC(theDisplay,  theDialogBox->xwingc);
  XDestroyWindow(theDisplay, theDialogBox->xwin);
}

/****************************************************************/

ReadFromKeyborad(theXKeyEvent)
XKeyEvent *theXKeyEvent;
{
  char   theChar;
  int    i, theType;
  KeySym theKeySym;
  
  theType = theOpenDialogBox->button_list[theOpenDialogBox->on_button].type;
  if(!theType) return;
  theChar =0;
  XLookupString(theXKeyEvent, &theChar, 1, &theKeySym, NULL);
  i= strlen(theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.Label);

  switch(theKeySym)
  {
    case XK_BackSpace:
    case XK_Delete:
      if(i) theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.Label[i-1]=0;
      Paint_Button(theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton, BUTTON_PUSHED);
      break;
    
    case XK_Return:
      Paint_Button(theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton, BUTTON_NORMAL);
      theOpenDialogBox->on_button= -1;
      break; 
      
    default:
      if(i<MAX_BUFFER_SIZE-1)
      {
	if(theType == DOUBLE &&
	   ((theChar>=48 && theChar<= 57) || theChar== 'e' ||
	    theChar== 'E' || theChar== '.' || theChar== '-' || theChar== '+'))
	  theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.Label[i]= theChar;
	
	else if(theType == INTEGER &&
		((theChar>=48 && theChar<= 57) || theChar== '-' || theChar== '+'))
	  theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.Label[i]= theChar;
	
	else if(theType == CHAR &&
		theChar >= 32 && theChar <= 125)
	  theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.Label[i]= theChar;

	theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton.Label[i+1]= 0;
	Paint_Button(theOpenDialogBox->button_list[theOpenDialogBox->on_button].xbutton, BUTTON_PUSHED);
      }
      break;
    }
}      

/****************************************************************/
/* 
IncFileName(FileName)
char *FileName;
{
  int i, j, dotindex, zeroindex, success_flag;
  
  success_flag = FALSE;
  i = dotindex = 0;
  while (FileName[i])
  {
    if (FileName[i]=='.') dotindex = i;
    i++;
  }
  
  zeroindex = i;
  j = dotindex;
  if (isdigit(FileName[--j]))
  {
    while (isdigit(FileName[j]) && j>=0)
    {
      if (FileName[j] == '9') FileName[j] = '0';
      else
      {
	FileName[j]++;
	success_flag = TRUE;
	break;
      }
      j--;
    }
    if (!success_flag)
      if (dotindex<8)
      {
	for (i=zeroindex; i>=dotindex; i--) FileName[i+1] = FileName[i];
	FileName[++i] = '0';
	FileName[++j] = '1';
      }
      else FileName[j] = '1';
  }
  else if (dotindex<8)
  {
    for (j=zeroindex; j>=dotindex; j--) FileName[j+1] = FileName[j];
    FileName[++j] = '0';
  }
  else FileName[dotindex-1] = '0';
}
*/
/****************************************************************/

#define	NPOINTMAX   500
FILE *fp;

PostScriptOpenWindow()
{
  XY_DATA  data_Ptr, ptr;
  XY_LABEL label_Ptr;
  int      i, j, n, nplot;
  int      width, height;
  
  if(!(fp= fopen(theOpenDialogBoxWindow->ps_file_name, "w"))) 
  {
    printf("PostScriptOpenWindow(): File was not opened properly!");
    return;
  }
  PSHeader();

  /*****************************/
  data_Ptr = theOpenDialogBoxWindow->data;
  label_Ptr= theOpenDialogBoxWindow->label;
  width    = theOpenDialogBoxWindow->width;
  height   = theOpenDialogBoxWindow->height;

  fprintf(fp, "612 0 translate\n");
  fprintf(fp, "90 rotate\n");
  
  fprintf(fp, "1 setlinewidth\n");
  fprintf(fp, "0 setgray\n");
  fprintf(fp, "1 %d %d %d %d rectangle\n", 0, 0, width, height);
  
  /************************/
  /* Printing the labels */

  fprintf(fp, "90 rotate\n");
  fprintf(fp, "%/Times-BoldItalic findfont 12 scalefont setfont\n");
  fprintf(fp, "306 -100 moveto\n");
  fprintf(fp, "(%s) prtstrfrommidlow\n", label_Ptr->Y_Label);
  fprintf(fp, "-90 rotate\n");
  
  fprintf(fp, "396 92 moveto\n");
  fprintf(fp, "(%s) prtstrfrommidup\n", label_Ptr->X_Label);

  fprintf(fp, "%/Times-Bold findfont 12 scalefont setfont\n");
  fprintf(fp, "108 92 moveto\n");
  fprintf(fp, "(%.3G) prtstrfromlftup\n", label_Ptr->X_Min);
  fprintf(fp, "684 92 moveto\n");
  fprintf(fp, "(%.3G) prtstrfromritup\n", label_Ptr->X_Max);

  fprintf(fp, "396 512 moveto\n");
  fprintf(fp, "%/Times-Bold findfont 14 scalefont setfont\n");
  fprintf(fp, "(%s) prtstrfrommidlow\n", theOpenDialogBoxWindow->plot_title);

  if(label_Ptr->Y_Max == label_Ptr->Y_Min || label_Ptr->X_Max == label_Ptr->X_Min) return;
  
  fprintf(fp, "%/Times-Bold findfont 12 scalefont setfont\n");
  if(Is_Semilog(theOpenDialogBoxWindow))
  {
    fprintf(fp, "100 495 moveto\n");
    fprintf(fp, "(1E%.3G) prtstrfromritup\n", label_Ptr->Y_Max);
    fprintf(fp, "100 108 moveto\n");
    fprintf(fp, "(1E%.3G) prtstrfromritlow\n", label_Ptr->Y_Min);
  }
  else
  {
    fprintf(fp, "100 495 moveto\n");
    fprintf(fp, "(%.3G) prtstrfromritup\n", label_Ptr->Y_Max);
    fprintf(fp, "100 108 moveto\n");
    fprintf(fp, "(%.3G) prtstrfromritlow\n", label_Ptr->Y_Min);
  }
  
  fprintf(fp, "%d %d %d %d clientarea\n", 0, 0, width, height);
  
  fprintf(fp, "gsave 0 setgray .8 setlinewidth\n");
  fprintf(fp, "%/dtick {y2 y1 sub %lG %lG sub div} def\n", label_Ptr->Y_Max, label_Ptr->Y_Min); 

  if(Is_Semilog(theOpenDialogBoxWindow))
  {
    for(i=1; i< label_Ptr->Y_Max -label_Ptr->Y_Min; i++)
    {
      fprintf(fp, "x1 y1 %d dtick mul add mv\n", i);
      fprintf(fp, "x1 4 add y1 %d dtick mul add ln\n", i);
      
      fprintf(fp, "x2 4 sub y1 %d dtick mul add mv\n", i);
      fprintf(fp, "x2 y1 %d dtick mul add ln\n", i);
    }
  }
  else if(label_Ptr->Y_Max >0 && label_Ptr->Y_Min <0 )
  {
    fprintf(fp, "x1 y1 %lG dtick mul add mv\n", label_Ptr->Y_Max);
    fprintf(fp, "x1 4 add y1 %lG dtick mul add ln\n", label_Ptr->Y_Max);
    
    fprintf(fp, "x2 4 sub y1 %lG dtick mul add mv\n", label_Ptr->Y_Max);
    fprintf(fp, "x2 y1 %lG dtick mul add ln\n", label_Ptr->Y_Max);
  }
  
  fprintf(fp, "stroke grestore\n\n");
  
  fprintf(fp, "gsave boundary clip\n");
  fprintf(fp, "%/x1 x1 1 add def %/y1 y1 1 add def\n"); 
  fprintf(fp, "%/x2 x2 1 sub def %/y2 y2 1 sub def\n");
  
  fprintf(fp, "%/b {%lG y2 mul %lG y1 mul sub\n", label_Ptr->Y_Max, label_Ptr->Y_Min);
  fprintf(fp, " %lG %lG sub div .5 add} def\n", label_Ptr->Y_Max, label_Ptr->Y_Min);
  
  fprintf(fp, "%/b1 {%lG x1 mul %lG x2 mul sub\n", label_Ptr->X_Max, label_Ptr->X_Min);
  fprintf(fp, " %lG %lG sub div .5 add} def\n", label_Ptr->X_Max, label_Ptr->X_Min);

  fprintf(fp, "%/m {y1 y2 sub %lG %lG sub div %lG mul} def\n",
	  label_Ptr->Y_Max, label_Ptr->Y_Min, label_Ptr->Y_Scale);
  fprintf(fp, "%/m1 {x2 x1 sub %lG %lG sub div %lG mul} def\n",
	  label_Ptr->X_Max, label_Ptr->X_Min, label_Ptr->X_Scale);
  
  fprintf(fp, ".25 setlinewidth\n\n");
  for(nplot=0, ptr= data_Ptr; ptr != NULL; ptr= ptr->next) nplot++;
  
  for(ptr= data_Ptr, j=0; ptr != NULL; ptr= ptr->next, j++)
  {
    n = *(ptr->npoints);
    if(Is_Semilog(theOpenDialogBoxWindow) || Is_Linear(theOpenDialogBoxWindow))
    {
      fprintf(fp, "%.3G setgray\n", ((double)j)/nplot);
      fprintf(fp, "newpath m1 %lG mul b1 add m %lG mul b add mv\n", ptr->x[0], ptr->y[0]);
      for (i=1; i< n; i++)
      {
	fprintf(fp, "m1 %lG mul b1 add m %lG mul b add ln\n", ptr->x[i], ptr->y[i]);
	if(!(i%NPOINTMAX))
	{
	  fprintf(fp, "stroke\n\n");
	  fprintf(fp, "newpath m1 %lG mul b1 add m %lG mul b add mv\n", ptr->x[i], ptr->y[i]);
	}
      }
      fprintf(fp, "stroke\n\n");
    }
    else if(Is_Scatter(theOpenDialogBoxWindow))
    {
      fprintf(fp, "0 setgray\n");
      fprintf(fp, "%/Times-Bold findfont 6 scalefont setfont\n");
      fprintf(fp, "newpath\n");
      for(i=0; i< n; i++)
	fprintf(fp, "m1 %lG mul b1 add m %lG mul b add mv sp%d\n", ptr->x[i], ptr->y[i], j);
      fprintf(fp, "stroke\n\n");
    }
    else
      printf("Unrecognized plot type in PSWindow()");
  }

  fprintf(fp, "grestore\n\n");
  fprintf(fp, "showpage\n\n");
  if(fclose(fp)) printf("PostScriptOpenWindow(): File was not closed properly!");
}

/****************************************************************/

PSHeader()
{
  /*****************************/
  /* Loading the dictionary... */
  
  fprintf(fp, "%%!\n");
  fprintf(fp, "%/offset {108 add} def\n");
  fprintf(fp, "%/yscale {%d exch sub %lG mul offset} def\n", theOpenDialogBoxWindow->height,
	  396.0/theOpenDialogBoxWindow->height);

  fprintf(fp, "%/xscale {%lG mul offset} def\n\n", 576.0/theOpenDialogBoxWindow->width);
  fprintf(fp, "%/mv {yscale exch xscale exch moveto} def\n");
  fprintf(fp, "%/ln {yscale exch xscale exch lineto} def\n\n");
  
  fprintf(fp, "%/prtstrfrommidlow {dup stringwidth pop\n");
  fprintf(fp, " 2 div currentpoint pop exch sub currentpoint\n");
  fprintf(fp, " exch pop moveto show} def\n\n");

  fprintf(fp, "%/prtstrfrommidup {dup stringwidth currentpoint\n");
  fprintf(fp, " 3 -1 roll sub exch 3 -1 roll 2 div sub exch\n");
  fprintf(fp, " moveto show} def\n\n");
  
  fprintf(fp, "%/prtstrfromritlow {dup stringwidth pop\n");
  fprintf(fp, " currentpoint pop exch sub currentpoint\n");
  fprintf(fp, " exch pop moveto show} def\n\n");

  fprintf(fp, "%/prtstrfromritup {dup stringwidth currentpoint\n");
  fprintf(fp, " 3 -1 roll sub exch 3 -1 roll sub exch\n");
  fprintf(fp, " moveto show} def\n\n");

  fprintf(fp, "%/prtstrfromlftup {dup stringwidth exch pop\n");
  fprintf(fp, " currentpoint 3 -1 roll sub moveto show} def\n\n");
   
  fprintf(fp, "%/rectangle {4 -1 roll 4 -1 roll\n");
  fprintf(fp, " dup 5 1 roll exch dup 6 1 roll exch\n");
  fprintf(fp, " mv 3 1 roll dup 5 1 roll exch ln dup\n"); 
  fprintf(fp, " 4 -1 roll exch ln ln closepath gsave setgray\n");
  fprintf(fp, " fill grestore stroke} def\n\n");
  
  fprintf(fp, "%/clientarea { %/y2 exch def %/x2 exch def\n");
  fprintf(fp, " %/y1 exch def %/x1 exch def} def\n\n");
  
  fprintf(fp, "%/boundary {newpath x1 y1 mv x2 y1 ln\n");
  fprintf(fp, " x2 y2 ln x1 y2 ln closepath} def\n\n");
  
  fprintf(fp, "%/sp0 {(.) prtstrfrommidlow} def\n");
  fprintf(fp, "%/sp1 {(o) prtstrfrommidlow} def\n");
  fprintf(fp, "%/sp2 {-1 -1 rmoveto 0 2 rlineto\n");
  fprintf(fp, " 2 0 rlineto 0 -2 rlineto closepath} def\n");
  fprintf(fp, "%/sp3 {(*) prtstrfrommidlow} def\n");
  fprintf(fp, "%/sp4 {(x) prtstrfrommidlow} def\n");
  fprintf(fp, "%/sp5 {(+) prtstrfrommidlow} def\n");
  fprintf(fp, "%/sp6 {(@) prtstrfrommidlow} def\n\n");
  
  /****************************************************/
  /* Now write the time, file name and the timestep.. */
  
}

/****************************************************************/


