/************************/
/* Including the C libs */

#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <time.h>
#include <math.h>
/* #include <sys/file.h> */

/************************/
/* Including the X libs */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/cursorfont.h>

/********************************************/
/* Including the XGRAFIX bitmap and strings */

#include "xgrafix.ico"
#include "xgrafix.str"

/**********************************************/
/* Definition of constants used in XGRAFIX    */

#define    TRUE                             1
#define    FALSE                            0

#define    DEFAULT_WIN_HEIGHT             300
#define    DEFAULT_WIN_WIDTH              400
#define    DEFAULT_WIN_BORDER_WIDTH         2

#define    DEFAULT_MAIN_MENU_X             10
#define    DEFAULT_MAIN_MENU_Y             40
#define    DEFAULT_MAIN_MENU_WIDTH        200

#define    DEFAULT_DIALOGBOX_X            400
#define    DEFAULT_DIALOGBOX_Y            400

#define	   MAX_MAIN_MENU_BUTTON		    6
#define	   MAX_WIN_BUTTON		    4
#define    MAX_BUFFER_SIZE                 40

#define    NULLTYPE                         0
#define	   DOUBLE			    3
#define	   INTEGER			    5
#define	   CHAR				    7

#define	   OPEN				    1
#define	   OPEN_BEFORE			    2
#define	   TRACE			    4
#define    SCATTERED                        0
#define    LINEAR                           8
#define    SEMILOG                         16
#define    THREED_WM                       24
#define    Y_AUTORESCALE		   32
#define    X_AUTORESCALE		   64
#define    Y_AUTORESCALE_BEFORE	          128
#define    X_AUTORESCALE_BEFORE	          256
#define    CROSSHAIR                      512

#define    OPEN_STATE                       0
#define    ICONIC_STATE                     1

#define    BUTTON_NORMAL                    0
#define    BUTTON_PUSHED                    1

#ifndef    DBL_MIN
#define	   DBL_MIN		       1E-200
#endif

/**********************************************/
/* Definition of structures used in XGRAFIX   */

typedef struct xy_data *XY_DATA;
struct xy_data {
  double **z;
  double  *y;
  double  *x;
  int     *mpoints;
  int     *npoints;
  int      color;
  XY_DATA  next;
}; 

typedef struct xy_label *XY_LABEL;
struct xy_label {
  double  Z_Scale;
  double  Z_Min;
  double  Z_Max;
  char   *Z_Label;
  int     Z_Auto_Rescale;
  double  Y_Scale;
  double  Y_Min;
  double  Y_Max;
  char   *Y_Label;
  int     Y_Auto_Rescale;
  double  X_Scale;
  double  X_Min;
  double  X_Max;
  char   *X_Label;
  int     X_Auto_Rescale;
};

typedef struct win_button ButtonType;
struct win_button {
  Window  xwin;
  GC      xwingc;
  char    Label[MAX_BUFFER_SIZE];
};

typedef struct window_struct *WindowType;
struct window_struct {
  int        height;
  int        width;
  int        ulxc;
  int        ulyc;
  int        lrxc;
  int        lryc;
  int        state;
  XY_DATA    data;
  XY_LABEL   label;
  char       ps_file_name[MAX_BUFFER_SIZE];
  char       plot_title[MAX_BUFFER_SIZE];
  ButtonType xbutton[MAX_WIN_BUTTON];
  Window     xwin;
  GC         xwingc;
  int       (*paint_function)();
  int       (*print_function)();
  WindowType next;
};

typedef struct menu_struct MenuType;
struct menu_struct {
  Window      xwin;
  ButtonType *xbutton;
};

typedef struct dialogbox_item {
  int         type;
  char       *name;
  char       *ptr;
  ButtonType  xbutton;
} *DialogBoxItemType;

typedef struct dialogbox {
  char             *name;
  char              msg[100];
  int               width;
  int               on_button;
  Window            xwin;
  GC                xwingc;
  int               number;
  DialogBoxItemType button_list;
} *DialogBoxType;

/********************************************/
/********************************************/
/* Global variables used in XGRAFIX         */
/* But first check if this is the origin    */
/* for the global variables or just an      */
/* external reference.                      */

#if defined(GLOBALORIGIN)
#define GLOBAL
#else
#define GLOBAL  extern
#endif

GLOBAL double       *theTimeStep;
GLOBAL int           theRunFlag;
GLOBAL int           thePrevRunFlag;
GLOBAL int           theStepFlag;

GLOBAL WindowType    theWindowList;
GLOBAL WindowType    theOpenDialogBoxWindow;
GLOBAL MenuType      theMainMenu;
GLOBAL Cursor        theButtonCursor;
GLOBAL int           theCurrentCursorPosition;

GLOBAL double        theCrosshairmx, theCrosshairbx;
GLOBAL double        theCrosshairmy, theCrosshairby;
GLOBAL int           theCrosshairx1, theCrosshairx2;
GLOBAL int           theCrosshairy1, theCrosshairy2;
GLOBAL WindowType    theCrosshairedWindow;
GLOBAL MenuType      theXYCrosshairWindow;
GLOBAL Cursor        theCrosshairCursor;

GLOBAL Font          theFont;
GLOBAL XFontStruct  *theFontStruct;
GLOBAL int           theFontAscent;
GLOBAL int           theFontDescent;
GLOBAL int           theFontHeight;

GLOBAL Pixmap        theIcon;
GLOBAL int           theIconULX;
GLOBAL int           theIconULY;

GLOBAL char          theCodeName[80];
GLOBAL char          theInputFile[80];
GLOBAL char          theDumpFile[80];
GLOBAL char          thePSFile[80];

GLOBAL char          ReqDisp[120];
GLOBAL Display      *theDisplay;
GLOBAL int           theDisplayWidth;
GLOBAL int           theDisplayHeight;

GLOBAL Window        theRootWindow;
GLOBAL int           theScreen;
GLOBAL int           theDepth;

GLOBAL unsigned long theBlackPixel;
GLOBAL unsigned long theWhitePixel;
GLOBAL Colormap      theColormap;
GLOBAL int           theWinTextColor;
GLOBAL int           theDBTextColor;
GLOBAL int           theBorderColor;
GLOBAL int           theBKGDColor;
GLOBAL int           theGrayShade1;
GLOBAL int           theGrayShade2;
GLOBAL int           theGrayShade3;
GLOBAL int           theGrayShade4;
GLOBAL int           theGrayShade5;
GLOBAL int           theGrayShade6;
GLOBAL int           theGrayShade7;
GLOBAL int           theGrayShade8;
GLOBAL int           theGrayShade9;
GLOBAL int           theGrayShade10;
GLOBAL int           Color[MAXCOLORS];

GLOBAL int           theDialogBoxButtonLabelX;
GLOBAL int           theDialogBoxButtonLabelY;
GLOBAL int           theDialogBoxButtonSpacing;

GLOBAL DialogBoxType theOpenDialogBox;  
GLOBAL DialogBoxType theRescaleDB;  
GLOBAL DialogBoxType thePSDB;
GLOBAL DialogBoxType theDFDB;
GLOBAL DialogBoxType theMessageDB;

GLOBAL XY_DATA       SetupDataStruct();
GLOBAL XY_LABEL      SetupLabelStruct();
GLOBAL int           Paint_Window();
GLOBAL int           Paint_ThreeDWM_Window();
GLOBAL int           PostScriptOpenWindow();
#undef GLOBAL

/***************************************************/
/* Defining macros used in XGRAFIX macros         */

#define Is_Zeroth_Set(x)	(((x)&  1) ==  1)
#define Is_First_Set(x)		(((x)&  2) ==  2)
#define Is_Second_Set(x)	(((x)&  4) ==  4)
#define Is_Third_Set(x)		(((x)&  8) ==  8)
#define Is_Fourth_Set(x)	(((x)& 16) == 16)
#define Is_Fifth_Set(x)		(((x)& 32) == 32)
#define Is_Sixth_Set(x)		(((x)& 64) == 64)
#define Is_Seventh_Set(x)	(((x)&128) ==128)
#define Is_Eighth_Set(x)	(((x)&256) ==256)
#define Is_Ninth_Set(x)         (((x)&512) ==512)

#define Turn_On(x, flag)	((x) |= (flag))
#define Turn_Off(x, flag) 	((x) &= ~(flag))

/*************************************************/
/* XGRAFIX  macros.                              */

#define Is_Open(object)			    Is_Zeroth_Set((object)->state)
#define Was_Open_Before(object)		    Is_First_Set((object)->state)
#define Is_Trace_On(object)		    Is_Second_Set((object)->state)

#define Is_Scatter(object)		   !Is_Third_Set((object)->state) && !Is_Fourth_Set((object)->state)
#define Is_Linear(object)		    Is_Third_Set((object)->state) && !Is_Fourth_Set((object)->state)
#define Is_Semilog(object)		   !Is_Third_Set((object)->state) &&  Is_Fourth_Set((object)->state)
#define Is_ThreeDWM(object)	            Is_Third_Set((object)->state) &&  Is_Fourth_Set((object)->state)

#define Is_Y_AutoRescale(object)            Is_Fifth_Set((object)->state)
#define Is_X_AutoRescale(object)            Is_Sixth_Set((object)->state)
#define Is_Y_AutoRescale_Before(object)     Is_Seventh_Set((object)->state)
#define Is_X_AutoRescale_Before(object)     Is_Eighth_Set((object)->state)
#define Is_Crosshair_On(object)             Is_Ninth_Set((object)->state)

#define Open(object)			    Turn_On((object)->state,  OPEN)
#define Iconify(object)			    Turn_Off((object)->state, OPEN)

#define Open_Before(object)		    Turn_On((object)->state,  OPEN_BEFORE)
#define Iconic_Before(object)		    Turn_Off((object)->state, OPEN_BEFORE)

#define Trace_On(object)		    Turn_On((object)->state,  TRACE)
#define Trace_Off(object)		    Turn_Off((object)->state, TRACE)

#define Set_Linear(object)		    Turn_On((object)->state,  LINEAR)	
#define Set_Semilog(object)		    Turn_On((object)->state,  SEMILOG)	
#define Set_ThreeDWM(object)	            Turn_On((object)->state,  THREED_WM)	

#define Y_AutoRescale(object)               Turn_On((object)->state,  Y_AUTORESCALE)
#define X_AutoRescale(object)               Turn_On((object)->state,  X_AUTORESCALE)
#define Y_AutoRescale_Before(object)        Turn_On((object)->state,  Y_AUTORESCALE_BEFORE)
#define X_AutoRescale_Before(object)        Turn_On((object)->state,  X_AUTORESCALE_BEFORE)

#define Y_AutoRescale_Off(object)           Turn_Off((object)->state, Y_AUTORESCALE)
#define X_AutoRescale_Off(object)           Turn_Off((object)->state, X_AUTORESCALE)
#define Y_AutoRescale_Off_Before(object)    Turn_Off((object)->state, Y_AUTORESCALE_BEFORE)
#define X_AutoRescale_Off_Before(object)    Turn_Off((object)->state, X_AUTORESCALE_BEFORE)

#define Crosshair_On(object)                Turn_On((object)->state,  CROSSHAIR)
#define Crosshair_Off(object)               Turn_Off((object)->state, CROSSHAIR)

#define max(x, y)                           (((x) > (y)) ? (x) : (y))
#define min(x, y)                           (((x) < (y)) ? (x) : (y))


